@@ -1,11 +1,11 @@
 /*****************************************************************************\
  *  job_info.c - get/print the job state information of slurm
- *  $Id: job_info.c 12627 2007-11-06 19:48:55Z jette $
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov> et. al.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -126,8 +126,8 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	int i, j;
 	char time_str[32], select_buf[122];
 	struct group *group_info = NULL;
-	char tmp1[128], tmp2[128];
-	char tmp_line[128];
+	char tmp1[128], tmp2[128], *tmp3_ptr;
+	char tmp_line[512];
 	char *ionodes = NULL;
 	uint16_t exit_status = 0, term_sig = 0;
 	char *out = NULL;
@@ -189,8 +189,11 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		sprintf(tmp_line, "UNLIMITED ");
 	else if (job_ptr->time_limit == NO_VAL)
 		sprintf(tmp_line, "Partition_Limit ");
-	else
-		sprintf(tmp_line, "%u ", job_ptr->time_limit);
+	else {
+		secs2time_str(job_ptr->time_limit * 60, tmp1,
+			sizeof(tmp1));
+		sprintf(tmp_line, "%s ", tmp1);
+	}
 	xstrcat(out, tmp_line);
 	if (WIFSIGNALED(job_ptr->exit_code))
 		term_sig = WTERMSIG(job_ptr->exit_code);
@@ -206,11 +209,16 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		xstrcat(out, "\n   ");
 
 	/****** Line 5 ******/
+	if (job_ptr->job_state == JOB_PENDING)
+		tmp3_ptr = "EligibleTime";
+	else
+		tmp3_ptr = "StartTime";
 	slurm_make_time_str((time_t *)&job_ptr->start_time, time_str,
 		sizeof(time_str));
 	snprintf(tmp_line, sizeof(tmp_line),
-		"JobState=%s StartTime=%s EndTime=",
-		job_state_string(job_ptr->job_state), time_str);
+		"JobState=%s %s=%s EndTime=",
+		job_state_string(job_ptr->job_state), 
+		tmp3_ptr, time_str);
 	xstrcat(out, tmp_line);
 	if ((job_ptr->time_limit == INFINITE) && 
 	    (job_ptr->end_time > time(NULL)))
@@ -230,12 +238,12 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	xstrfmtcat(out, "%s=", nodelist);
 	xstrcat(out, job_ptr->nodes);
 	if(ionodes) {
-		sprintf(tmp_line, "[%s]", ionodes);
+		snprintf(tmp_line, sizeof(tmp_line), "[%s]", ionodes);
 		xstrcat(out, tmp_line);
 		xfree(ionodes);
 	} 
 	
-	sprintf(tmp_line, " %sIndices=", nodelist);
+	snprintf(tmp_line, sizeof(tmp_line), " %sIndices=", nodelist);
 	xstrcat(out, tmp_line);
 	for (j = 0;  (job_ptr->node_inx && (job_ptr->node_inx[j] != -1)); 
 			j+=2) {
@@ -307,10 +315,12 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 #ifdef HAVE_BG
 	convert_num_unit((float)job_ptr->num_nodes, tmp2, sizeof(tmp2),
 			 UNIT_NONE);
-	sprintf(tmp_line, "ReqProcs=%s MinBPs=%s ", tmp1, tmp2);
+	snprintf(tmp_line, sizeof(tmp_line), "ReqProcs=%s MinBPs=%s ", 
+		 tmp1, tmp2);
 #else
 	_sprint_range(tmp2, job_ptr->num_nodes, job_ptr->max_nodes);
-	sprintf(tmp_line, "ReqProcs=%s ReqNodes=%s ", tmp1, tmp2);
+	snprintf(tmp_line, sizeof(tmp_line), "ReqProcs=%s ReqNodes=%s ", 
+		 tmp1, tmp2);
 #endif
 	xstrcat(out, tmp_line);
 
@@ -337,10 +347,11 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 
 	/****** Line 8 ******/
 	snprintf(tmp_line, sizeof(tmp_line),
-		"Shared=%s Contiguous=%d CPUs/task=%u", 
+		"Shared=%s Contiguous=%d CPUs/task=%u Licenses=%s", 
 		 (job_ptr->shared == 0 ? "0" :
 		  job_ptr->shared == 1 ? "1" : "OK"),
-		 job_ptr->contiguous, job_ptr->cpus_per_task);
+		 job_ptr->contiguous, job_ptr->cpus_per_task,
+		 job_ptr->licenses);
 	xstrcat(out, tmp_line);
 
 	if (one_liner)
@@ -360,13 +371,18 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		xstrcat(out, "\n   ");
 
 	/****** Line 10 ******/
+	if (job_ptr->job_min_memory & MEM_PER_CPU) {
+		job_ptr->job_min_memory &= (~MEM_PER_CPU);
+		tmp3_ptr = "CPU";
+	} else
+		tmp3_ptr = "Node";
 	convert_num_unit((float)job_ptr->job_min_memory, tmp1, sizeof(tmp1),
 			 UNIT_NONE);
 	convert_num_unit((float)job_ptr->job_min_tmp_disk, tmp2, sizeof(tmp2),
 			 UNIT_NONE);
 	snprintf(tmp_line, sizeof(tmp_line), 
-		"MinMemory=%s MinTmpDisk=%s Features=%s",
-		tmp1, tmp2, job_ptr->features);
+		"MinMemory%s=%s MinTmpDisk=%s Features=%s",
+		tmp3_ptr, tmp1, tmp2, job_ptr->features);
 	xstrcat(out, tmp_line);
 	if (one_liner)
 		xstrcat(out, " ");
@@ -375,8 +391,17 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 
 	/****** Line 11 ******/
 	snprintf(tmp_line, sizeof(tmp_line), 
-		"Dependency=%u Account=%s Reason=%s Network=%s",
-		job_ptr->dependency, job_ptr->account,
+		"Dependency=%s Account=%s Requeue=%u",
+		job_ptr->dependency, job_ptr->account, job_ptr->requeue);
+	xstrcat(out, tmp_line);
+	if (one_liner)
+		xstrcat(out, " ");
+	else
+		xstrcat(out, "\n   ");
+
+	/****** Line 12 ******/
+	snprintf(tmp_line, sizeof(tmp_line), 
+		"Reason=%s Network=%s",
 		job_reason_string(job_ptr->state_reason), 
 		job_ptr->network);
 	xstrcat(out, tmp_line);
@@ -385,7 +410,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	else
 		xstrcat(out, "\n   ");
 
-	/****** Line 12 ******/
+	/****** Line 13 ******/
 	snprintf(tmp_line, sizeof(tmp_line), "Req%s=%s Req%sIndices=", 
 		nodelist, job_ptr->req_nodes, nodelist);
 	xstrcat(out, tmp_line);
@@ -402,7 +427,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	else
 		xstrcat(out, "\n   ");
 
-	/****** Line 13 ******/
+	/****** Line 14 ******/
 	snprintf(tmp_line, sizeof(tmp_line), "Exc%s=%s Exc%sIndices=", 
 		nodelist, job_ptr->exc_nodes, nodelist);
 	xstrcat(out, tmp_line);
@@ -419,10 +444,11 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	else
 		xstrcat(out, "\n   ");
 
-	/****** Line 14 ******/
+	/****** Line 15 ******/
 	slurm_make_time_str((time_t *)&job_ptr->submit_time, time_str, 
 		sizeof(time_str));
-	sprintf(tmp_line, "SubmitTime=%s ", time_str);
+	snprintf(tmp_line, sizeof(tmp_line), "SubmitTime=%s ", 
+		 time_str);
 	xstrcat(out, tmp_line);
 	if (job_ptr->suspend_time) {
 		slurm_make_time_str ((time_t *)&job_ptr->suspend_time, 
@@ -434,17 +460,35 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 		  time_str, (long int)job_ptr->pre_sus_time);
 	xstrcat(out, tmp_line);
 
-	/****** Line 15 (optional) ******/
+	/****** Lines 16, 17 (optional, batch only) ******/
+	if (job_ptr->batch_flag) {
+		if (one_liner)
+			xstrcat(out, " ");
+		else
+			xstrcat(out, "\n   ");
+		sprintf(tmp_line, "Command=%s", job_ptr->command);
+		xstrcat(out, tmp_line);
+
+		if (one_liner)
+			xstrcat(out, " ");
+		else
+			xstrcat(out, "\n   ");
+		sprintf(tmp_line, "WorkDir=%s", job_ptr->work_dir);
+		xstrcat(out, tmp_line);
+	}
+
+	/****** Line 18 (optional) ******/
 	if (job_ptr->comment) {
 		if (one_liner)
 			xstrcat(out, " ");
 		else
 			xstrcat(out, "\n   ");
-		sprintf(tmp_line, "Comment=%s ", job_ptr->comment);
+		snprintf(tmp_line, sizeof(tmp_line), "Comment=%s ", 
+			 job_ptr->comment);
 		xstrcat(out, tmp_line);
 	}
 
-	/****** Line 16 (optional) ******/
+	/****** Line 19 (optional) ******/
 	select_g_sprint_jobinfo(job_ptr->select_jobinfo,
 				select_buf, sizeof(select_buf),
 				SELECT_PRINT_MIXED);
@@ -455,7 +499,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 			xstrcat(out, "\n   ");
 		xstrcat(out, select_buf);
 	}
-	/****** Line 17 (optional) ******/
+	/****** Line 20 (optional) ******/
 	select_g_sprint_jobinfo(job_ptr->select_jobinfo,
 				select_buf, sizeof(select_buf),
 				SELECT_PRINT_BLRTS_IMAGE);
@@ -468,7 +512,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 			 "BlrtsImage=%s", select_buf);
 		xstrcat(out, tmp_line);
 	}
-	/****** Line 18 (optional) ******/
+	/****** Line 21 (optional) ******/
 	select_g_sprint_jobinfo(job_ptr->select_jobinfo,
 				select_buf, sizeof(select_buf),
 				SELECT_PRINT_LINUX_IMAGE);
@@ -481,7 +525,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 			 "LinuxImage=%s", select_buf);
 		xstrcat(out, tmp_line);
 	}
-	/****** Line 19 (optional) ******/
+	/****** Line 22 (optional) ******/
 	select_g_sprint_jobinfo(job_ptr->select_jobinfo,
 				select_buf, sizeof(select_buf),
 				SELECT_PRINT_MLOADER_IMAGE);
@@ -494,7 +538,7 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 			 "MloaderImage=%s", select_buf);
 		xstrcat(out, tmp_line);
 	}
-	/****** Line 20 (optional) ******/
+	/****** Line 23 (optional) ******/
 	select_g_sprint_jobinfo(job_ptr->select_jobinfo,
 				select_buf, sizeof(select_buf),
 				SELECT_PRINT_RAMDISK_IMAGE);
@@ -512,8 +556,9 @@ slurm_sprint_job_info ( job_info_t * job_ptr, int one_liner )
 	return out;
 
 }
+
 /*
- * slurm_load_jobs - issue RPC to get slurm all job configuration  
+ * slurm_load_jobs - issue RPC to get all job configuration  
  *	information if changed since update_time 
  * IN update_time - time of current configuration data
  * IN job_info_msg_pptr - place to store a job configuration pointer
@@ -559,6 +604,48 @@ slurm_load_jobs (time_t update_time, job_info_msg_t **resp,
 	return SLURM_PROTOCOL_SUCCESS ;
 }
 
+/*
+ * slurm_load_job - issue RPC to get job information for one job ID
+ * IN job_info_msg_pptr - place to store a job configuration pointer
+ * IN job_id -  ID of job we want information about 
+ * RET 0 or -1 on error
+ * NOTE: free the response using slurm_free_job_info_msg
+ */
+extern int
+slurm_load_job (job_info_msg_t **resp, uint32_t job_id)
+{
+	int rc;
+	slurm_msg_t resp_msg;
+	slurm_msg_t req_msg;
+	job_id_msg_t req;
+
+	slurm_msg_t_init(&req_msg);
+	slurm_msg_t_init(&resp_msg);
+
+	req.job_id = job_id;
+	req_msg.msg_type = REQUEST_JOB_INFO_SINGLE;
+	req_msg.data     = &req;
+
+	if (slurm_send_recv_controller_msg(&req_msg, &resp_msg) < 0)
+		return SLURM_ERROR;
+
+	switch (resp_msg.msg_type) {
+	case RESPONSE_JOB_INFO:
+		*resp = (job_info_msg_t *)resp_msg.data;
+		break;
+	case RESPONSE_SLURM_RC:
+		rc = ((return_code_msg_t *) resp_msg.data)->return_code;
+		slurm_free_return_code_msg(resp_msg.data);	
+		if (rc) 
+			slurm_seterrno_ret(rc);
+		break;
+	default:
+		slurm_seterrno_ret(SLURM_UNEXPECTED_MSG_ERROR);
+		break;
+	}
+
+	return SLURM_PROTOCOL_SUCCESS ;
+}
 
 /*
  * slurm_pid2jobid - issue RPC to get the slurm job_id given a process_id 
