@@ -1,12 +1,11 @@
 /*****************************************************************************\
  *  step_launch.c - launch a parallel job step
- *
- *  $Id: step_launch.c 10920 2007-02-02 03:01:14Z morrone $
  *****************************************************************************
- *  Copyright (C) 2006 The Regents of the University of California.
+ *  Copyright (C) 2006-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Christopher J. Morrone <morrone2@llnl.gov>
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -31,6 +30,7 @@
 #endif
 
 #include <errno.h>
+#include <fcntl.h>
 #include <pthread.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -40,6 +40,7 @@
 #include <netinet/in.h>
 #include <sys/param.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/un.h>
 #include <netdb.h> /* for gethostbyname */
@@ -69,18 +70,25 @@ extern char **environ;
 /**********************************************************************
  * General declarations for step launch code
  **********************************************************************/
-static int _launch_tasks(slurm_step_ctx ctx,
-			 launch_tasks_request_msg_t *launch_msg);
+static int _launch_tasks(slurm_step_ctx_t *ctx,
+			 launch_tasks_request_msg_t *launch_msg,
+			 uint32_t timeout);
 static char *_lookup_cwd(void);
+static void _print_launch_msg(launch_tasks_request_msg_t *msg,
+			      char *hostname, int nodeid);
 
 /**********************************************************************
  * Message handler declarations
  **********************************************************************/
+static pid_t  srun_ppid = (pid_t) 0;
 static uid_t  slurm_uid;
-static int _msg_thr_create(struct step_launch_state *sls, int num_nodes);
+static bool   force_terminated_job = false;
+static int    task_exit_signal = 0;
+static void _exec_prog(slurm_msg_t *msg);
+static int  _msg_thr_create(struct step_launch_state *sls, int num_nodes);
 static void _handle_msg(struct step_launch_state *sls, slurm_msg_t *msg);
 static bool _message_socket_readable(eio_obj_t *obj);
-static int _message_socket_accept(eio_obj_t *obj, List objs);
+static int  _message_socket_accept(eio_obj_t *obj, List objs);
 
 static struct io_operations message_socket_ops = {
 	readable:	&_message_socket_readable,
@@ -109,15 +117,17 @@ void slurm_step_launch_params_t_init (slurm_step_launch_params_t *ptr)
 	ptr->buffered_stdio = true;
 	memcpy(&ptr->local_fds, &fds, sizeof(fds));
 	ptr->gid = getgid();
+	ptr->acctg_freq = (uint16_t) NO_VAL;
 }
 
 /*
  * slurm_step_launch - launch a parallel job step
  * IN ctx - job step context generated by slurm_step_ctx_create
+ * IN launcher_host - address used for PMI communications
  * IN callbacks - Identify functions to be called when various events occur
  * RET SLURM_SUCCESS or SLURM_ERROR (with errno set)
  */
-int slurm_step_launch (slurm_step_ctx ctx,
+int slurm_step_launch (slurm_step_ctx_t *ctx, char *launcher_host,
 		       const slurm_step_launch_params_t *params,
 		       const slurm_step_launch_callbacks_t *callbacks)
 {
@@ -131,8 +141,7 @@ int slurm_step_launch (slurm_step_ctx ctx,
 	memset(&launch, 0, sizeof(launch));
 
 	if (ctx == NULL || ctx->magic != STEP_CTX_MAGIC) {
-		error("Not a valid slurm_step_ctx!");
-
+		error("Not a valid slurm_step_ctx_t!");
 		slurm_seterrno(EINVAL);
 		return SLURM_ERROR;
 	}
@@ -166,7 +175,8 @@ int slurm_step_launch (slurm_step_ctx ctx,
 	}
 
 	/* Create message receiving sockets and handler thread */
-	_msg_thr_create(ctx->launch_state, ctx->step_req->node_count);
+	_msg_thr_create(ctx->launch_state,
+			ctx->step_resp->step_layout->node_cnt);
 
 	/* Start tasks on compute nodes */
 	launch.job_id = ctx->step_req->job_id;
@@ -178,23 +188,13 @@ int slurm_step_launch (slurm_step_ctx ctx,
 	launch.job_step_id = ctx->step_resp->job_step_id;
 	if (params->env == NULL) {
 		/* if the user didn't specify an environment, grab the
-		   environment of the running process */
+		 * environment of the running process */
 		env_array_merge(&env, (const char **)environ);
 	} else {
 		env_array_merge(&env, (const char **)params->env);
 	}
-	{
-		/* FIXME - hostname and IP need to be user settable */
-		char *launcher_hostname = xshort_hostname();
-		struct hostent *ent = gethostbyname(launcher_hostname);
-
-		env_array_for_step(&env,
-				   ctx->step_resp,
-				   launcher_hostname,
-				   ctx->launch_state->resp_port[0],
-				   ent->h_addr_list[0]);
-		xfree(launcher_hostname);
-	}
+	env_array_for_step(&env, ctx->step_resp, launcher_host,
+			   ctx->launch_state->resp_port[0]);
 	env_array_merge(&env, (const char **)mpi_env);
 	env_array_free(mpi_env);
 
@@ -205,24 +205,31 @@ int slurm_step_launch (slurm_step_ctx ctx,
 	} else {
 		launch.cwd = _lookup_cwd();
 	}
-	launch.nnodes = ctx->step_req->node_count;
-	launch.nprocs = ctx->step_req->num_tasks;
-	launch.slurmd_debug = params->slurmd_debug;
-	launch.switch_job = ctx->step_resp->switch_job;
-	launch.task_prolog = params->task_prolog;
-	launch.task_epilog = params->task_epilog;
-	launch.cpu_bind_type = params->cpu_bind_type;
-	launch.cpu_bind = params->cpu_bind;
-	launch.mem_bind_type = params->mem_bind_type;
-	launch.mem_bind = params->mem_bind;
-	launch.multi_prog = params->multi_prog ? 1 : 0;
+	launch.nnodes		= ctx->step_resp->step_layout->node_cnt;
+	launch.nprocs		= ctx->step_resp->step_layout->task_cnt;
+	launch.slurmd_debug	= params->slurmd_debug;
+	launch.switch_job	= ctx->step_resp->switch_job;
+	launch.task_prolog	= params->task_prolog;
+	launch.task_epilog	= params->task_epilog;
+	launch.cpu_bind_type	= params->cpu_bind_type;
+	launch.cpu_bind		= params->cpu_bind;
+	launch.mem_bind_type	= params->mem_bind_type;
+	launch.mem_bind		= params->mem_bind;
+	launch.multi_prog	= params->multi_prog ? 1 : 0;
+	launch.max_sockets	= params->max_sockets;
+	launch.max_cores	= params->max_cores;
+	launch.max_threads	= params->max_threads;
 	launch.cpus_per_task	= params->cpus_per_task;
 	launch.ntasks_per_node	= params->ntasks_per_node;
 	launch.ntasks_per_socket= params->ntasks_per_socket;
 	launch.ntasks_per_core	= params->ntasks_per_core;
 	launch.task_dist	= params->task_dist;
 	launch.plane_size	= params->plane_size;
-	launch.options = job_options_create();
+	launch.pty              = params->pty;
+	launch.ckpt_path        = params->ckpt_path;
+	launch.acctg_freq	= params->acctg_freq;
+	launch.open_mode        = params->open_mode;
+	launch.options          = job_options_create();
 	launch.complete_nodelist = 
 		xstrdup(ctx->step_resp->step_layout->node_list);
 	spank_set_remote_options (launch.options);
@@ -236,6 +243,7 @@ int slurm_step_launch (slurm_step_ctx ctx,
 	
 	launch.user_managed_io = params->user_managed_io ? 1 : 0;
 	ctx->launch_state->user_managed_io = params->user_managed_io;
+	
 	if (!ctx->launch_state->user_managed_io) {
 		launch.ofname = params->remote_output_filename;
 		launch.efname = params->remote_error_filename;
@@ -277,7 +285,7 @@ int slurm_step_launch (slurm_step_ctx ctx,
 		launch.resp_port[i] = ctx->launch_state->resp_port[i];
 	}
 
-	_launch_tasks(ctx, &launch);
+	rc = _launch_tasks(ctx, &launch, params->msg_timeout);
 
 	/* clean up */
 	xfree(launch.resp_port);
@@ -298,7 +306,7 @@ done:
 /*
  * Block until all tasks have started.
  */
-int slurm_step_launch_wait_start(slurm_step_ctx ctx)
+int slurm_step_launch_wait_start(slurm_step_ctx_t *ctx)
 {
 	struct step_launch_state *sls = ctx->launch_state;
 	/* Wait for all tasks to start */
@@ -341,7 +349,7 @@ int slurm_step_launch_wait_start(slurm_step_ctx ctx)
 /*
  * Block until all tasks have finished (or failed to start altogether).
  */
-void slurm_step_launch_wait_finish(slurm_step_ctx ctx)
+void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 {
 	struct step_launch_state *sls = ctx->launch_state;
 	struct timespec ts = {0, 0};
@@ -400,10 +408,16 @@ void slurm_step_launch_wait_finish(slurm_step_ctx ctx)
 			}
 		}
 	}
-	
+
+	if (!force_terminated_job && task_exit_signal)
+		info("Force Terminated job step %u.%u",
+		     ctx->job_id, ctx->step_resp->job_step_id);
+
 	/* Then shutdown the message handler thread */
 	eio_signal_shutdown(sls->msg_handle);
+	pthread_mutex_unlock(&sls->lock);
 	pthread_join(sls->msg_thread, NULL);
+	pthread_mutex_lock(&sls->lock);
 	eio_handle_destroy(sls->msg_handle);
 
 	/* Then wait for the IO thread to finish */
@@ -413,7 +427,6 @@ void slurm_step_launch_wait_finish(slurm_step_ctx ctx)
 	}
 
 	mpi_hook_client_fini(sls->mpi_state);
-
 	pthread_mutex_unlock(&sls->lock);
 }
 
@@ -422,7 +435,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx ctx)
  *
  * Can be called from a signal handler.
  */
-void slurm_step_launch_abort(slurm_step_ctx ctx)
+void slurm_step_launch_abort(slurm_step_ctx_t *ctx)
 {
 	struct step_launch_state *sls = ctx->launch_state;
 
@@ -430,6 +443,100 @@ void slurm_step_launch_abort(slurm_step_ctx ctx)
 	pthread_cond_signal(&sls->cond);
 }
 
+/* 
+ * Forward a signal to all those nodes with running tasks 
+ */
+void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
+{
+	int node_id, j, active, num_tasks;
+	slurm_msg_t req;
+	kill_tasks_msg_t msg;
+	hostlist_t hl;
+	char *name = NULL;
+	char buf[8192];
+	List ret_list = NULL;
+	ListIterator itr;
+	ret_data_info_t *ret_data_info = NULL;
+	int rc = SLURM_SUCCESS;
+	struct step_launch_state *sls = ctx->launch_state;
+	
+	debug2("forward signal %d to job", signo);
+	
+	/* common to all tasks */
+	msg.job_id      = ctx->job_id;
+	msg.job_step_id = ctx->step_resp->job_step_id;
+	msg.signal      = (uint32_t) signo;
+	
+	pthread_mutex_lock(&sls->lock);
+	
+	hl = hostlist_create("");
+	for (node_id = 0;
+	     node_id < ctx->step_resp->step_layout->node_cnt;
+	     node_id++) {
+		active = 0;		
+		num_tasks = sls->layout->tasks[node_id];
+		for (j = 0; j < num_tasks; j++) {
+			if(bit_test(sls->tasks_started,
+				    sls->layout->tids[node_id][j]) &&
+			   !bit_test(sls->tasks_exited,
+				     sls->layout->tids[node_id][j])) {
+				/* this one has active tasks */
+				active = 1;
+				break;
+			}
+		}
+		
+		if (!active)
+			continue;
+		
+		name = nodelist_nth_host(sls->layout->node_list, node_id);
+		hostlist_push(hl, name);
+		free(name);
+	}
+
+	pthread_mutex_unlock(&sls->lock);
+	
+	if(!hostlist_count(hl)) {
+		hostlist_destroy(hl);
+		goto nothing_left;
+	}
+	hostlist_ranged_string(hl, sizeof(buf), buf);
+	hostlist_destroy(hl);
+	name = xstrdup(buf);
+	
+	slurm_msg_t_init(&req);	
+	req.msg_type = REQUEST_SIGNAL_TASKS;
+	req.data     = &msg;
+	
+	debug3("sending signal to host %s", name);
+	
+	if (!(ret_list = slurm_send_recv_msgs(name, &req, 0))) { 
+		error("fwd_signal: slurm_send_recv_msgs really failed bad");
+		xfree(name);
+		return;
+	}
+	xfree(name);
+	itr = list_iterator_create(ret_list);		
+	while((ret_data_info = list_next(itr))) {
+		rc = slurm_get_return_code(ret_data_info->type, 
+					   ret_data_info->data);
+		/*
+		 *  Report error unless it is "Invalid job id" which 
+		 *    probably just means the tasks exited in the meanwhile.
+		 */
+		if ((rc != 0) && (rc != ESLURM_INVALID_JOB_ID)
+		    &&  (rc != ESLURMD_JOB_NOTRUNNING) && (rc != ESRCH)) {
+			error("%s: signal: %s", 
+			      ret_data_info->node_name, 
+			      slurm_strerror(rc));
+		}
+	}
+	list_iterator_destroy(itr);
+	list_destroy(ret_list);
+nothing_left:
+	debug2("All tasks have been signalled");
+	
+}
 
 /**********************************************************************
  * Functions used by step_ctx code, but not exported throught the API
@@ -437,27 +544,30 @@ void slurm_step_launch_abort(slurm_step_ctx ctx)
 /*
  * Create a launch state structure for a specified step context, "ctx".
  */
-struct step_launch_state *step_launch_state_create(slurm_step_ctx ctx)
+struct step_launch_state *step_launch_state_create(slurm_step_ctx_t *ctx)
 {
 	struct step_launch_state *sls;
+	slurm_step_layout_t *layout = ctx->step_resp->step_layout;
 
 	sls = xmalloc(sizeof(struct step_launch_state));
-	if (sls != NULL) {
-		sls->slurmctld_socket_fd = -1;
-		sls->tasks_requested = ctx->step_req->num_tasks;
-		sls->tasks_started = bit_alloc(ctx->step_req->num_tasks);
-		sls->tasks_exited = bit_alloc(ctx->step_req->num_tasks);
-		sls->layout = ctx->step_resp->step_layout;
-		sls->resp_port = NULL;
-		sls->abort = false;
-		sls->abort_action_taken = false;
-		sls->mpi_info->jobid = ctx->step_req->job_id;
-		sls->mpi_info->stepid = ctx->step_resp->job_step_id;
-		sls->mpi_info->step_layout = ctx->step_resp->step_layout;
-		sls->mpi_state = NULL;
-		pthread_mutex_init(&sls->lock, NULL);
-		pthread_cond_init(&sls->cond, NULL);
-	}
+	sls->slurmctld_socket_fd = -1;
+	/* Hack for LAM-MPI's lamboot, launch one task per node */
+	if (mpi_hook_client_single_task_per_node())
+		sls->tasks_requested = layout->node_cnt;
+	else
+		sls->tasks_requested = layout->task_cnt;
+	sls->tasks_started = bit_alloc(layout->task_cnt);
+	sls->tasks_exited = bit_alloc(layout->task_cnt);
+	sls->layout = layout;
+	sls->resp_port = NULL;
+	sls->abort = false;
+	sls->abort_action_taken = false;
+	sls->mpi_info->jobid = ctx->step_req->job_id;
+	sls->mpi_info->stepid = ctx->step_resp->job_step_id;
+	sls->mpi_info->step_layout = layout;
+	sls->mpi_state = NULL;
+	pthread_mutex_init(&sls->lock, NULL);
+	pthread_cond_init(&sls->cond, NULL);
 	return sls;
 }
 
@@ -478,7 +588,6 @@ void step_launch_state_destroy(struct step_launch_state *sls)
 	}
 }
 
-
 /**********************************************************************
  * Message handler functions
  **********************************************************************/
@@ -504,7 +613,8 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 	int sock = -1;
 	short port = -1;
 	eio_obj_t *obj;
-	int i;
+	int i, rc = SLURM_SUCCESS;
+	pthread_attr_t attr;
 
 	debug("Entering _msg_thr_create()");
 	slurm_uid = (uid_t) slurm_get_slurm_user_id();
@@ -529,12 +639,15 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 		eio_new_initial_obj(sls->msg_handle, obj);
 	}
 
-	if (pthread_create(&sls->msg_thread, NULL,
+	slurm_attr_init(&attr);
+	if (pthread_create(&sls->msg_thread, &attr,
 			   _msg_thr_internal, (void *)sls) != 0) {
 		error("pthread_create of message thread: %m");
-		return SLURM_ERROR;
+		
+		rc = SLURM_ERROR;
 	}
-	return SLURM_SUCCESS;
+	slurm_attr_destroy(&attr);
+	return rc;
 }
 
 static bool _message_socket_readable(eio_obj_t *obj)
@@ -590,7 +703,7 @@ static int _message_socket_accept(eio_obj_t *obj, List objs)
 	   in /etc/hosts. */
 	uc = (unsigned char *)&((struct sockaddr_in *)&addr)->sin_addr.s_addr;
 	port = ((struct sockaddr_in *)&addr)->sin_port;
-	debug2("got message connection from %u.%u.%u.%u:%hu",
+	debug2("step got message connection from %u.%u.%u.%u:%hu",
 	       uc[0], uc[1], uc[2], uc[3], ntohs(port));
 	fflush(stdout);
 
@@ -628,10 +741,18 @@ _launch_handler(struct step_launch_state *sls, slurm_msg_t *resp)
 
 	pthread_mutex_lock(&sls->lock);
 
-	for (i = 0; i < msg->count_of_pids; i++) {
-		bit_set(sls->tasks_started, msg->task_ids[i]);
+	if (msg->return_code) {
+		for (i = 0; i < msg->count_of_pids; i++) {
+			error("task %u launch failed: %s", 
+			      msg->task_ids[i], 
+			      slurm_strerror(msg->return_code));
+			bit_set(sls->tasks_started, msg->task_ids[i]);
+			bit_set(sls->tasks_exited, msg->task_ids[i]);
+		}
+	} else {
+		for (i = 0; i < msg->count_of_pids; i++)
+			bit_set(sls->tasks_started, msg->task_ids[i]);
 	}
-
 	if (sls->callback.task_start != NULL)
 		(sls->callback.task_start)(msg);
 
@@ -646,10 +767,18 @@ _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 	task_exit_msg_t *msg = (task_exit_msg_t *) exit_msg->data;
 	int i;
 
+	/* Record SIGTERM and SIGKILL termination codes to 
+	 * recognize abnormal termination */
+	if (WIFSIGNALED(msg->return_code)) {
+		i = WTERMSIG(msg->return_code);
+		if ((i == SIGKILL) || (i == SIGTERM))
+			task_exit_signal = i;
+	}
+
 	pthread_mutex_lock(&sls->lock);
 
 	for (i = 0; i < msg->num_tasks; i++) {
-		debug("task %d done", msg->task_id_list[i]);
+		debug("task %u done", msg->task_id_list[i]);
 		bit_set(sls->tasks_exited, msg->task_id_list[i]);
 	}
 
@@ -660,6 +789,36 @@ _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 	pthread_mutex_unlock(&sls->lock);
 }
 
+static void 
+_job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
+{
+	srun_job_complete_msg_t *step_msg = 
+		(srun_job_complete_msg_t *) complete_msg->data;
+	
+	if (step_msg->step_id == NO_VAL) {
+		verbose("Complete job %u received",
+			step_msg->job_id);
+	} else {
+		verbose("Complete job step %u.%u received",
+			step_msg->job_id, step_msg->step_id);
+	}
+
+	/* FIXME: does nothing yet */
+
+	pthread_mutex_lock(&sls->lock);
+	pthread_cond_signal(&sls->cond);
+	pthread_mutex_unlock(&sls->lock);
+}
+
+static void 
+_timeout_handler(struct step_launch_state *sls, slurm_msg_t *timeout_msg)
+{
+	/* FIXME: does nothing yet */
+	pthread_mutex_lock(&sls->lock);
+	pthread_cond_signal(&sls->cond);
+	pthread_mutex_unlock(&sls->lock);
+}
+
 /*
  * Take the list of node names of down nodes and convert into an
  * array of nodeids for the step.  The nodeid array is passed to
@@ -678,6 +837,7 @@ _node_fail_handler(struct step_launch_state *sls, slurm_msg_t *fail_msg)
 	int i, j;
 	int node_id, num_tasks;
 
+	error("Node failure on %s", nf->nodelist);
 	fail_nodes = hostset_create(nf->nodelist);
 	fail_itr = hostset_iterator_create(fail_nodes);
 	num_node_ids = hostset_count(fail_nodes);
@@ -765,8 +925,9 @@ _task_user_managed_io_handler(struct step_launch_state *sls,
 static void
 _handle_msg(struct step_launch_state *sls, slurm_msg_t *msg)
 {
-	uid_t req_uid = g_slurm_auth_get_uid(msg->auth_cred);
+	uid_t req_uid = g_slurm_auth_get_uid(msg->auth_cred, NULL);
 	uid_t uid = getuid();
+	srun_user_msg_t *um;
 	int rc;
 	
 	if ((req_uid != slurm_uid) && (req_uid != 0) && (req_uid != uid)) {
@@ -786,21 +947,36 @@ _handle_msg(struct step_launch_state *sls, slurm_msg_t *msg)
 		_exit_handler(sls, msg);
 		slurm_free_task_exit_msg(msg->data);
 		break;
+	case SRUN_PING:
+		debug3("slurmctld ping received");
+		slurm_send_rc_msg(msg, SLURM_SUCCESS);
+		slurm_free_srun_ping_msg(msg->data);
+		break;
+	case SRUN_EXEC:
+		_exec_prog(msg);
+		slurm_free_srun_exec_msg(msg->data);
+		break;
+	case SRUN_JOB_COMPLETE:
+		debug2("received job step complete message");
+		force_terminated_job = true;
+		_job_complete_handler(sls, msg);
+		slurm_free_srun_job_complete_msg(msg->data);
+		break;
+	case SRUN_TIMEOUT:
+		debug2("received job step timeout message");
+		_timeout_handler(sls, msg);
+		slurm_free_srun_timeout_msg(msg->data);
+		break;
+	case SRUN_USER_MSG:
+		um = msg->data;
+		info("%s", um->msg);
+		slurm_free_srun_user_msg(msg->data);
+		break;
 	case SRUN_NODE_FAIL:
 		debug2("received srun node fail");
 		_node_fail_handler(sls, msg);
 		slurm_free_srun_node_fail_msg(msg->data);
 		break;
-	case SRUN_TIMEOUT:
-		debug2("received job step timeout message");
-		/* FIXME - does nothing yet */
-		slurm_free_srun_timeout_msg(msg->data);
-		break;
-	case SRUN_JOB_COMPLETE:
-		debug2("received job step complete message");
-		/* FIXME - does nothing yet */
-		slurm_free_srun_job_complete_msg(msg->data);
-		break;
 	case PMI_KVS_PUT_REQ:
 		debug2("PMI_KVS_PUT_REQ received");
 		rc = pmi_kvs_put((struct kvs_comm_set *) msg->data);
@@ -827,23 +1003,36 @@ _handle_msg(struct step_launch_state *sls, slurm_msg_t *msg)
 /**********************************************************************
  * Task launch functions
  **********************************************************************/
-static int _launch_tasks(slurm_step_ctx ctx,
-			 launch_tasks_request_msg_t *launch_msg)
+static int _launch_tasks(slurm_step_ctx_t *ctx,
+			 launch_tasks_request_msg_t *launch_msg,
+			 uint32_t timeout)
 {
 	slurm_msg_t msg;
 	List ret_list = NULL;
 	ListIterator ret_itr;
 	ret_data_info_t *ret_data = NULL;
 	int rc = SLURM_SUCCESS;
+	int tot_rc = SLURM_SUCCESS;
 
 	debug("Entering _launch_tasks");
+	if (ctx->verbose_level) {
+		char *name = NULL;
+		hostlist_t hl = hostlist_create(launch_msg->complete_nodelist);
+		int i = 0;
+		while((name = hostlist_shift(hl))) {
+			_print_launch_msg(launch_msg, name, i++);
+			free(name);			
+		}
+		hostlist_destroy(hl);
+	}
+
 	slurm_msg_t_init(&msg);
 	msg.msg_type = REQUEST_LAUNCH_TASKS;
 	msg.data = launch_msg;
 	
 	if(!(ret_list = slurm_send_recv_msgs(
 		     ctx->step_resp->step_layout->node_list,
-		     &msg, 0))) {
+		     &msg, timeout))) {
 		error("slurm_send_recv_msgs failed miserably: %m");
 		return SLURM_ERROR;
 	}
@@ -854,9 +1043,14 @@ static int _launch_tasks(slurm_step_ctx ctx,
 		debug("launch returned msg_rc=%d err=%d type=%d",
 		      rc, ret_data->err, ret_data->type);
 		if (rc != SLURM_SUCCESS) {
-			errno = ret_data->err;
+			if (ret_data->err)
+				errno = ret_data->err;
+			else
+				errno = rc;
 			error("Task launch failed on node %s: %m",
 			      ret_data->node_name);
+			rc = SLURM_ERROR;
+			tot_rc = rc;
 		} else {
 #if 0 /* only for debugging, might want to make this a callback */
 			errno = ret_data->err;
@@ -867,7 +1061,10 @@ static int _launch_tasks(slurm_step_ctx ctx,
 	}
 	list_iterator_destroy(ret_itr);
 	list_destroy(ret_list);
-	return SLURM_SUCCESS;
+
+	if(tot_rc != SLURM_SUCCESS)
+		return tot_rc;
+	return rc;
 }
 
 /* returns an xmalloc cwd string, or NULL if lookup failed. */
@@ -881,3 +1078,123 @@ static char *_lookup_cwd(void)
 		return NULL;
 	}
 }
+
+static void _print_launch_msg(launch_tasks_request_msg_t *msg,
+			      char *hostname, int nodeid)
+{
+	int i;
+	char tmp_str[10], task_list[4096];
+	hostlist_t hl = hostlist_create("");
+
+	for (i=0; i<msg->tasks_to_launch[nodeid]; i++) {
+		sprintf(tmp_str, "%u", msg->global_task_ids[nodeid][i]);
+		hostlist_push(hl, tmp_str);
+	}
+	hostlist_ranged_string(hl, 4096, task_list);
+	hostlist_destroy(hl);
+	
+	info("launching %u.%u on host %s, %u tasks: %s", 
+	     msg->job_id, msg->job_step_id, hostname, 
+	     msg->tasks_to_launch[nodeid], task_list);
+
+	debug3("uid:%ld gid:%ld cwd:%s %d", (long) msg->uid,
+		(long) msg->gid, msg->cwd, nodeid);
+}
+
+void record_ppid(void)
+{
+	srun_ppid = getppid();
+}
+
+/* This is used to initiate an OpenMPI checkpoint program, 
+ * but is written to be general purpose */
+static void
+_exec_prog(slurm_msg_t *msg)
+{
+	pid_t child;
+	int pfd[2], status, exit_code = 0, i;
+	ssize_t len;
+	char *argv[4], buf[256] = "";
+	time_t now = time(NULL);
+	bool checkpoint = false;
+	srun_exec_msg_t *exec_msg = msg->data;
+
+	if (exec_msg->argc > 2) {
+		verbose("Exec '%s %s' for %u.%u", 
+			exec_msg->argv[0], exec_msg->argv[1],
+			exec_msg->job_id, exec_msg->step_id);
+	} else {
+		verbose("Exec '%s' for %u.%u", 
+			exec_msg->argv[0], 
+			exec_msg->job_id, exec_msg->step_id);
+	}
+
+	if (strcmp(exec_msg->argv[0], "ompi-checkpoint") == 0) {
+		if (srun_ppid)
+			checkpoint = true;
+		else {
+			error("Can not create checkpoint, no srun_ppid set");
+			exit_code = EINVAL;
+			goto fini;
+		}
+	}
+	if (checkpoint) {
+		/* OpenMPI specific checkpoint support */
+		info("Checkpoint started at %s", ctime(&now));
+		for (i=0; (exec_msg->argv[i] && (i<2)); i++) {
+			argv[i] = exec_msg->argv[i];
+		}
+		snprintf(buf, sizeof(buf), "%ld", (long) srun_ppid);
+		argv[i] = buf;
+		argv[i+1] = NULL;
+	}
+
+	if (pipe(pfd) == -1) {
+		snprintf(buf, sizeof(buf), "pipe: %s", strerror(errno));
+		error("%s", buf);
+		exit_code = errno;
+		goto fini;
+	}
+
+	child = fork();
+	if (child == 0) {
+		int fd = open("/dev/null", O_RDONLY);
+		dup2(fd, 0);		/* stdin from /dev/null */
+		dup2(pfd[1], 1);	/* stdout to pipe */
+		dup2(pfd[1], 2);	/* stderr to pipe */
+		close(pfd[0]);
+		close(pfd[1]);
+		if (checkpoint)
+			execvp(exec_msg->argv[0], argv);
+		else
+			execvp(exec_msg->argv[0], exec_msg->argv);
+		error("execvp(%s): %m", exec_msg->argv[0]);
+	} else if (child < 0) {
+		snprintf(buf, sizeof(buf), "fork: %s", strerror(errno));
+		error("%s", buf);
+		exit_code = errno;
+		goto fini;
+	} else {
+		close(pfd[1]);
+		len = read(pfd[0], buf, sizeof(buf));
+		close(pfd[0]);
+		waitpid(child, &status, 0);
+		exit_code = WEXITSTATUS(status);
+	}
+
+fini:	if (checkpoint) {
+		now = time(NULL);
+		if (exit_code) {
+			info("Checkpoint completion code %d at %s", 
+				exit_code, ctime(&now));
+		} else {
+			info("Checkpoint completed successfully at %s",
+				ctime(&now));
+		}
+		if (buf[0])
+			info("Checkpoint location: %s", buf);
+		slurm_checkpoint_complete(exec_msg->job_id, exec_msg->step_id,
+			time(NULL), (uint32_t) exit_code, buf);
+	}
+}
+
