@@ -1,11 +1,11 @@
 /*****************************************************************************\
  *  src/common/env.c - add an environment variable to environment vector
- *  $Id: env.c 12697 2007-11-27 22:02:29Z jette $
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>, Danny Auble <da@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -45,14 +45,17 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <unistd.h>
 #include <sys/poll.h>
+#include <sys/stat.h>
 #include <sys/types.h>
-
+#include <sys/param.h>		/* MAXPATHLEN */
 #include "src/common/macros.h"
 #include "slurm/slurm.h"
 #include "src/common/log.h"
 #include "src/common/env.h"
+#include "src/common/read_config.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
@@ -75,9 +78,7 @@ strong_alias(env_array_append_fmt,	slurm_env_array_append_fmt);
 strong_alias(env_array_overwrite,	slurm_env_array_overwrite);
 strong_alias(env_array_overwrite_fmt,	slurm_env_array_overwrite_fmt);
 
-#define SU_WAIT_MSEC 8000	/* 8000 msec for /bin/su to return user 
-				 * env vars for --get-user-env option */
-#define ENV_BUFSIZE (64 * 1024)
+#define ENV_BUFSIZE (256 * 1024)
 
 /*
  *  Return pointer to `name' entry in environment if found, or
@@ -132,6 +133,18 @@ _extend_env(char ***envp)
 	return (++ep);
 }
 
+/* return true if the environment variables should not be set for 
+ *	srun's --get-user-env option */
+static bool _discard_env(char *name, char *value)
+{
+	if ((strcmp(name, "DISPLAY")     == 0) ||
+	    (strcmp(name, "ENVIRONMENT") == 0) ||
+	    (strcmp(name, "HOSTNAME")    == 0))
+		return true;
+
+	return false;
+}
+
 /*
  * Return the number of elements in the environment `env'
  */
@@ -155,15 +168,16 @@ int
 setenvfs(const char *fmt, ...)
 {
 	va_list ap;
-	char buf[ENV_BUFSIZE];
-	char *bufcpy;
+	char *buf, *bufcpy;
 	int rc;
 
+	buf = xmalloc(ENV_BUFSIZE);
 	va_start(ap, fmt);
 	vsnprintf(buf, ENV_BUFSIZE, fmt, ap);
 	va_end(ap);
 	
 	bufcpy = xstrdup(buf);
+	xfree(buf);
 	rc = putenv(bufcpy);
 	return rc;
 }
@@ -171,17 +185,18 @@ setenvfs(const char *fmt, ...)
 int 
 setenvf(char ***envp, const char *name, const char *fmt, ...)
 {
-	char buf[ENV_BUFSIZE];
 	char **ep = NULL;
 	char *str = NULL;
 	va_list ap;
 	int rc;
-	char *bufcpy;
+	char *buf, *bufcpy;
 
+	buf = xmalloc(ENV_BUFSIZE);
 	va_start(ap, fmt);
 	vsnprintf (buf, ENV_BUFSIZE, fmt, ap);
 	va_end(ap);
 	bufcpy = xstrdup(buf);
+	xfree(buf);
 	
 	xstrfmtcat (str, "%s=%s", name, bufcpy);
 	xfree(bufcpy);
@@ -625,20 +640,39 @@ int setup_env(env_t *env)
 		 * determine what command a user will execute. We could
 		 * possibly add a "srestart" command which would set
 		 * MP_POERESTART_ENV, but that presently seems unnecessary. */
+		/* setenvf(&env->env, "MP_POERESTART_ENV", res_env); */
 		if (debug_env)
 			debug_num = atoi(debug_env);
 		snprintf(res_env, sizeof(res_env), "SLURM_LL_API_DEBUG=%d",
 			debug_num);
 		
-		setenvf(&env->env, "MP_POERESTART_ENV", res_env);
-
 		/* Required for AIX/POE systems indicating pre-allocation */
 		setenvf(&env->env, "LOADLBATCH", "yes");
 		setenvf(&env->env, "LOADL_ACTIVE", "3.2.0");
 	}
 #endif
-	
-	return SLURM_SUCCESS;
+
+	if (env->pty_port
+	&&  setenvf(&env->env, "SLURM_PTY_PORT", "%hu", env->pty_port)) {
+		error("Can't set SLURM_PTY_PORT env variable");
+		rc = SLURM_FAILURE;
+	}
+	if (env->ws_col
+	&&  setenvf(&env->env, "SLURM_PTY_WIN_COL", "%hu", env->ws_col)) {
+		error("Can't set SLURM_PTY_WIN_COL env variable");
+		rc = SLURM_FAILURE;
+	}
+	if (env->ws_row
+	&&  setenvf(&env->env, "SLURM_PTY_WIN_ROW", "%hu", env->ws_row)) {
+		error("Can't set SLURM_PTY_WIN_ROW env variable");
+		rc = SLURM_FAILURE;
+	}
+	if (env->ckpt_path 
+        && setenvf(&env->env, "SLURM_CHECKPOINT_PATH", "%s", env->ckpt_path)) {
+		error("Can't set SLURM_CHECKPOINT_PATH env variable");
+		rc = SLURM_FAILURE;
+	}
+	return rc;
 }
 
 /**********************************************************************
@@ -702,7 +736,7 @@ static char *_uint16_array_to_str(int array_len, const uint16_t *array)
  * This function returns the string representation of the compressed
  * array.  Free with xfree().
  */
-static char *_uint32_compressed_to_str(uint32_t array_len,
+extern char *uint32_compressed_to_str(uint32_t array_len,
 				       const uint32_t *array,
 				       const uint32_t *array_reps)
 {
@@ -756,7 +790,7 @@ env_array_for_job(char ***dest, const resource_allocation_response_msg_t *alloc)
 	env_array_overwrite_fmt(dest, "SLURM_JOB_NODELIST", "%s",
 				alloc->node_list);
 
-	tmp = _uint32_compressed_to_str((uint32_t)alloc->num_cpu_groups,
+	tmp = uint32_compressed_to_str((uint32_t)alloc->num_cpu_groups,
 					alloc->cpus_per_node,
 					alloc->cpu_count_reps);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_CPUS_PER_NODE", "%s", tmp);
@@ -787,6 +821,7 @@ env_array_for_job(char ***dest, const resource_allocation_response_msg_t *alloc)
  *	SLURM_JOB_NODELIST
  *	SLURM_JOB_CPUS_PER_NODE
  *	ENVIRONMENT=BATCH
+ *	HOSTNAME
  *	LOADLBATCH (AIX only)
  *
  * Sets OBSOLETE variables:
@@ -796,8 +831,9 @@ env_array_for_job(char ***dest, const resource_allocation_response_msg_t *alloc)
  *	SLURM_TASKS_PER_NODE <- poorly named, really CPUs per node
  *	? probably only needed for users...
  */
-void
-env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch)
+extern void
+env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
+			const char *node_name)
 {
 	char *tmp;
 	uint32_t num_nodes = 0;
@@ -812,11 +848,13 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch)
 	env_array_overwrite_fmt(dest, "SLURM_JOB_ID", "%u", batch->job_id);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_NUM_NODES", "%u", num_nodes);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_NODELIST", "%s", batch->nodes);
-	tmp = _uint32_compressed_to_str((uint32_t)batch->num_cpu_groups,
+	tmp = uint32_compressed_to_str((uint32_t)batch->num_cpu_groups,
 					batch->cpus_per_node,
 					batch->cpu_count_reps);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_CPUS_PER_NODE", "%s", tmp);
 	env_array_overwrite_fmt(dest, "ENVIRONMENT", "BATCH");
+	if (node_name)
+		env_array_overwrite_fmt(dest, "HOSTNAME", "%s", node_name);
 #ifdef HAVE_AIX
 	/* this puts the "poe" command into batch mode */
 	env_array_overwrite(dest, "LOADLBATCH", "yes");
@@ -862,8 +900,7 @@ void
 env_array_for_step(char ***dest, 
 		   const job_step_create_response_msg_t *step,
 		   const char *launcher_hostname,
-		   uint16_t launcher_port,
-		   const char *ip_addr_str)
+		   uint16_t launcher_port)
 {
 	char *tmp;
 
@@ -881,8 +918,6 @@ env_array_for_step(char ***dest,
 			 "%s", launcher_hostname);
 	env_array_overwrite_fmt(dest, "SLURM_STEP_LAUNCHER_PORT",
 			 "%hu", launcher_port);
-/* 	env_array_overwrite_fmt(dest, "SLURM_STEP_LAUNCHER_IPADDR", */
-/* 			 "%s", ip_addr_str); */
 
 	/* OBSOLETE */
 	env_array_overwrite_fmt(dest, "SLURM_STEPID", "%u", step->job_step_id);
@@ -895,8 +930,6 @@ env_array_for_step(char ***dest,
 			 "%s", launcher_hostname);
 	env_array_overwrite_fmt(dest, "SLURM_SRUN_COMM_PORT",
 			 "%hu", launcher_port);
-/* 	env_array_overwrite_fmt(dest, "SLURM_LAUNCH_NODE_IPADDR", */
-/* 			 "%s", ip_addr_str); */
 
 	xfree(tmp);
 }
@@ -946,30 +979,28 @@ char **env_array_create(void)
 int env_array_append_fmt(char ***array_ptr, const char *name,
 			 const char *value_fmt, ...)
 {
-	char buf[ENV_BUFSIZE];
+	char *buf;
 	char **ep = NULL;
 	char *str = NULL;
 	va_list ap;
 
-	buf[0] = '\0';
-	if (array_ptr == NULL) {
+	if (array_ptr == NULL)
 		return 0;
-	}
 
-	if (*array_ptr == NULL) {
+	if (*array_ptr == NULL)
 		*array_ptr = env_array_create();
-	}
 
+	ep = _find_name_in_env(*array_ptr, name);
+	if (*ep != NULL)
+		return 0;
+
+	buf = xmalloc(ENV_BUFSIZE);
 	va_start(ap, value_fmt);
 	vsnprintf (buf, ENV_BUFSIZE, value_fmt, ap);
 	va_end(ap);
-	
-	ep = _find_name_in_env(*array_ptr, name);
-	if (*ep != NULL) {
-		return 0;
-	}
 
 	xstrfmtcat (str, "%s=%s", name, buf);
+	xfree(buf);
 	ep = _extend_env(array_ptr);
 	*ep = str;
 	
@@ -989,18 +1020,15 @@ int env_array_append(char ***array_ptr, const char *name,
 	char **ep = NULL;
 	char *str = NULL;
 
-	if (array_ptr == NULL) {
+	if (array_ptr == NULL)
 		return 0;
-	}
 
-	if (*array_ptr == NULL) {
+	if (*array_ptr == NULL)
 		*array_ptr = env_array_create();
-	}
 
 	ep = _find_name_in_env(*array_ptr, name);
-	if (*ep != NULL) {
+	if (*ep != NULL)
 		return 0;
-	}
 
 	xstrfmtcat (str, "%s=%s", name, value);
 	ep = _extend_env(array_ptr);
@@ -1022,25 +1050,24 @@ int env_array_append(char ***array_ptr, const char *name,
 int env_array_overwrite_fmt(char ***array_ptr, const char *name,
 			    const char *value_fmt, ...)
 {
-	char buf[ENV_BUFSIZE];
+	char *buf;
 	char **ep = NULL;
 	char *str = NULL;
 	va_list ap;
 
-	buf[0] = '\0';
-	if (array_ptr == NULL) {
+	if (array_ptr == NULL)
 		return 0;
-	}
 
-	if (*array_ptr == NULL) {
+	if (*array_ptr == NULL)
 		*array_ptr = env_array_create();
-	}
 
+	buf = xmalloc(ENV_BUFSIZE);
 	va_start(ap, value_fmt);
 	vsnprintf (buf, ENV_BUFSIZE, value_fmt, ap);
 	va_end(ap);
 	
 	xstrfmtcat (str, "%s=%s", name, buf);
+	xfree(buf);
 	ep = _find_name_in_env(*array_ptr, name);
 	if (*ep != NULL) {
 		xfree (*ep);
@@ -1131,6 +1158,8 @@ static int _env_array_entry_splitter(const char *entry,
 	int len;
 
 	ptr = index(entry, '=');
+	if (ptr == NULL)	/* Bad parsing, no '=' found */
+		return 0;
 	len = ptr - entry;
 	if (len > name_len-1)
 		return 0;
@@ -1156,16 +1185,17 @@ static int _env_array_entry_splitter(const char *entry,
  */
 static int _env_array_putenv(const char *string)
 {
-	char name[ENV_BUFSIZE];
-	char value[ENV_BUFSIZE];
+	int rc = 0;
+	char name[256], *value;
 
-	if (!_env_array_entry_splitter(string, name, ENV_BUFSIZE, value, 
-				       ENV_BUFSIZE))
-		return 0;
-	if (setenv(name, value, 1) == -1)
-		return 0;
-	
-	return 1;
+	value = xmalloc(ENV_BUFSIZE);
+	if ((_env_array_entry_splitter(string, name, sizeof(name),
+				       value, ENV_BUFSIZE)) &&
+	    (setenv(name, value, 1) != -1))
+		rc = 1;
+
+	xfree(value);
+	return rc;
 }
 
 /*
@@ -1192,17 +1222,18 @@ void env_array_set_environment(char **env_array)
 void env_array_merge(char ***dest_array, const char **src_array)
 {
 	char **ptr;
-	char name[ENV_BUFSIZE];
-	char value[ENV_BUFSIZE];
+	char name[256], *value;
 
 	if (src_array == NULL)
 		return;
 
+	value = xmalloc(ENV_BUFSIZE);
 	for (ptr = (char **)src_array; *ptr != NULL; ptr++) {
-		_env_array_entry_splitter(*ptr, name, ENV_BUFSIZE, value, 
-					  ENV_BUFSIZE);
-		env_array_overwrite(dest_array, name, value);
+		if (_env_array_entry_splitter(*ptr, name, sizeof(name),
+					      value, ENV_BUFSIZE))
+			env_array_overwrite(dest_array, name, value);
 	}
+	xfree(value);
 }
 
 /*
@@ -1222,14 +1253,31 @@ static void _strip_cr_nl(char *line)
 	}
 }
 
+/* Return the net count of curly brackets in a string
+ * '{' adds one and '}' subtracts one (zero means it is balanced).
+ * Special case: return -1 if no open brackets are found */
+static int _bracket_cnt(char *value)
+{
+	int open_br = 0, close_br = 0, i;
+	for (i=0; value[i]; i++) {
+		if (value[i] == '{')
+			open_br++;
+		else if (value[i] == '}')
+			close_br++;
+	}
+	if (open_br == 0)
+		return -1;
+	return (open_br - close_br);
+}
+
 /*
  * Load user environment from a cache file located in
  * <state_save_location>/env_username
  */
-char **_load_env_cache(const char *username)
+static char **_load_env_cache(const char *username)
 {
-	char *state_save_loc, fname[ENV_BUFSIZE];
-	char line[ENV_BUFSIZE], name[ENV_BUFSIZE], value[ENV_BUFSIZE];
+	char *state_save_loc, fname[MAXPATHLEN];
+	char *line, name[256], *value;
 	char **env = NULL;
 	FILE *fp;
 	int i;
@@ -1239,25 +1287,47 @@ char **_load_env_cache(const char *username)
 		     username);
 	xfree(state_save_loc);
 	if (i < 0) {
-		fatal("Environment cache filename overflow");
+		error("Environment cache filename overflow");
 		return NULL;
 	}
 	if (!(fp = fopen(fname, "r"))) {
-		fatal("Could not open user environment cache at %s: %m",
+		error("Could not open user environment cache at %s: %m",
 			fname);
 		return NULL;
 	}
 
-	info("Getting cached environment variables at %s", fname);
+	verbose("Getting cached environment variables at %s", fname);
 	env = env_array_create();
+	line  = xmalloc(ENV_BUFSIZE);
+	value = xmalloc(ENV_BUFSIZE);
 	while (1) {
 		if (!fgets(line, ENV_BUFSIZE, fp))
 			break;
 		_strip_cr_nl(line);
-		_env_array_entry_splitter(line, name, ENV_BUFSIZE, value, 
-					  ENV_BUFSIZE);
-		env_array_overwrite(&env, name, value);
+		if (_env_array_entry_splitter(line, name, sizeof(name), 
+					      value, ENV_BUFSIZE) &&
+		    (!_discard_env(name, value))) {
+			if (value[0] == '(') {
+				/* This is a bash function.
+				 * It may span multiple lines */
+				int bracket_cnt;
+				while ((bracket_cnt = _bracket_cnt(value))) {
+					if (!fgets(line, ENV_BUFSIZE, fp))
+						break;
+					_strip_cr_nl(line);
+					if ((strlen(value) + strlen(line)) >
+					    (sizeof(value) - 1))
+						break;
+					strcat(value, "\n");
+					strcat(value, line);
+				}
+			}
+			env_array_overwrite(&env, name, value);
+		}
 	}
+	xfree(line);
+	xfree(value);
+
 	fclose(fp);
 	return env;
 }
@@ -1271,7 +1341,7 @@ char **_load_env_cache(const char *username)
  * 2. Load the user environment from a cache file. This is used
  *    in the event that option 1 times out.
  *
- * timeout value is in seconds or zero for default (8 secs) 
+ * timeout value is in seconds or zero for default (2 secs) 
  * mode is 1 for short ("su <user>"), 2 for long ("su - <user>")
  * On error, returns NULL.
  *
@@ -1280,24 +1350,42 @@ char **_load_env_cache(const char *username)
  */
 char **env_array_user_default(const char *username, int timeout, int mode)
 {
-	FILE *su;
-	char line[ENV_BUFSIZE];
-	char name[ENV_BUFSIZE];
-	char value[ENV_BUFSIZE];
+	char *line = NULL, *last = NULL, name[128], *value, *buffer;
 	char **env = NULL;
 	char *starttoken = "XXXXSLURMSTARTPARSINGHEREXXXX";
 	char *stoptoken  = "XXXXSLURMSTOPPARSINGHEREXXXXX";
-	char cmdstr[256];
+	char cmdstr[256], *env_loc = NULL;
 	int fildes[2], found, fval, len, rc, timeleft;
+	int buf_read, buf_rem, config_timeout;
 	pid_t child;
 	struct timeval begin, now;
 	struct pollfd ufds;
+	struct stat buf;
 
 	if (geteuid() != (uid_t)0) {
-		info("WARNING: you must be root to use --get-user-env");
+		fatal("WARNING: you must be root to use --get-user-env");
 		return NULL;
 	}
 
+	config_timeout = slurm_get_env_timeout();
+	if (config_timeout == 0)	/* just read directly from cache */
+		 return _load_env_cache(username);
+
+	if (stat("/bin/su", &buf))
+		fatal("Could not locate command: /bin/su");
+	if (stat("/bin/echo", &buf))
+		fatal("Could not locate command: /bin/echo");
+	if (stat("/bin/env", &buf) == 0)
+		env_loc = "/bin/env";
+	else if (stat("/usr/bin/env", &buf) == 0)
+		env_loc = "/usr/bin/env";
+	else
+		fatal("Could not location command: env");
+	snprintf(cmdstr, sizeof(cmdstr),
+		 "/bin/echo; /bin/echo; /bin/echo; "
+		 "/bin/echo %s; %s; /bin/echo %s",
+		 starttoken, env_loc, stoptoken);
+
 	if (pipe(fildes) < 0) {
 		fatal("pipe: %m");
 		return NULL;
@@ -1309,14 +1397,12 @@ char **env_array_user_default(const char *username, int timeout, int mode)
 		return NULL;
 	}
 	if (child == 0) {
+		setpgid(0, 0);
 		close(0);
 		open("/dev/null", O_RDONLY);
 		dup2(fildes[1], 1);
 		close(2);
 		open("/dev/null", O_WRONLY);
-		snprintf(cmdstr, sizeof(cmdstr),
-			 "echo; echo; echo; echo %s; env; echo %s",
-			 starttoken, stoptoken);
 		if      (mode == 1)
 			execl("/bin/su", "su", username, "-c", cmdstr, NULL);
 		else if (mode == 2)
@@ -1332,27 +1418,31 @@ char **env_array_user_default(const char *username, int timeout, int mode)
 	}
 
 	close(fildes[1]);
-	if ((fval = fcntl(fildes[0], F_GETFL, 0)) >= 0)
-		fcntl(fildes[0], F_SETFL, fval | O_NONBLOCK);
-	su= fdopen(fildes[0], "r");
+	if ((fval = fcntl(fildes[0], F_GETFL, 0)) < 0)
+		error("fcntl(F_GETFL) failed: %m");
+	else if (fcntl(fildes[0], F_SETFL, fval | O_NONBLOCK) < 0)
+		error("fcntl(F_SETFL) failed: %m");
 
 	gettimeofday(&begin, NULL);
 	ufds.fd = fildes[0];
 	ufds.events = POLLIN;
 
-	/* First look for the start token in the output */
-	len = strlen(starttoken);
+	/* Read all of the output from /bin/su into buffer */
+	if (timeout == 0)
+		timeout = config_timeout;	/* != 0 test above */
 	found = 0;
-	while (!found) {
+	buf_read = 0;
+	buffer = xmalloc(ENV_BUFSIZE);
+	while (1) {
 		gettimeofday(&now, NULL);
-		if (timeout > 0)
-			timeleft = timeout * 1000;
-		else
-			timeleft = SU_WAIT_MSEC;
+		timeleft = timeout * 1000;
 		timeleft -= (now.tv_sec -  begin.tv_sec)  * 1000;
 		timeleft -= (now.tv_usec - begin.tv_usec) / 1000;
-		if (timeleft <= 0)
+		if (timeleft <= 0) {
+			verbose("timeout waiting for /bin/su to complete");
+			kill(-child, 9);
 			break;
+		}
 		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
 			if (rc == 0) {
 				verbose("timeout waiting for /bin/su to complete");
@@ -1360,64 +1450,114 @@ char **env_array_user_default(const char *username, int timeout, int mode)
 			}
 			if ((errno == EINTR) || (errno == EAGAIN))
 				continue;
-			error("poll: %m");
+			error("poll(): %m");
 			break;
 		}
-		if (!(ufds.revents & POLLIN))
-			break;
-		while (fgets(line, ENV_BUFSIZE, su)) {
-			if (!strncmp(line, starttoken, len)) {
-				found = 1;
-				break;
+		if (!(ufds.revents & POLLIN)) {
+			if (ufds.revents & POLLHUP) {	/* EOF */
+				found = 1;		/* success */
+			} else if (ufds.revents & POLLERR) {
+				error("POLLERR");
+			} else {
+				error("poll() revents=%d", ufds.revents);
 			}
+			break;
+		}
+		buf_rem = ENV_BUFSIZE - buf_read;
+		if (buf_rem == 0) {
+			error("buffer overflow loading env vars");
+			break;
+		}
+		rc = read(fildes[0], &buffer[buf_read], buf_rem);
+		if (rc > 0)
+			buf_read += rc;
+		else if (rc == 0) {	/* EOF */
+			found = 1;	/* success */
+			break;
+		} else {		/* error */
+			error("read(env pipe): %m");
+			break;
 		}
 	}
+	close(fildes[0]);
+	for (config_timeout=0; ; config_timeout++) {
+		kill(-child, SIGKILL);	/* Typically a no-op */
+		if (config_timeout)
+			sleep(1);
+		if (waitpid(child, &rc, WNOHANG) > 0)
+			break;
+		if (config_timeout >= 2) {
+			/* Non-killable processes are indicative of file system
+			 * problems. The process will remain as a zombie, but 
+			 * slurmd/salloc/moab will not otherwise be effected. */
+			error("Failed to kill program loading user environment");
+			break;
+		}
+	}
+	
 	if (!found) {
-		error("Failed to get current user environment variables");
-		close(fildes[0]);
+		error("Failed to load current user environment variables");
+		xfree(buffer);
 		return _load_env_cache(username);
 	}
 
-	/* Now read in the environment variable strings. */
-	env = env_array_create();
-	len = strlen(stoptoken);
+	/* First look for the start token in the output */
+	len = strlen(starttoken);
 	found = 0;
-	while (!found) {
-		gettimeofday(&now, NULL);
-		if (timeout > 0)
-			timeleft = timeout * 1000;
-		else
-			timeleft = SU_WAIT_MSEC;
-		timeleft -= (now.tv_sec -  begin.tv_sec)  * 1000;
-		timeleft -= (now.tv_usec - begin.tv_usec) / 1000;
-		if (timeleft <= 0)
-			break;
-		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
-			if (rc == 0) {
-				verbose("timeout waiting for /bin/su to complete");
-				break;
-			}
-			if ((errno == EINTR) || (errno == EAGAIN))
-				continue;
-			error("poll: %m");
-			break;
-		}
-		/* stop at the line containing the stoptoken string */
-		if (!(ufds.revents & POLLIN))
-			break;
-		if ((fgets(line, ENV_BUFSIZE, su) == 0) ||
-		    (!strncmp(line, stoptoken, len))) {
+	line = strtok_r(buffer, "\n", &last);
+	while (!found && line) {
+		if (!strncmp(line, starttoken, len)) {
 			found = 1;
 			break;
 		}
+		line = strtok_r(NULL, "\n", &last);
+	}
+	if (!found) {
+		error("Failed to get current user environment variables");
+		xfree(buffer);
+		return _load_env_cache(username);
+	}
 
-		_strip_cr_nl(line);
-		_env_array_entry_splitter(line, name, ENV_BUFSIZE, value, 
-					  ENV_BUFSIZE);
-		env_array_overwrite(&env, name, value);
+	/* Process environment variables until we find the stop token */
+	len = strlen(stoptoken);
+	found = 0;
+	env = env_array_create();
+	line = strtok_r(NULL, "\n", &last);
+	value = xmalloc(ENV_BUFSIZE);
+	while (!found && line) {
+		if (!strncmp(line, stoptoken, len)) {
+			found = 1;
+			break;
+		}
+		if (_env_array_entry_splitter(line, name, sizeof(name), 
+					      value, ENV_BUFSIZE) &&
+		    (!_discard_env(name, value))) {
+			if (value[0] == '(') {
+				/* This is a bash function.
+				 * It may span multiple lines */
+				int bracket_cnt;
+				while ((bracket_cnt = _bracket_cnt(value))) {
+					line = strtok_r(NULL, "\n", &last);
+					if (!line)
+						break;
+					if ((strlen(value) + strlen(line)) >
+					    (sizeof(value) - 1))
+						break;
+					strcat(value, "\n");
+					strcat(value, line);
+				}
+			}
+			env_array_overwrite(&env, name, value);
+		}
+		line = strtok_r(NULL, "\n", &last);
+	}
+	xfree(value);
+	xfree(buffer);
+	if (!found) {
+		error("Failed to get all user environment variables");
+		env_array_free(env);
+		return _load_env_cache(username);
 	}
-	close(fildes[0]);
 
 	return env;
 }
-
