@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2005 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -63,6 +63,7 @@
 
 struct spank_plugin_operations {
 	spank_f *init;
+	spank_f *init_post_opt;
 	spank_f *local_user_init;
 	spank_f *user_init;
 	spank_f *user_task_init;
@@ -71,9 +72,10 @@ struct spank_plugin_operations {
 	spank_f *exit;
 };
 
-const int n_spank_syms = 7;
+const int n_spank_syms = 8;
 const char *spank_syms[] = {
 	"slurm_spank_init",
+	"slurm_spank_init_post_opt",
 	"slurm_spank_local_user_init",
 	"slurm_spank_user_init",
 	"slurm_spank_task_init",
@@ -131,6 +133,7 @@ typedef enum spank_handle_type {
  */
 typedef enum step_fn {
 	SPANK_INIT = 0,
+	SPANK_INIT_POST_OPT,
 	LOCAL_USER_INIT,
 	STEP_USER_INIT,
 	STEP_USER_TASK_INIT,
@@ -377,8 +380,13 @@ static int _spank_stack_create(const char *path, List * listp)
 
 	verbose("spank: opening plugin stack %s\n", path);
 
-	if (!(fp = safeopen(path, "r", SAFEOPEN_NOCREATE)))
+	if (!(fp = safeopen(path, "r", SAFEOPEN_NOCREATE))) {
+		if (errno == ENOENT)
+			debug("spank: Failed to open %s: %m\n", path);
+		else
+			error("spank: Failed to open %s: %m\n", path);
 		return -1;
+	}
 
 	line = 1;
 	while (fgets(buf, sizeof(buf), fp)) {
@@ -444,6 +452,8 @@ static const char *_step_fn_name(step_fn_t type)
 	switch (type) {
 	case SPANK_INIT:
 		return ("init");
+	case SPANK_INIT_POST_OPT:
+		return ("init_post_opt");
 	case LOCAL_USER_INIT:
 		return ("local_user_init");
 	case STEP_USER_INIT:
@@ -493,6 +503,14 @@ static int _do_call_stack(step_fn_t type, void * job, int taskid)
 				       fn_name, rc);
 			}
 			break;
+		case SPANK_INIT_POST_OPT:
+			if (sp->ops.init_post_opt) {
+				rc = (*sp->ops.init_post_opt) (spank, sp->ac,
+						      sp->argv);
+				debug2("spank: %s: %s = %d\n", name,
+				       fn_name, rc);
+			}
+			break;
 		case LOCAL_USER_INIT:
 			if (sp->ops.local_user_init) {
 				rc = (*sp->ops.local_user_init) (spank, sp->ac, 
@@ -581,6 +599,9 @@ int spank_init(slurmd_job_t * job)
 		return (-1);
 	}
 
+	if (_do_call_stack(SPANK_INIT_POST_OPT, job, -1) < 0)
+		return (-1);
+
 	return (0);
 }
 
@@ -1017,7 +1038,7 @@ static struct spank_plugin_opt *_find_remote_option_by_name(const char
 	}
 
 	if (!(name = strchr(buf, ':'))) {
-		error("Malformed plugin option \"%s\" recieved. Ignoring",
+		error("Malformed plugin option \"%s\" received. Ignoring",
 		      str);
 		return (NULL);
 	}
@@ -1105,7 +1126,7 @@ global_to_local_id (slurmd_job_t *job, uint32_t gid, uint32_t *p2uint32)
 /*
  *  Return 1 if spank_item_t is valid for S_TYPE_LOCAL
  */
-static int valid_in_local_context (spank_item_t item)
+static int _valid_in_local_context (spank_item_t item)
 {
 	int rc = 0;
 	switch (item) {
@@ -1125,6 +1146,24 @@ static int valid_in_local_context (spank_item_t item)
 	return (rc);
 }
 
+/*
+ *  Return 1 if spank_item_t is just getting version (valid anywhere)
+ */
+static int _version_check (spank_item_t item)
+{
+	int rc = 0;
+	switch (item) {
+	case S_SLURM_VERSION:
+	case S_SLURM_VERSION_MAJOR:
+	case S_SLURM_VERSION_MINOR:
+	case S_SLURM_VERSION_MICRO:
+		rc = 1;
+		break;
+	default:
+		rc = 0;
+	}
+	return (rc);
+}
 
 /*
  *  Global functions for SPANK plugins
@@ -1167,6 +1206,7 @@ spank_err_t spank_get_item(spank_t spank, spank_item_t item, ...)
 	pid_t *p2pid;
 	pid_t  pid;
 	char ***p2argv;
+	char **p2vers;
 	slurmd_task_info_t *task;
 	slurmd_job_t  *slurmd_job = NULL;
 	struct spank_launcher_job_info *launcher_job = NULL;
@@ -1176,17 +1216,20 @@ spank_err_t spank_get_item(spank_t spank, spank_item_t item, ...)
 	if ((spank == NULL) || (spank->magic != SPANK_MAGIC))
 		return (ESPANK_BAD_ARG);
 
-	if ( (spank->type != S_TYPE_REMOTE) 
-	  && (!valid_in_local_context(item)))
-		return (ESPANK_NOT_REMOTE);
+	if (!_version_check(item)) {
+		/* Need job pointer to process other items */
+		if ( (spank->type != S_TYPE_REMOTE) 
+		  && (!_valid_in_local_context(item)))
+			return (ESPANK_NOT_REMOTE);
 
-	if (spank->job == NULL)
-		return (ESPANK_BAD_ARG);
+		if (spank->job == NULL)
+			return (ESPANK_BAD_ARG);
 
-	if (spank->type == S_TYPE_LOCAL)
-		launcher_job = spank->job;
-	else
-		slurmd_job = spank->job;
+		if (spank->type == S_TYPE_LOCAL)
+			launcher_job = spank->job;
+		else
+			slurmd_job = spank->job;
+	}
 
 	va_start(vargs, item);
 	switch (item) {
@@ -1250,6 +1293,10 @@ spank_err_t spank_get_item(spank_t spank, spank_item_t item, ...)
 		p2uint16 = va_arg(vargs, uint16_t *);
 		*p2uint16 = slurmd_job->cpus;
 		break;
+	case S_STEP_CPUS_PER_TASK:
+		p2uint32 = va_arg(vargs, uint32_t *);
+		*p2uint32 = slurmd_job->cpus_per_task;
+		break;
 	case S_JOB_ARGV:
 		p2int = va_arg(vargs, int *);
 		p2argv = va_arg(vargs, char ***);
@@ -1338,6 +1385,22 @@ spank_err_t spank_get_item(spank_t spank, spank_item_t item, ...)
 		p2uint32 = va_arg(vargs, uint32_t *);
 		rc = global_to_local_id (slurmd_job, uint32, p2uint32);
 		break;
+	case S_SLURM_VERSION:
+		p2vers = va_arg(vargs, char  **);
+		*p2vers = SLURM_VERSION;
+		break;
+	case S_SLURM_VERSION_MAJOR:
+		p2vers = va_arg(vargs, char  **);
+		*p2vers = SLURM_MAJOR;
+		break;
+	case S_SLURM_VERSION_MINOR:
+		p2vers = va_arg(vargs, char  **);
+		*p2vers = SLURM_MINOR;
+		break;
+	case S_SLURM_VERSION_MICRO:
+		p2vers = va_arg(vargs, char  **);
+		*p2vers = SLURM_MICRO;
+		break;
 	default:
 		rc = ESPANK_BAD_ARG;
 		break;
