@@ -1,10 +1,12 @@
 /*****************************************************************************\
  *  read_config.c - read the overall slurm configuration file
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Portions Copyright (C) 2008 Vijay Ramasubramanian.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -42,6 +44,7 @@
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <netdb.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,7 +67,9 @@
 #include "src/common/xstring.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/parse_config.h"
+#include "src/common/parse_time.h"
 #include "src/common/slurm_selecttype_info.h"
+#include "src/common/util-net.h"
 
 /* Instantiation of the "extern slurm_ctl_conf_t slurmcltd_conf"
  * found in slurmctld.h */
@@ -119,46 +124,86 @@ static void validate_and_set_defaults(slurm_ctl_conf_t *conf,
 				      s_p_hashtbl_t *hashtbl);
 
 s_p_options_t slurm_conf_options[] = {
+	{"AccountingStorageEnforce", S_P_UINT16},
+	{"AccountingStorageHost", S_P_STRING},
+	{"AccountingStorageLoc", S_P_STRING},
+	{"AccountingStoragePass", S_P_STRING},
+	{"AccountingStoragePort", S_P_UINT32},
+	{"AccountingStorageType", S_P_STRING},
+	{"AccountingStorageUser", S_P_STRING},
 	{"AuthType", S_P_STRING},
-	{"CheckpointType", S_P_STRING},
-	{"CacheGroups", S_P_UINT16},
 	{"BackupAddr", S_P_STRING},
 	{"BackupController", S_P_STRING},
+	{"CheckpointType", S_P_STRING},
+	{"CacheGroups", S_P_UINT16},
+	{"ClusterName", S_P_STRING},
 	{"ControlAddr", S_P_STRING},
 	{"ControlMachine", S_P_STRING},
+	{"CryptoType", S_P_STRING},
+	{"DefaultStorageHost", S_P_STRING},
+	{"DefaultStorageLoc", S_P_STRING},
+	{"DefaultStoragePass", S_P_STRING},
+	{"DefaultStoragePort", S_P_UINT32},
+	{"DefaultStorageType", S_P_STRING},
+	{"DefaultStorageUser", S_P_STRING},
+	{"DefMemPerCPU", S_P_UINT32},
+	{"DefMemPerNode", S_P_UINT32},
+	{"DefMemPerTask", S_P_UINT32},	/* defunct */
+	{"DisableRootJobs", S_P_BOOLEAN},
+	{"EnforcePartLimits", S_P_BOOLEAN},
 	{"Epilog", S_P_STRING},
+	{"EpilogMsgTime", S_P_UINT32},
 	{"FastSchedule", S_P_UINT16},
 	{"FirstJobId", S_P_UINT32},
+	{"GetEnvTimeout", S_P_UINT16},
 	{"HashBase", S_P_LONG, defunct_option},
 	{"HeartbeatInterval", S_P_LONG, defunct_option},
+	{"HealthCheckInterval", S_P_UINT16},
+	{"HealthCheckProgram", S_P_STRING},
 	{"InactiveLimit", S_P_UINT16},
+	{"JobAcctGatherType", S_P_STRING},
+	{"JobAcctFrequency", S_P_UINT16, defunct_option},
+	{"JobAcctGatherFrequency", S_P_UINT16},
 	{"JobAcctLogFile", S_P_STRING},
-	{"JobAcctFrequency", S_P_UINT16},
 	{"JobAcctType", S_P_STRING},
+	{"JobCompHost", S_P_STRING},
 	{"JobCompLoc", S_P_STRING},
+	{"JobCompPass", S_P_STRING},
+	{"JobCompPort", S_P_UINT32},
 	{"JobCompType", S_P_STRING},
+	{"JobCompUser", S_P_STRING},
 	{"JobCredentialPrivateKey", S_P_STRING},
 	{"JobCredentialPublicCertificate", S_P_STRING},
 	{"JobFileAppend", S_P_UINT16},
+	{"JobRequeue", S_P_UINT16},
 	{"KillTree", S_P_UINT16, defunct_option},
 	{"KillWait", S_P_UINT16},
+	{"Licenses", S_P_STRING},
 	{"MailProg", S_P_STRING},
 	{"MaxJobCount", S_P_UINT16},
+	{"MaxMemPerCPU", S_P_UINT32},
+	{"MaxMemPerNode", S_P_UINT32},
+	{"MaxMemPerTask", S_P_UINT32},	/* defunct */
 	{"MessageTimeout", S_P_UINT16},
 	{"MinJobAge", S_P_UINT16},
-	{"MpichGmDirectSupport", S_P_LONG},
+	{"MpichGmDirectSupport", S_P_LONG, defunct_option},
 	{"MpiDefault", S_P_STRING},
 	{"PluginDir", S_P_STRING},
 	{"PlugStackConfig", S_P_STRING},
+	{"PrivateData", S_P_STRING},
 	{"ProctrackType", S_P_STRING},
 	{"Prolog", S_P_STRING},
 	{"PropagatePrioProcess", S_P_UINT16},
 	{"PropagateResourceLimitsExcept", S_P_STRING},
 	{"PropagateResourceLimits", S_P_STRING},
+	{"ResumeProgram", S_P_STRING},
+	{"ResumeRate", S_P_UINT16},
 	{"ReturnToService", S_P_UINT16},
-	{"SchedulerAuth", S_P_STRING},
+	{"SchedulerAuth", S_P_STRING, defunct_option},
+	{"SchedulerParameters", S_P_STRING},
 	{"SchedulerPort", S_P_UINT16},
 	{"SchedulerRootFilter", S_P_UINT16},
+	{"SchedulerTimeSlice", S_P_UINT16},
 	{"SchedulerType", S_P_STRING},
 	{"SelectType", S_P_STRING},
 	{"SelectTypeParameters", S_P_STRING},
@@ -177,6 +222,11 @@ s_p_options_t slurm_conf_options[] = {
 	{"SrunEpilog", S_P_STRING},
 	{"SrunProlog", S_P_STRING},
 	{"StateSaveLocation", S_P_STRING},
+	{"SuspendExcNodes", S_P_STRING},
+	{"SuspendExcParts", S_P_STRING},
+	{"SuspendProgram", S_P_STRING},
+	{"SuspendRate", S_P_UINT16},
+	{"SuspendTime", S_P_LONG},
 	{"SwitchType", S_P_STRING},
 	{"TaskEpilog", S_P_STRING},
 	{"TaskProlog", S_P_STRING},
@@ -212,10 +262,10 @@ static int parse_nodename(void **dest, slurm_parser_enum_t type,
 	s_p_hashtbl_t *tbl, *dflt;
 	slurm_conf_node_t *n;
 	static s_p_options_t _nodename_options[] = {
-		{"NodeHostname", S_P_STRING},
-		{"NodeAddr", S_P_STRING},
 		{"CoresPerSocket", S_P_UINT16},
 		{"Feature", S_P_STRING},
+		{"NodeAddr", S_P_STRING},
+		{"NodeHostname", S_P_STRING},
 		{"Port", S_P_UINT16},
 		{"Procs", S_P_UINT16},
 		{"RealMemory", S_P_UINT32},
@@ -278,8 +328,8 @@ static int parse_nodename(void **dest, slurm_parser_enum_t type,
 
 		if (!s_p_get_uint16(&n->port, "Port", tbl)
 		    && !s_p_get_uint16(&n->port, "Port", dflt)) {
-			/* This gets resolved in slurm_conf_get_port()
-			 * and slurm_conf_get_addr(). For now just
+			/* This gets resolved in slurm_conf_get_port() 
+			 * and slurm_conf_get_addr(). For now just 
 			 * leave with a value of zero */
 			n->port = 0;
 		}
@@ -315,7 +365,7 @@ static int parse_nodename(void **dest, slurm_parser_enum_t type,
 
 		if (!s_p_get_uint32(&n->tmp_disk, "TmpDisk", tbl)
 		    && !s_p_get_uint32(&n->tmp_disk, "TmpDisk", dflt))
-			n->tmp_disk = 1;
+			n->tmp_disk = 0;
 
 		if (!s_p_get_uint32(&n->weight, "Weight", tbl)
 		    && !s_p_get_uint32(&n->weight, "Weight", dflt))
@@ -398,11 +448,13 @@ static int parse_partitionname(void **dest, slurm_parser_enum_t type,
 	static s_p_options_t _partition_options[] = {
 		{"AllowGroups", S_P_STRING},
 		{"Default", S_P_BOOLEAN}, /* YES or NO */
+		{"DisableRootJobs", S_P_BOOLEAN}, /* YES or NO */
 		{"Hidden", S_P_BOOLEAN}, /* YES or NO */
-		{"MaxTime", S_P_UINT32}, /* INFINITE or a number */
+		{"MaxTime", S_P_STRING},
 		{"MaxNodes", S_P_UINT32}, /* INFINITE or a number */
 		{"MinNodes", S_P_UINT32},
 		{"Nodes", S_P_STRING},
+		{"Priority", S_P_UINT16},
 		{"RootOnly", S_P_BOOLEAN}, /* YES or NO */
 		{"Shared", S_P_STRING}, /* YES, NO, or FORCE */
 		{"State", S_P_BOOLEAN}, /* UP or DOWN */
@@ -437,13 +489,29 @@ static int parse_partitionname(void **dest, slurm_parser_enum_t type,
 		    && !s_p_get_boolean(&p->default_flag, "Default", dflt))
 			p->default_flag = false;
 
+		if (!s_p_get_boolean((bool *)&p->disable_root_jobs,
+				     "DisableRootJobs", tbl))
+			p->disable_root_jobs = (uint16_t)NO_VAL;
+
 		if (!s_p_get_boolean(&p->hidden_flag, "Hidden", tbl)
 		    && !s_p_get_boolean(&p->hidden_flag, "Hidden", dflt))
 			p->hidden_flag = false;
 
-		if (!s_p_get_uint32(&p->max_time, "MaxTime", tbl)
-		    && !s_p_get_uint32(&p->max_time, "MaxTime", dflt))
+		if (!s_p_get_string(&tmp, "MaxTime", tbl) &&
+		    !s_p_get_string(&tmp, "MaxTime", dflt))
 			p->max_time = INFINITE;
+		else {
+			int max_time = time_str2mins(tmp);
+			if ((max_time < 0) && (max_time != INFINITE)) {
+				error("Bad value \"%s\" for MaxTime", tmp);
+				destroy_partitionname(p);
+				s_p_hashtbl_destroy(tbl);
+				xfree(tmp);
+				return -1;
+			}
+			p->max_time = max_time;
+			xfree(tmp);
+		}
 
 		if (!s_p_get_uint32(&p->max_nodes, "MaxNodes", tbl)
 		    && !s_p_get_uint32(&p->max_nodes, "MaxNodes", dflt))
@@ -456,25 +524,39 @@ static int parse_partitionname(void **dest, slurm_parser_enum_t type,
 		if (!s_p_get_string(&p->nodes, "Nodes", tbl)
 		    && !s_p_get_string(&p->nodes, "Nodes", dflt))
 			p->nodes = NULL;
+		else {
+			int i;
+			for (i=0; p->nodes[i]; i++) {
+				if (isspace(p->nodes[i]))
+					p->nodes[i] = ',';
+			}
+		}
 
 		if (!s_p_get_boolean(&p->root_only_flag, "RootOnly", tbl)
 		    && !s_p_get_boolean(&p->root_only_flag, "RootOnly", dflt))
 			p->root_only_flag = false;
 
-		if (!s_p_get_string(&tmp, "Shared", tbl)
-		    && !s_p_get_string(&tmp, "Shared", dflt)) {
-			p->shared = SHARED_NO;
-		} else {
+		if (!s_p_get_uint16(&p->priority, "Priority", tbl) &&
+		    !s_p_get_uint16(&p->priority, "Priority", dflt))
+			p->priority = 1;
+
+		if (s_p_get_string(&tmp, "Shared", tbl) ||
+		    s_p_get_string(&tmp, "Shared", dflt)) {
 			if (strcasecmp(tmp, "NO") == 0)
-				p->shared = SHARED_NO;
+				p->max_share = 1;
 #ifndef HAVE_XCPU
 			/* Only "Shared=NO" is valid on XCPU systems */
-			else if (strcasecmp(tmp, "YES") == 0)
-				p->shared = SHARED_YES;
 			else if (strcasecmp(tmp, "EXCLUSIVE") == 0)
-				p->shared = SHARED_EXCLUSIVE;
-			else if (strcasecmp(tmp, "FORCE") == 0)
-				p->shared = SHARED_FORCE;
+				p->max_share = 0;
+			else if (strncasecmp(tmp, "YES:", 4) == 0)
+				p->max_share = strtol(&tmp[4], (char **) NULL, 10);
+			else if (strcasecmp(tmp, "YES") == 0) 
+				p->max_share = 4;
+			else if (strncasecmp(tmp, "FORCE:", 6) == 0) {
+				p->max_share = strtol(&tmp[6], (char **) NULL, 10) |
+					SHARED_FORCE;
+			} else if (strcasecmp(tmp, "FORCE") == 0)
+				p->max_share = 4 | SHARED_FORCE;
 #endif
 			else {
 				error("Bad value \"%s\" for Shared", tmp);
@@ -483,7 +565,9 @@ static int parse_partitionname(void **dest, slurm_parser_enum_t type,
 				xfree(tmp);
 				return -1;
 			}
-		}
+		} else
+			p->max_share = 1;
+
 		xfree(tmp);
 
 		if (!s_p_get_boolean(&p->state_up_flag, "State", tbl)
@@ -781,14 +865,6 @@ static void _init_slurmd_nodehash(void)
 	}
 }
 
-extern void slurm_conf_nodehash_init(void)
-{
-	slurm_conf_lock();
-	_init_slurmd_nodehash();
-	slurm_conf_unlock();
-}
-
-
 /*
  * Caller needs to call slurm_conf_lock() and hold the lock before
  * calling this function (and call slurm_conf_unlock() afterwards).
@@ -851,6 +927,58 @@ extern char *slurm_conf_get_nodename(const char *node_hostname)
 	return NULL;
 }
 
+/*
+ * slurm_conf_get_aliased_nodename - Return the NodeName for the
+ * complete hostname string returned by gethostname if there is
+ * such a match, otherwise iterate through any aliases returned
+ * by get_host_by_name
+ */
+extern char *slurm_conf_get_aliased_nodename()
+{
+	char hostname_full[1024];
+	int error_code;
+	char *nodename;
+
+	error_code = gethostname(hostname_full, sizeof(hostname_full));
+	/* we shouldn't have any problem here since by the time
+	 * this function has been called, gethostname_short,
+	 * which invokes gethostname, has probably already been called
+	 * successfully, so just return NULL if something weird
+	 * happens at this point
+	 */
+	if (error_code)
+		return NULL;
+
+	nodename = slurm_conf_get_nodename(hostname_full);
+	/* if the full hostname did not match a nodename */
+	if (nodename == NULL) {
+		/* use get_host_by_name; buffer sizes, semantics, etc.
+		 * copied from slurm_protocol_socket_implementation.c
+		 */
+		struct hostent * he = NULL;
+		char * h_buf[4096];
+		int h_err;
+
+		he = get_host_by_name(hostname_full, (void *)&h_buf,
+				      sizeof(h_buf), &h_err);
+		if (he != NULL) {
+			unsigned int i = 0;
+			/* check the "official" host name first */
+			nodename = slurm_conf_get_nodename(he->h_name);
+			while ((nodename == NULL) &&
+			       (he->h_aliases[i] != NULL)) {
+				/* the "official" name still didn't match --
+				 * iterate through the aliases */
+				nodename =
+				     slurm_conf_get_nodename(he->h_aliases[i]);
+				i++;
+			}
+		}
+	}
+
+	return nodename;
+}
+
 /*
  * slurm_conf_get_port - Return the port for a given NodeName
  */
@@ -866,8 +994,8 @@ extern uint16_t slurm_conf_get_port(const char *node_name)
 	p = node_to_host_hashtbl[idx];
 	while (p) {
 		if (strcmp(p->alias, node_name) == 0) {
-			uint16_t port = p->port;
-			if (!port)
+			uint16_t port;
+			if (!p->port)
 				p->port = (uint16_t) conf_ptr->slurmd_port;
 			port = p->port;
 			slurm_conf_unlock();
@@ -989,46 +1117,60 @@ gethostname_short (char *name, size_t len)
 extern void
 free_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr, bool purge_node_hash)
 {
+	xfree (ctl_conf_ptr->accounting_storage_host);
+	xfree (ctl_conf_ptr->accounting_storage_loc);
+	xfree (ctl_conf_ptr->accounting_storage_pass);
+	xfree (ctl_conf_ptr->accounting_storage_type);
+	xfree (ctl_conf_ptr->accounting_storage_user);
 	xfree (ctl_conf_ptr->authtype);
-	xfree (ctl_conf_ptr->checkpoint_type);
 	xfree (ctl_conf_ptr->backup_addr);
 	xfree (ctl_conf_ptr->backup_controller);
+	xfree (ctl_conf_ptr->checkpoint_type);
+	xfree (ctl_conf_ptr->cluster_name);
 	xfree (ctl_conf_ptr->control_addr);
 	xfree (ctl_conf_ptr->control_machine);
+	xfree (ctl_conf_ptr->crypto_type);
 	xfree (ctl_conf_ptr->epilog);
-	xfree (ctl_conf_ptr->job_acct_logfile);
-	xfree (ctl_conf_ptr->job_acct_type);
+	xfree (ctl_conf_ptr->health_check_program);
+	xfree (ctl_conf_ptr->job_acct_gather_type);
+	xfree (ctl_conf_ptr->job_comp_host);
 	xfree (ctl_conf_ptr->job_comp_loc);
+	xfree (ctl_conf_ptr->job_comp_pass);
 	xfree (ctl_conf_ptr->job_comp_type);
+	xfree (ctl_conf_ptr->job_comp_user);
 	xfree (ctl_conf_ptr->job_credential_private_key);
 	xfree (ctl_conf_ptr->job_credential_public_certificate);
+	xfree (ctl_conf_ptr->licenses);
 	xfree (ctl_conf_ptr->mail_prog);
 	xfree (ctl_conf_ptr->mpi_default);
+	xfree (ctl_conf_ptr->node_prefix);
 	xfree (ctl_conf_ptr->plugindir);
 	xfree (ctl_conf_ptr->plugstack);
 	xfree (ctl_conf_ptr->proctrack_type);
 	xfree (ctl_conf_ptr->prolog);
 	xfree (ctl_conf_ptr->propagate_rlimits_except);
 	xfree (ctl_conf_ptr->propagate_rlimits);
+	xfree (ctl_conf_ptr->resume_program);
+	xfree (ctl_conf_ptr->slurm_conf);
 	xfree (ctl_conf_ptr->schedtype);
 	xfree (ctl_conf_ptr->select_type);
-	xfree (ctl_conf_ptr->slurm_conf);
 	xfree (ctl_conf_ptr->slurm_user_name);
 	xfree (ctl_conf_ptr->slurmctld_logfile);
 	xfree (ctl_conf_ptr->slurmctld_pidfile);
 	xfree (ctl_conf_ptr->slurmd_logfile);
 	xfree (ctl_conf_ptr->slurmd_pidfile);
 	xfree (ctl_conf_ptr->slurmd_spooldir);
-	xfree (ctl_conf_ptr->state_save_location);
-	xfree (ctl_conf_ptr->switch_type);
-	xfree (ctl_conf_ptr->tmp_fs);
-	xfree (ctl_conf_ptr->task_epilog);
-	xfree (ctl_conf_ptr->task_prolog);
-	xfree (ctl_conf_ptr->task_plugin);
-	xfree (ctl_conf_ptr->tmp_fs);
-	xfree (ctl_conf_ptr->srun_prolog);
 	xfree (ctl_conf_ptr->srun_epilog);
-	xfree (ctl_conf_ptr->node_prefix);
+	xfree (ctl_conf_ptr->srun_prolog);
+	xfree (ctl_conf_ptr->state_save_location);
+	xfree (ctl_conf_ptr->suspend_exc_nodes);
+	xfree (ctl_conf_ptr->suspend_exc_parts);
+	xfree (ctl_conf_ptr->suspend_program);
+	xfree (ctl_conf_ptr->switch_type);
+	xfree (ctl_conf_ptr->task_epilog);
+	xfree (ctl_conf_ptr->task_plugin);
+	xfree (ctl_conf_ptr->task_prolog);
+	xfree (ctl_conf_ptr->tmp_fs);
 	xfree (ctl_conf_ptr->unkillable_program);
 
 	if (purge_node_hash)
@@ -1045,43 +1187,70 @@ void
 init_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr)
 {
 	ctl_conf_ptr->last_update		= time(NULL);
-	xfree (ctl_conf_ptr->authtype);
 	ctl_conf_ptr->cache_groups		= (uint16_t) NO_VAL;
-	xfree (ctl_conf_ptr->checkpoint_type);
+	xfree (ctl_conf_ptr->accounting_storage_host);
+	xfree (ctl_conf_ptr->accounting_storage_loc);
+	xfree (ctl_conf_ptr->accounting_storage_pass);
+	ctl_conf_ptr->accounting_storage_port             = 0;
+	xfree (ctl_conf_ptr->accounting_storage_type);
+	xfree (ctl_conf_ptr->accounting_storage_user);
+	xfree (ctl_conf_ptr->authtype);
 	xfree (ctl_conf_ptr->backup_addr);
 	xfree (ctl_conf_ptr->backup_controller);
+	ctl_conf_ptr->cache_groups		= 0;
+	xfree (ctl_conf_ptr->checkpoint_type);
+	xfree (ctl_conf_ptr->cluster_name);
 	xfree (ctl_conf_ptr->control_addr);
 	xfree (ctl_conf_ptr->control_machine);
+	xfree (ctl_conf_ptr->crypto_type);
+	ctl_conf_ptr->def_mem_per_task          = 0;
+	ctl_conf_ptr->disable_root_jobs         = 0;
+	ctl_conf_ptr->enforce_part_limits       = 0;
 	xfree (ctl_conf_ptr->epilog);
+	ctl_conf_ptr->epilog_msg_time		= (uint32_t) NO_VAL;
 	ctl_conf_ptr->fast_schedule		= (uint16_t) NO_VAL;
 	ctl_conf_ptr->first_job_id		= (uint32_t) NO_VAL;
+	ctl_conf_ptr->get_env_timeout		= 0;
+	ctl_conf_ptr->health_check_interval	= 0;
+	xfree(ctl_conf_ptr->health_check_program);
 	ctl_conf_ptr->inactive_limit		= (uint16_t) NO_VAL;
-	xfree (ctl_conf_ptr->job_acct_logfile);
-	ctl_conf_ptr->job_acct_freq             = 0;
-	xfree (ctl_conf_ptr->job_acct_type);
+	xfree (ctl_conf_ptr->job_acct_gather_type);
+	ctl_conf_ptr->job_acct_gather_freq             = 0;
 	xfree (ctl_conf_ptr->job_comp_loc);
+	xfree (ctl_conf_ptr->job_comp_pass);
+	ctl_conf_ptr->job_comp_port             = 0;
 	xfree (ctl_conf_ptr->job_comp_type);
+	xfree (ctl_conf_ptr->job_comp_user);
 	xfree (ctl_conf_ptr->job_credential_private_key);
 	xfree (ctl_conf_ptr->job_credential_public_certificate);
 	ctl_conf_ptr->job_file_append		= (uint16_t) NO_VAL;
+	ctl_conf_ptr->job_requeue		= (uint16_t) NO_VAL;
 	ctl_conf_ptr->kill_wait			= (uint16_t) NO_VAL;
+	xfree (ctl_conf_ptr->licenses);
 	xfree (ctl_conf_ptr->mail_prog);
 	ctl_conf_ptr->max_job_cnt		= (uint16_t) NO_VAL;
+	ctl_conf_ptr->max_mem_per_task          = 0;
 	ctl_conf_ptr->min_job_age		= (uint16_t) NO_VAL;
 	xfree (ctl_conf_ptr->mpi_default);
 	ctl_conf_ptr->msg_timeout		= (uint16_t) NO_VAL;
 	ctl_conf_ptr->next_job_id		= (uint32_t) NO_VAL;
+	xfree (ctl_conf_ptr->node_prefix);
 	xfree (ctl_conf_ptr->plugindir);
 	xfree (ctl_conf_ptr->plugstack);
+	ctl_conf_ptr->private_data              = 0;
 	xfree (ctl_conf_ptr->proctrack_type);
 	xfree (ctl_conf_ptr->prolog);
 	ctl_conf_ptr->propagate_prio_process	= (uint16_t) NO_VAL;
-	xfree (ctl_conf_ptr->propagate_rlimits_except);
 	xfree (ctl_conf_ptr->propagate_rlimits);
+	xfree (ctl_conf_ptr->propagate_rlimits_except);
+	xfree (ctl_conf_ptr->resume_program);
+	ctl_conf_ptr->resume_rate		= (uint16_t) NO_VAL;
 	ctl_conf_ptr->ret2service		= (uint16_t) NO_VAL;
+	xfree( ctl_conf_ptr->sched_params );
+	ctl_conf_ptr->sched_time_slice		= (uint16_t) NO_VAL;
+	xfree( ctl_conf_ptr->schedtype );
 	ctl_conf_ptr->schedport			= (uint16_t) NO_VAL;
 	ctl_conf_ptr->schedrootfltr		= (uint16_t) NO_VAL;
-	xfree( ctl_conf_ptr->schedtype );
 	xfree( ctl_conf_ptr->select_type );
 	ctl_conf_ptr->select_type_param         = (uint16_t) NO_VAL;
 	ctl_conf_ptr->slurm_user_id		= (uint16_t) NO_VAL; 
@@ -1097,20 +1266,25 @@ init_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr)
  	ctl_conf_ptr->slurmd_port		= (uint32_t) NO_VAL;
 	xfree (ctl_conf_ptr->slurmd_spooldir);
 	ctl_conf_ptr->slurmd_timeout		= (uint16_t) NO_VAL;
-	xfree (ctl_conf_ptr->state_save_location);
-	xfree (ctl_conf_ptr->switch_type);
-	xfree (ctl_conf_ptr->task_epilog);
-	xfree (ctl_conf_ptr->task_prolog);
-	xfree (ctl_conf_ptr->task_plugin);
-	xfree (ctl_conf_ptr->tmp_fs);
-	ctl_conf_ptr->wait_time			= (uint16_t) NO_VAL;
 	xfree (ctl_conf_ptr->srun_prolog);
 	xfree (ctl_conf_ptr->srun_epilog);
-	xfree (ctl_conf_ptr->node_prefix);
+	xfree (ctl_conf_ptr->state_save_location);
+	xfree (ctl_conf_ptr->suspend_exc_nodes);
+	xfree (ctl_conf_ptr->suspend_exc_parts);
+	xfree (ctl_conf_ptr->suspend_program);
+	ctl_conf_ptr->suspend_rate		= (uint16_t) NO_VAL;
+	ctl_conf_ptr->suspend_time		= (uint16_t) NO_VAL;
+	xfree (ctl_conf_ptr->switch_type);
+	xfree (ctl_conf_ptr->task_epilog);
+	xfree (ctl_conf_ptr->task_plugin);
+	ctl_conf_ptr->task_plugin_param		= 0;
+	xfree (ctl_conf_ptr->task_prolog);
+	xfree (ctl_conf_ptr->tmp_fs);
 	ctl_conf_ptr->tree_width       		= (uint16_t) NO_VAL;
-	ctl_conf_ptr->use_pam			= 0;
 	xfree (ctl_conf_ptr->unkillable_program);
 	ctl_conf_ptr->unkillable_timeout        = (uint16_t) NO_VAL;
+	ctl_conf_ptr->use_pam			= 0;
+	ctl_conf_ptr->wait_time			= (uint16_t) NO_VAL;
 
 	_free_name_hashtbl();
 	_init_name_hashtbl();
@@ -1329,8 +1503,13 @@ static void
 validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 {
 	char *temp_str = NULL;
+	long long_suspend_time;
 	bool truth;
-
+	char *default_storage_type = NULL, *default_storage_host = NULL;
+	char *default_storage_user = NULL, *default_storage_pass = NULL;
+	char *default_storage_loc = NULL;
+	uint32_t default_storage_port = 0;
+		
 	if (s_p_get_string(&conf->backup_controller, "BackupController",
 			   hashtbl)
 	    && strcasecmp("localhost", conf->backup_controller) == 0) {
@@ -1349,6 +1528,8 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 			conf->backup_addr = xstrdup(conf->backup_controller);
 	}
 
+	s_p_get_string(&conf->cluster_name, "ClusterName", hashtbl);
+
 	if (!s_p_get_string(&conf->control_machine, "ControlMachine", hashtbl))
 		fatal ("validate_and_set_defaults: "
 		       "ControlMachine not specified.");
@@ -1370,6 +1551,13 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 		xfree(conf->backup_controller);
 	}
 
+	s_p_get_string(&default_storage_type, "DefaultStorageType", hashtbl);
+	s_p_get_string(&default_storage_host, "DefaultStorageHost", hashtbl);
+	s_p_get_string(&default_storage_user, "DefaultStorageUser", hashtbl);
+	s_p_get_string(&default_storage_pass, "DefaultStoragePass", hashtbl);
+	s_p_get_string(&default_storage_loc,  "DefaultStorageLoc", hashtbl);
+	s_p_get_uint32(&default_storage_port, "DefaultStoragePort", hashtbl);
+
 	if (!s_p_get_string(&conf->job_credential_private_key,
 			    "JobCredentialPrivateKey", hashtbl))
 		fatal("JobCredentialPrivateKey not set");
@@ -1390,8 +1578,28 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_string(&conf->checkpoint_type, "CheckpointType", hashtbl))
 		conf->checkpoint_type = xstrdup(DEFAULT_CHECKPOINT_TYPE);
 
+	if (!s_p_get_string(&conf->crypto_type, "CryptoType", hashtbl))
+		 conf->crypto_type = xstrdup(DEFAULT_CRYPTO_TYPE);
+
+	if ((s_p_get_uint32(&conf->def_mem_per_task, "DefMemPerCPU", hashtbl)) ||
+	    (s_p_get_uint32(&conf->def_mem_per_task, "DefMemPerTask", hashtbl)))
+		conf->def_mem_per_task |= MEM_PER_CPU;
+	else if (!s_p_get_uint32(&conf->def_mem_per_task, "DefMemPerNode", hashtbl))
+		conf->def_mem_per_task = DEFAULT_MEM_PER_CPU;
+
+	if (!s_p_get_boolean((bool *) &conf->disable_root_jobs, 
+			     "DisableRootJobs", hashtbl))
+		conf->disable_root_jobs = DEFAULT_DISABLE_ROOT_JOBS;
+
+	if (!s_p_get_boolean((bool *) &conf->enforce_part_limits, 
+			     "EnforcePartLimits", hashtbl))
+		conf->enforce_part_limits = DEFAULT_ENFORCE_PART_LIMITS;
+
 	s_p_get_string(&conf->epilog, "Epilog", hashtbl);
 
+	if (!s_p_get_uint32(&conf->epilog_msg_time, "EpilogMsgTime", hashtbl))
+		conf->epilog_msg_time = DEFAULT_EPILOG_MSG_TIME;
+
 	if (!s_p_get_uint16(&conf->fast_schedule, "FastSchedule", hashtbl))
 		conf->fast_schedule = DEFAULT_FAST_SCHEDULE;
 
@@ -1414,37 +1622,121 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 		conf->inactive_limit = DEFAULT_INACTIVE_LIMIT;
 	}
 
-	if (!s_p_get_string(&conf->job_acct_logfile, 
-			    "JobAcctLogFile", hashtbl))
-		conf->job_acct_logfile = xstrdup(DEFAULT_JOB_ACCT_LOGFILE);
+	if (!s_p_get_uint16(&conf->job_acct_gather_freq,
+			    "JobAcctGatherFrequency", hashtbl))
+		conf->job_acct_gather_freq = DEFAULT_JOB_ACCT_GATHER_FREQ;
 
-	if (!s_p_get_uint16(&conf->job_acct_freq, "JobAcctFrequency", hashtbl))
-		conf->job_acct_freq = DEFAULT_JOB_ACCT_FREQ;
+	if (s_p_get_string(&conf->job_acct_gather_type,
+			    "JobAcctType", hashtbl)) {
+		fatal("JobAcctType is no longer a valid parameter.\n"
+		      "The job accounting plugin has changed to 2 different "
+		      "plugins one for gathering and one for storing the "
+		      "gathered information.\n"
+		      "Please change this to JobAcctGatherType to "
+		      "correctly work.\n"
+		      "The major 'jobacct' is now 'jobacct_gather' and "
+		      "'jobacct_storage' your declarations will also need "
+		      "to change in your slurm.conf file.\n"
+		      "Refer to the slurm.conf man page or the web "
+		      "documentation for further explanation.");
+	}
+	
+	if(!s_p_get_string(&conf->job_acct_gather_type,
+			   "JobAcctGatherType", hashtbl))
+		conf->job_acct_gather_type =
+			xstrdup(DEFAULT_JOB_ACCT_GATHER_TYPE);
 
-	if (!s_p_get_string(&conf->job_acct_type, "JobAcctType", hashtbl))
-		conf->job_acct_type = xstrdup(DEFAULT_JOB_ACCT_TYPE);
-
-	s_p_get_string(&conf->job_comp_loc, "JobCompLoc", hashtbl);
-
-	if (!s_p_get_string(&conf->job_comp_type, "JobCompType", hashtbl))
-		conf->job_comp_type = xstrdup(DEFAULT_JOB_COMP_TYPE);
+	if (!s_p_get_string(&conf->job_comp_type, "JobCompType", hashtbl)) {
+		if(default_storage_type) {
+			if(!strcasecmp("slurmdbd", default_storage_type)) {
+				error("Can not use the default storage type "
+				      "specified for jobcomp since there is "
+				      "not slurmdbd type.  We are using %s "
+				      "as the type. To disable this message "
+				      "set JobCompType in your slurm.conf",
+				      DEFAULT_JOB_COMP_TYPE);
+				conf->job_comp_type =
+					xstrdup(DEFAULT_JOB_COMP_TYPE);
+			} else 
+				conf->job_comp_type =
+					xstrdup_printf("jobcomp/%s",
+						       default_storage_type);
+		} else
+			conf->job_comp_type = xstrdup(DEFAULT_JOB_COMP_TYPE);
+	}
+	if (!s_p_get_string(&conf->job_comp_loc, "JobCompLoc", hashtbl)) {
+		if(default_storage_loc)
+			conf->job_comp_loc = xstrdup(default_storage_loc);
+		else
+			conf->job_comp_loc = xstrdup(DEFAULT_JOB_COMP_LOC);
+	}
 
+	if (!s_p_get_string(&conf->job_comp_host, "JobCompHost",
+			    hashtbl)) {
+		if(default_storage_host)
+			conf->job_comp_host =
+				xstrdup(default_storage_host);
+		else
+			conf->job_comp_host = xstrdup(DEFAULT_STORAGE_HOST);
+	}
+	if (!s_p_get_string(&conf->job_comp_user, "JobCompUser",
+			    hashtbl)) {
+		if(default_storage_user)
+			conf->job_comp_user =
+				xstrdup(default_storage_user);
+		else
+			conf->job_comp_user = xstrdup(DEFAULT_STORAGE_USER);
+	}
+	if (!s_p_get_string(&conf->job_comp_pass, "JobCompPass",
+			    hashtbl)) {
+		if(default_storage_pass)
+			conf->job_comp_pass =
+				xstrdup(default_storage_pass);
+	}
+	if (!s_p_get_uint32(&conf->job_comp_port, "JobCompPort",
+			    hashtbl)) {
+		if(default_storage_port)
+			conf->job_comp_port = default_storage_port;
+		else
+			conf->job_comp_port = DEFAULT_STORAGE_PORT;
+	}
 	if (!s_p_get_uint16(&conf->job_file_append, "JobFileAppend", hashtbl))
 		conf->job_file_append = 0;
 
+	if (!s_p_get_uint16(&conf->job_requeue, "JobRequeue", hashtbl))
+		conf->job_requeue = 1;
+	else if (conf->job_requeue > 1)
+		conf->job_requeue = 1;
+
+	if (!s_p_get_uint16(&conf->get_env_timeout, "GetEnvTimeout", hashtbl))
+		conf->get_env_timeout = DEFAULT_GET_ENV_TIMEOUT;
+
+	s_p_get_uint16(&conf->health_check_interval, "HealthCheckInterval", hashtbl);
+	s_p_get_string(&conf->health_check_program, "HealthCheckProgram", hashtbl);
+
 	if (!s_p_get_uint16(&conf->kill_wait, "KillWait", hashtbl))
 		conf->kill_wait = DEFAULT_KILL_WAIT;
 
+	s_p_get_string(&conf->licenses, "Licenses", hashtbl);
+
 	if (!s_p_get_string(&conf->mail_prog, "MailProg", hashtbl))
 		conf->mail_prog = xstrdup(DEFAULT_MAIL_PROG);
 
 	if (!s_p_get_uint16(&conf->max_job_cnt, "MaxJobCount", hashtbl))
 		conf->max_job_cnt = DEFAULT_MAX_JOB_COUNT;
 
+	if ((s_p_get_uint32(&conf->max_mem_per_task, "MaxMemPerCPU", hashtbl)) ||
+	    (s_p_get_uint32(&conf->max_mem_per_task, "MaxMemPerTask", hashtbl)))
+		conf->max_mem_per_task |= MEM_PER_CPU;
+	else if (!s_p_get_uint32(&conf->max_mem_per_task, "MaxMemPerNode", hashtbl))
+		conf->max_mem_per_task = DEFAULT_MAX_MEM_PER_CPU;
+
 	if (!s_p_get_uint16(&conf->msg_timeout, "MessageTimeout", hashtbl))
 		conf->msg_timeout = DEFAULT_MSG_TIMEOUT;
-	else if (conf->msg_timeout > 100)
-		info("WARNING: MessageTimeout is too high for effective fault-tolerance");
+	else if (conf->msg_timeout > 100) {
+		info("WARNING: MessageTimeout is too high for effective "
+			"fault-tolerance");
+	}
 
 	if (!s_p_get_uint16(&conf->min_job_age, "MinJobAge", hashtbl))
 		conf->min_job_age = DEFAULT_MIN_JOB_AGE;
@@ -1452,6 +1744,69 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_string(&conf->mpi_default, "MpiDefault", hashtbl))
 		conf->mpi_default = xstrdup(DEFAULT_MPI_DEFAULT);
 
+	if (!s_p_get_string(&conf->accounting_storage_type,
+			    "AccountingStorageType", hashtbl)) {
+		if(default_storage_type)
+			conf->accounting_storage_type =
+				xstrdup_printf("accounting_storage/%s",
+					       default_storage_type);
+		else	
+			conf->accounting_storage_type =
+				xstrdup(DEFAULT_ACCOUNTING_STORAGE_TYPE);
+	}
+
+	if (!s_p_get_uint16(&conf->accounting_storage_enforce, 
+			    "AccountingStorageEnforce", hashtbl))
+		conf->accounting_storage_enforce = DEFAULT_ACCOUNTING_ENFORCE;
+
+	if (!s_p_get_string(&conf->accounting_storage_host,
+			    "AccountingStorageHost", hashtbl)) {
+		if(default_storage_host)
+			conf->accounting_storage_host =
+				xstrdup(default_storage_host);
+		else
+			conf->accounting_storage_host =
+				xstrdup(DEFAULT_STORAGE_HOST);
+	}
+
+	/* AccountingStorageLoc replaces JobAcctLogFile since it now represents
+	 * the database name also depending on the storage type you
+	 * use so we still check JobAcctLogFile for the same thing
+	 */
+	if (!s_p_get_string(&conf->accounting_storage_loc,
+			    "AccountingStorageLoc", hashtbl)
+		&& !s_p_get_string(&conf->accounting_storage_loc,
+			       "JobAcctLogFile", hashtbl)) {
+		if(default_storage_loc)
+			conf->accounting_storage_loc =
+				xstrdup(default_storage_loc);
+		else
+			conf->accounting_storage_loc =
+				xstrdup(DEFAULT_STORAGE_LOC);
+	}
+
+	if (!s_p_get_string(&conf->accounting_storage_user,
+			    "AccountingStorageUser", hashtbl)) {
+		if(default_storage_user)
+			conf->accounting_storage_user =
+				xstrdup(default_storage_user);
+		else
+			conf->accounting_storage_user =
+				xstrdup(DEFAULT_STORAGE_USER);
+	}
+	if (!s_p_get_string(&conf->accounting_storage_pass,
+			    "AccountingStoragePass", hashtbl)) {
+		if(default_storage_pass)
+			conf->accounting_storage_pass =
+				xstrdup(default_storage_pass);
+	}
+	if (!s_p_get_uint32(&conf->accounting_storage_port,
+			    "AccountingStoragePort", hashtbl)) {
+		if(default_storage_port)
+			conf->accounting_storage_port = default_storage_port;
+		else
+			conf->accounting_storage_port = DEFAULT_STORAGE_PORT;
+	}
 	if (!s_p_get_string(&conf->plugindir, "PluginDir", hashtbl))
 		conf->plugindir = xstrdup(default_plugin_path);
 
@@ -1472,6 +1827,18 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	    && (!strcmp(conf->proctrack_type,"proctrack/linuxproc")))
 		fatal("proctrack/linuxproc is incompatable with switch/elan");
 
+	if (s_p_get_string(&temp_str, "PrivateData", hashtbl)) {
+		if (strstr(temp_str, "job"))
+			conf->private_data |= PRIVATE_DATA_JOBS;
+		if (strstr(temp_str, "node"))
+			conf->private_data |= PRIVATE_DATA_NODES;
+		if (strstr(temp_str, "partition"))
+			conf->private_data |= PRIVATE_DATA_PARTITIONS;
+		if (strstr(temp_str, "all"))
+			conf->private_data = 0xffff;
+		xfree(temp_str);
+	}
+
 	s_p_get_string(&conf->prolog, "Prolog", hashtbl);
 
 	if (!s_p_get_uint16(&conf->propagate_prio_process,
@@ -1501,6 +1868,12 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_uint16(&conf->ret2service, "ReturnToService", hashtbl))
 		conf->ret2service = DEFAULT_RETURN_TO_SERVICE;
 
+	s_p_get_string(&conf->resume_program, "ResumeProgram", hashtbl);
+	if (!s_p_get_uint16(&conf->resume_rate, "ResumeRate", hashtbl))
+		conf->resume_rate = DEFAULT_RESUME_RATE;
+
+	s_p_get_string(&conf->sched_params, "SchedulerParameters", hashtbl);
+
 	if (s_p_get_uint16(&conf->schedport, "SchedulerPort", hashtbl)) {
 		if (conf->schedport == 0) {
 			error("SchedulerPort=0 is invalid");
@@ -1514,6 +1887,10 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 			    "SchedulerRootFilter", hashtbl))
 		conf->schedrootfltr = DEFAULT_SCHEDROOTFILTER;
 
+	if (!s_p_get_uint16(&conf->sched_time_slice, "SchedulerTimeSlice",
+	    hashtbl))
+		conf->sched_time_slice = DEFAULT_SCHED_TIME_SLICE;
+
 	if (!s_p_get_string(&conf->schedtype, "SchedulerType", hashtbl))
 		conf->schedtype = xstrdup(DEFAULT_SCHEDTYPE);
 
@@ -1548,11 +1925,7 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 			       conf->slurm_user_name);
 			xfree(conf->slurm_user_name);
 		} else {
-			if (slurm_passwd->pw_uid > 0xffff)
-				error("SlurmUser numeric overflow, "
-				      "will be fixed soon");
-			else
-				conf->slurm_user_id = slurm_passwd->pw_uid;
+			conf->slurm_user_id = slurm_passwd->pw_uid;
 		}
 	}
 
@@ -1600,6 +1973,16 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 			    "StateSaveLocation", hashtbl))
 		conf->state_save_location = xstrdup(DEFAULT_SAVE_STATE_LOC);
 
+	s_p_get_string(&conf->suspend_exc_nodes, "SuspendExcNodes", hashtbl);
+	s_p_get_string(&conf->suspend_exc_parts, "SuspendExcParts", hashtbl);
+	s_p_get_string(&conf->suspend_program, "SuspendProgram", hashtbl);
+	if (!s_p_get_uint16(&conf->suspend_rate, "SuspendRate", hashtbl))
+		conf->suspend_rate = DEFAULT_SUSPEND_RATE;
+	if (s_p_get_long(&long_suspend_time, "SuspendTime", hashtbl))
+		conf->suspend_time = long_suspend_time + 1;
+	else
+		conf->suspend_time = 0;
+
 	/* see above for switch_type, order dependent */
 
 	if (!s_p_get_string(&conf->task_plugin, "TaskPlugin", hashtbl))
@@ -1646,6 +2029,11 @@ validate_and_set_defaults(slurm_ctl_conf_t *conf, s_p_hashtbl_t *hashtbl)
 	if (!s_p_get_uint16(&conf->unkillable_timeout,
 			    "UnkillableStepTimeout", hashtbl))
 		conf->unkillable_timeout = DEFAULT_UNKILLABLE_TIMEOUT;
+	xfree(default_storage_type);
+	xfree(default_storage_loc);
+	xfree(default_storage_host);
+	xfree(default_storage_user);
+	xfree(default_storage_pass);
 }
 
 /*
