@@ -1,12 +1,12 @@
 /*****************************************************************************\
  *  slurm_protocol_socket_implementation.c - slurm communications interfaces 
- *                                           based upon sockets.
- *  $Id: slurm_protocol_socket_implementation.c 12005 2007-08-14 17:20:11Z jette $
+ *					     based upon sockets.
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Kevin Tew <tew1@llnl.gov>, et. al.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -81,7 +81,7 @@
 #define MIN_USER_PORT   (IPPORT_RESERVED + 1)
 #define MAX_USER_PORT   0xffff
 #define RANDOM_USER_PORT ((uint16_t) ((lrand48() % \
-                (MAX_USER_PORT - MIN_USER_PORT + 1)) + MIN_USER_PORT))
+		(MAX_USER_PORT - MIN_USER_PORT + 1)) + MIN_USER_PORT))
 
 /*
  *  Maximum message size. Messages larger than this value (in bytes)
@@ -98,28 +98,28 @@
  */
 static int _tot_wait (struct timeval *start_time)
 {
-        struct timeval end_time;
-        int msec_delay;
+	struct timeval end_time;
+	int msec_delay;
 
-        gettimeofday(&end_time, NULL);
-        msec_delay =   (end_time.tv_sec  - start_time->tv_sec ) * 1000;
-        msec_delay += ((end_time.tv_usec - start_time->tv_usec + 500) / 1000);
-        return msec_delay;
+	gettimeofday(&end_time, NULL);
+	msec_delay =   (end_time.tv_sec  - start_time->tv_sec ) * 1000;
+	msec_delay += ((end_time.tv_usec - start_time->tv_usec + 500) / 1000);
+	return msec_delay;
 }
 
 slurm_fd _slurm_init_msg_engine ( slurm_addr * slurm_address )
 {
-        return _slurm_listen_stream ( slurm_address ) ;
+	return _slurm_listen_stream ( slurm_address ) ;
 }
 
 slurm_fd _slurm_open_msg_conn ( slurm_addr * slurm_address )
 {
-        return _slurm_open_stream ( slurm_address, false ) ;
+	return _slurm_open_stream ( slurm_address, false ) ;
 }
 
 slurm_fd _slurm_accept_msg_conn (slurm_fd fd, slurm_addr *addr)
 {
-        return _slurm_accept_stream(fd, addr);
+	return _slurm_accept_stream(fd, addr);
 } 
 
 /*
@@ -130,27 +130,27 @@ slurm_fd _slurm_accept_msg_conn (slurm_fd fd, slurm_addr *addr)
  */
 static void _sock_bind_wild(int sockfd)
 {
-        int rc, retry;
-        slurm_addr sin;
-        static bool seeded = false;
+	int rc, retry;
+	slurm_addr sin;
+	static bool seeded = false;
 
-        if (!seeded) {
-                seeded = true;
-                srand48((long int) (time(NULL) + getpid()));
-        }
+	if (!seeded) {
+		seeded = true;
+		srand48((long int) (time(NULL) + getpid()));
+	}
 
-        memset(&sin, 0, sizeof(sin));
-        sin.sin_family = AF_INET;
-        sin.sin_addr.s_addr = htonl(INADDR_ANY);
-        sin.sin_port = htons(RANDOM_USER_PORT);
+	memset(&sin, 0, sizeof(sin));
+	sin.sin_family = AF_INET;
+	sin.sin_addr.s_addr = htonl(INADDR_ANY);
+	sin.sin_port = htons(RANDOM_USER_PORT);
 
-        for (retry=0; retry < PORT_RETRIES ; retry++) {
-                rc = bind(sockfd, (struct sockaddr *) &sin, sizeof(sin));
-                if (rc >= 0)
-                        break;
-                sin.sin_port  = htons(RANDOM_USER_PORT);
-        }
-        return;
+	for (retry=0; retry < PORT_RETRIES ; retry++) {
+		rc = bind(sockfd, (struct sockaddr *) &sin, sizeof(sin));
+		if (rc >= 0)
+			break;
+		sin.sin_port  = htons(RANDOM_USER_PORT);
+	}
+	return;
 }
        
 /* 
@@ -158,70 +158,70 @@ static void _sock_bind_wild(int sockfd)
  */
 int _slurm_close_accepted_conn (slurm_fd fd) 
 {
-        return _slurm_close (fd);
+	return _slurm_close (fd);
 }
 
 ssize_t _slurm_msg_recvfrom(slurm_fd fd, char **pbuf, size_t *lenp, 
-                            uint32_t flags)
+			    uint32_t flags)
 {
-        return _slurm_msg_recvfrom_timeout(fd, pbuf, lenp, flags, 
-                                (slurm_get_msg_timeout() * 1000));
+	return _slurm_msg_recvfrom_timeout(fd, pbuf, lenp, flags, 
+				(slurm_get_msg_timeout() * 1000));
 }
 
 ssize_t _slurm_msg_recvfrom_timeout(slurm_fd fd, char **pbuf, size_t *lenp, 
-                                    uint32_t flags, int tmout)
+				    uint32_t flags, int tmout)
 {
-        ssize_t  len;
-        uint32_t msglen;
+	ssize_t  len;
+	uint32_t msglen;
 
-        len = _slurm_recv_timeout( fd, (char *)&msglen, 
-                                   sizeof(msglen), 0, tmout );
+	len = _slurm_recv_timeout( fd, (char *)&msglen, 
+				   sizeof(msglen), 0, tmout );
 
-        if (len < ((ssize_t) sizeof(msglen))) 
-                return SLURM_ERROR;
+	if (len < ((ssize_t) sizeof(msglen))) 
+		return SLURM_ERROR;
 
-        msglen = ntohl(msglen);
+	msglen = ntohl(msglen);
 	
-        if (msglen > MAX_MSG_SIZE) 
-                slurm_seterrno_ret(SLURM_PROTOCOL_INSANE_MSG_LENGTH);
+	if (msglen > MAX_MSG_SIZE) 
+		slurm_seterrno_ret(SLURM_PROTOCOL_INSANE_MSG_LENGTH);
 
-        /*
-         *  Allocate memory on heap for message
-         */
-        *pbuf = xmalloc(msglen);
+	/*
+	 *  Allocate memory on heap for message
+	 */
+	*pbuf = xmalloc(msglen);
 
-        if (_slurm_recv_timeout(fd, *pbuf, msglen, 0, tmout) != msglen) {
-                xfree(*pbuf);
-                *pbuf = NULL;
-                return SLURM_ERROR;
-        }
+	if (_slurm_recv_timeout(fd, *pbuf, msglen, 0, tmout) != msglen) {
+		xfree(*pbuf);
+		*pbuf = NULL;
+		return SLURM_ERROR;
+	}
 
-        *lenp = msglen;
-        
-        return (ssize_t) msglen;
+	*lenp = msglen;
+	
+	return (ssize_t) msglen;
 }
 
 ssize_t _slurm_msg_sendto(slurm_fd fd, char *buffer, size_t size, 
-                          uint32_t flags)
+			  uint32_t flags)
 {
-        return _slurm_msg_sendto_timeout( fd, buffer, size, flags, 
-                                (slurm_get_msg_timeout() * 1000));
+	return _slurm_msg_sendto_timeout( fd, buffer, size, flags, 
+				(slurm_get_msg_timeout() * 1000));
 }
 
 ssize_t _slurm_msg_sendto_timeout(slurm_fd fd, char *buffer, size_t size, 
-                                  uint32_t flags, int timeout)
+				  uint32_t flags, int timeout)
 {
-        int   len;
-        uint32_t usize;
-        SigFunc *ohandler;
+	int   len;
+	uint32_t usize;
+	SigFunc *ohandler;
 
-        /* 
-         *  Ignore SIGPIPE so that send can return a error code if the 
-         *    other side closes the socket 
-         */
-        ohandler = xsignal(SIGPIPE, SIG_IGN);
+	/* 
+	 *  Ignore SIGPIPE so that send can return a error code if the 
+	 *    other side closes the socket 
+	 */
+	ohandler = xsignal(SIGPIPE, SIG_IGN);
 
-        usize = htonl(size);
+	usize = htonl(size);
 
 	if ((len = _slurm_send_timeout( 
 				fd, (char *)&usize, sizeof(usize), 0, 
@@ -233,41 +233,42 @@ ssize_t _slurm_msg_sendto_timeout(slurm_fd fd, char *buffer, size_t size,
 
 
      done:
-        xsignal(SIGPIPE, ohandler);
-        return len;
+	xsignal(SIGPIPE, ohandler);
+	return len;
 }
 
 /* Send slurm message with timeout
  * RET message size (as specified in argument) or SLURM_ERROR on error */
 int _slurm_send_timeout(slurm_fd fd, char *buf, size_t size, 
-                        uint32_t flags, int timeout)
+			uint32_t flags, int timeout)
 {
-        int rc;
-        int sent = 0;
-        int fd_flags;
-        struct pollfd ufds;
-        struct timeval tstart;
-        int timeleft = timeout;
+	int rc;
+	int sent = 0;
+	int fd_flags;
+	struct pollfd ufds;
+	struct timeval tstart;
+	int timeleft = timeout;
 	char temp[2];
-        ufds.fd     = fd;
-        ufds.events = POLLOUT;
 
-        fd_flags = _slurm_fcntl(fd, F_GETFL);
-        fd_set_nonblocking(fd);
+	ufds.fd     = fd;
+	ufds.events = POLLOUT;
 
-        gettimeofday(&tstart, NULL);
+	fd_flags = _slurm_fcntl(fd, F_GETFL);
+	fd_set_nonblocking(fd);
 
-        while (sent < size) {
+	gettimeofday(&tstart, NULL);
 
-                timeleft = timeout - _tot_wait(&tstart);
-                if (timeleft <= 0) {
+	while (sent < size) {
+		timeleft = timeout - _tot_wait(&tstart);
+		if (timeleft <= 0) {
 			debug("_slurm_send_timeout at %d of %d, timeout",
 				sent, size);
 			slurm_seterrno(SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT);
 			sent = SLURM_ERROR;
 			goto done;
-                }
-                if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
+		}
+
+		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
 			if ((rc == 0) || (errno == EINTR) || (errno == EAGAIN)) 
  				continue;
 			else {
@@ -278,7 +279,7 @@ int _slurm_send_timeout(slurm_fd fd, char *buf, size_t size,
 				sent = SLURM_ERROR;
 				goto done;
 			}
-                }
+		}
 
 		/*
 		 * Check here to make sure the socket really is there.
@@ -287,16 +288,26 @@ int _slurm_send_timeout(slurm_fd fd, char *buf, size_t size,
 		 * socket is gone, but getting 0 back from a
 		 * nonblocking read means just that. 
 		 */
-		rc = _slurm_recv(fd, &temp, 1, flags);
-		if (rc == 0) {
-			debug2("_slurm_send_timeout: Socket no longer there.");
+		if (ufds.revents & POLLERR) {
+			debug("_slurm_send_timeout: Socket POLLERR");
 			slurm_seterrno(ENOTCONN);
 			sent = SLURM_ERROR;
 			goto done;			
 		}
-
-                rc = _slurm_send(fd, &buf[sent], (size - sent), flags);
-                if (rc < 0) {
+		if ((ufds.revents & POLLHUP) || (ufds.revents & POLLNVAL) ||
+		    (_slurm_recv(fd, &temp, 1, flags) == 0)) {
+			debug2("_slurm_send_timeout: Socket no longer there");
+			slurm_seterrno(ENOTCONN);
+			sent = SLURM_ERROR;
+			goto done;			
+		}
+		if ((ufds.revents & POLLOUT) != POLLOUT) {
+			error("_slurm_send_timeout: Poll failure, revents:%d",
+			      ufds.revents);
+		}
+		
+		rc = _slurm_send(fd, &buf[sent], (size - sent), flags);
+		if (rc < 0) {
  			if (errno == EINTR)
 				continue;
 			else {
@@ -307,17 +318,17 @@ int _slurm_send_timeout(slurm_fd fd, char *buf, size_t size,
 				sent = SLURM_ERROR;
 				goto done;
 			}
-                }
-                if (rc == 0) {
+		}
+		if (rc == 0) {
 			debug("_slurm_send_timeout at %d of %d, "
 				"sent zero bytes", sent, size);
-                        slurm_seterrno(SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT);
-                        sent = SLURM_ERROR;
-                        goto done;
-                }
+			slurm_seterrno(SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT);
+			sent = SLURM_ERROR;
+			goto done;
+		}
 
-                sent += rc;
-        }
+		sent += rc;
+	}
 
     done:
 	/* Reset fd flags to prior state, preserve errno */
@@ -327,45 +338,44 @@ int _slurm_send_timeout(slurm_fd fd, char *buf, size_t size,
 		slurm_seterrno(slurm_err);
 	}
 
-        return sent;
-        
+	return sent;
+	
 }
 
 /* Get slurm message with timeout
  * RET message size (as specified in argument) or SLURM_ERROR on error */
 int _slurm_recv_timeout(slurm_fd fd, char *buffer, size_t size, 
-                        uint32_t flags, int timeout )
+			uint32_t flags, int timeout )
 {
-        int rc;
-        int recvlen = 0;
-        int fd_flags;
-        struct pollfd  ufds;
-        struct timeval tstart;
+	int rc;
+	int recvlen = 0;
+	int fd_flags;
+	struct pollfd  ufds;
+	struct timeval tstart;
 	int timeleft = timeout;
 
-        ufds.fd     = fd;
-        ufds.events = POLLIN;
+	ufds.fd     = fd;
+	ufds.events = POLLIN;
 
-        fd_flags = _slurm_fcntl(fd, F_GETFL);
-        fd_set_nonblocking(fd);
+	fd_flags = _slurm_fcntl(fd, F_GETFL);
+	fd_set_nonblocking(fd);
 
-        gettimeofday(&tstart, NULL);
+	gettimeofday(&tstart, NULL);
 
-        while (recvlen < size) {
-
-                timeleft = timeout - _tot_wait(&tstart);
-                if (timeleft <= 0) {
+	while (recvlen < size) {
+		timeleft = timeout - _tot_wait(&tstart);
+		if (timeleft <= 0) {
 			debug("_slurm_recv_timeout at %d of %d, timeout",
 				recvlen, size);
-                        slurm_seterrno(SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT);
-                        recvlen = SLURM_ERROR;
-                        goto done;
-                }
+			slurm_seterrno(SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT);
+			recvlen = SLURM_ERROR;
+			goto done;
+		}
 
-                if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
-                        if ((errno == EINTR) || (errno == EAGAIN) || (rc == 0))
-                                continue;
-                        else {
+		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
+			if ((errno == EINTR) || (errno == EAGAIN) || (rc == 0))
+				continue;
+			else {
 				debug("_slurm_recv_timeout at %d of %d, "
 					"poll error: %s",
 					recvlen, size, strerror(errno));
@@ -373,13 +383,32 @@ int _slurm_recv_timeout(slurm_fd fd, char *buffer, size_t size,
 					SLURM_COMMUNICATIONS_RECEIVE_ERROR);
  				recvlen = SLURM_ERROR; 
   				goto done;
-                        }
-                } 
-                rc = _slurm_recv(fd, &buffer[recvlen], (size - recvlen), flags);
-                if (rc < 0)  {
-                        if (errno == EINTR)
-                                continue;
-                        else {
+			}
+		}
+
+		if (ufds.revents & POLLERR) {
+			debug("_slurm_recv_timeout: Socket POLLERR");
+			slurm_seterrno(ENOTCONN);
+			recvlen = SLURM_ERROR;
+			goto done;			
+		}
+		if ((ufds.revents & POLLHUP) || (ufds.revents & POLLNVAL)) {
+			debug2("_slurm_recv_timeout: Socket no longer there");
+			slurm_seterrno(ENOTCONN);
+			recvlen = SLURM_ERROR;
+			goto done;			
+		}
+		if ((ufds.revents & POLLIN) != POLLIN) {
+			error("_slurm_recv_timeout: Poll failure, revents:%d",
+			      ufds.revents);
+			continue;
+		}
+
+		rc = _slurm_recv(fd, &buffer[recvlen], (size - recvlen), flags);
+		if (rc < 0)  {
+			if (errno == EINTR)
+				continue;
+			else {
 				debug("_slurm_recv_timeout at %d of %d, "
 					"recv error: %s",
 					recvlen, size, strerror(errno));
@@ -387,17 +416,17 @@ int _slurm_recv_timeout(slurm_fd fd, char *buffer, size_t size,
 					SLURM_COMMUNICATIONS_RECEIVE_ERROR);
 				recvlen = SLURM_ERROR; 
 				goto done;
-                        }
-                }
-                if (rc == 0) {
+			}
+		}
+		if (rc == 0) {
 			debug("_slurm_recv_timeout at %d of %d, "
 				"recv zero bytes", recvlen, size);
 			slurm_seterrno(SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT);
 			recvlen = SLURM_ERROR;
 			goto done;
-                }
-                recvlen += rc;
-        }
+		}
+		recvlen += rc;
+	}
 
 
     done:
@@ -408,147 +437,150 @@ int _slurm_recv_timeout(slurm_fd fd, char *buffer, size_t size,
 		slurm_seterrno(slurm_err);
 	}
 
-        return recvlen;
+	return recvlen;
 }
 
 int _slurm_shutdown_msg_engine ( slurm_fd open_fd )
 {
-        return _slurm_close ( open_fd ) ;
+	return _slurm_close ( open_fd ) ;
 }
 
 slurm_fd _slurm_listen_stream(slurm_addr *addr)
 {
-        int rc;
-        slurm_fd fd;
-        const int one = 1;
-        const size_t sz1 = sizeof(one);
+	int rc;
+	slurm_fd fd;
+	const int one = 1;
+	const size_t sz1 = sizeof(one);
 
-        if ((fd = _slurm_create_socket(SLURM_STREAM)) < 0) {
+	if ((fd = _slurm_create_socket(SLURM_STREAM)) < 0) {
 		error("Error creating slurm stream socket: %m");
-                return fd;
-        }
+		return fd;
+	}
 
-        rc = _slurm_setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sz1);
-        if (rc < 0) {
+	rc = _slurm_setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sz1);
+	if (rc < 0) {
 		error("setsockopt SO_REUSEADDR failed: %m");
-                goto error; 
-        }
+		goto error; 
+	}
 
-        rc = _slurm_bind(fd, (struct sockaddr const *) addr, sizeof(*addr));
-        if (rc < 0) {
+	rc = _slurm_bind(fd, (struct sockaddr const *) addr, sizeof(*addr));
+	if (rc < 0) {
 		error("Error binding slurm stream socket: %m");
-                goto error; 
-        }
+		goto error; 
+	}
 
-        if (_slurm_listen(fd, SLURM_PROTOCOL_DEFAULT_LISTEN_BACKLOG) < 0) {
+	if (_slurm_listen(fd, SLURM_PROTOCOL_DEFAULT_LISTEN_BACKLOG) < 0) {
 		error( "Error listening on slurm stream socket: %m" ) ;
-                rc = SLURM_ERROR;
-                goto error; 
-        }
-        
-        return fd;
+		rc = SLURM_ERROR;
+		goto error; 
+	}
+	
+	return fd;
 
     error:
 	if ((_slurm_close_stream(fd) < 0) && (errno == EINTR))
 		_slurm_close_stream(fd);	/* try again */
-        return rc;
-        
+	return rc;
+	
 }
 
 slurm_fd _slurm_accept_stream(slurm_fd fd, slurm_addr *addr)
 {
-        socklen_t len = sizeof(slurm_addr);
-        return _slurm_accept(fd, (struct sockaddr *)addr, &len);
+	socklen_t len = sizeof(slurm_addr);
+	return _slurm_accept(fd, (struct sockaddr *)addr, &len);
 }
 
 slurm_fd _slurm_open_stream(slurm_addr *addr, bool retry)
 {
-        int retry_cnt;
-        slurm_fd fd;
+	int retry_cnt;
+	slurm_fd fd;
 
-        if ( (addr->sin_family == 0) || (addr->sin_port  == 0) ) 
-                return SLURM_SOCKET_ERROR;
-
-        for (retry_cnt=0; ; retry_cnt++) {
-                int rc;
-                if ((fd =_slurm_create_socket(SLURM_STREAM)) < 0) {
-        		error("Error creating slurm stream socket: %m");
-                        slurm_seterrno(errno);
-                        return SLURM_SOCKET_ERROR;
-                }
-
-                if (retry_cnt) {
-                        if (retry_cnt == 1)
-                                debug3("Error connecting, picking new stream port");
-                        _sock_bind_wild(fd);
-                }
-
-                rc = _slurm_connect(fd, (struct sockaddr const *)addr, sizeof(*addr));
-                if (rc >= 0)                    /* success */
-                        break;
-                if ((errno != ECONNREFUSED) || 
-                    (!retry) || (retry_cnt >= PORT_RETRIES)) {
-                        slurm_seterrno(errno);
-                        goto error;
-                }
-
-                if ((_slurm_close_stream(fd) < 0) && (errno == EINTR))
-                        _slurm_close_stream(fd);        /* try again */
+	if ( (addr->sin_family == 0) || (addr->sin_port  == 0) ) {
+		error("Error connecting, bad data: family = %u, port = %u",
+			addr->sin_family, addr->sin_port);
+		return SLURM_SOCKET_ERROR;
 	}
 
-        return fd;
+	for (retry_cnt=0; ; retry_cnt++) {
+		int rc;
+		if ((fd =_slurm_create_socket(SLURM_STREAM)) < 0) {
+			error("Error creating slurm stream socket: %m");
+			slurm_seterrno(errno);
+			return SLURM_SOCKET_ERROR;
+		}
+
+		if (retry_cnt) {
+			if (retry_cnt == 1)
+				debug3("Error connecting, picking new stream port");
+			_sock_bind_wild(fd);
+		}
+
+		rc = _slurm_connect(fd, (struct sockaddr const *)addr, sizeof(*addr));
+		if (rc >= 0)		    /* success */
+			break;
+		if ((errno != ECONNREFUSED) || 
+		    (!retry) || (retry_cnt >= PORT_RETRIES)) {
+			slurm_seterrno(errno);
+			goto error;
+		}
+
+		if ((_slurm_close_stream(fd) < 0) && (errno == EINTR))
+			_slurm_close_stream(fd);	/* try again */
+	}
+
+	return fd;
 
     error:
-        debug2("Error connecting slurm stream socket: %m");
+	debug2("Error connecting slurm stream socket: %m");
 	if ((_slurm_close_stream(fd) < 0) && (errno == EINTR))
 		_slurm_close_stream(fd);	/* try again */
-        return SLURM_SOCKET_ERROR;
+	return SLURM_SOCKET_ERROR;
 }
 
 int _slurm_get_stream_addr(slurm_fd fd, slurm_addr *addr )
 {
-        socklen_t size = sizeof(addr);
-        return _slurm_getsockname(fd, (struct sockaddr *)addr, &size);
+	socklen_t size = sizeof(addr);
+	return _slurm_getsockname(fd, (struct sockaddr *)addr, &size);
 }
 
 int _slurm_close_stream ( slurm_fd open_fd )
 {
-        return _slurm_close ( open_fd ) ;
+	return _slurm_close ( open_fd ) ;
 }
 
 
 int _slurm_set_stream_non_blocking(slurm_fd fd)
 {
-        fd_set_nonblocking(fd);
-        return SLURM_SUCCESS;
+	fd_set_nonblocking(fd);
+	return SLURM_SUCCESS;
 }
 
 int _slurm_set_stream_blocking(slurm_fd fd) 
 {
-        fd_set_blocking(fd);
-        return SLURM_SUCCESS;
+	fd_set_blocking(fd);
+	return SLURM_SUCCESS;
 }
 
 extern int _slurm_socket (int __domain, int __type, int __protocol)
 {
-        return socket ( __domain, __type, __protocol ) ;
-}        
+	return socket ( __domain, __type, __protocol ) ;
+}	
 
 extern slurm_fd _slurm_create_socket ( slurm_socket_type_t type )
 {
-        switch ( type )
-        {
-                case SLURM_STREAM :
-                        return _slurm_socket ( AF_INET, SOCK_STREAM, 
-                                                IPPROTO_TCP) ;
-                        break;
-                case SLURM_MESSAGE :
-                        return _slurm_socket ( AF_INET, SOCK_DGRAM, 
-                                                IPPROTO_UDP ) ;
-                        break;
-                default :
-                        return SLURM_SOCKET_ERROR;
-        }
+	switch ( type )
+	{
+		case SLURM_STREAM :
+			return _slurm_socket ( AF_INET, SOCK_STREAM, 
+						IPPROTO_TCP) ;
+			break;
+		case SLURM_MESSAGE :
+			return _slurm_socket ( AF_INET, SOCK_DGRAM, 
+						IPPROTO_UDP ) ;
+			break;
+		default :
+			return SLURM_SOCKET_ERROR;
+	}
 }
 
 /* Create two new sockets, of type TYPE in domain DOMAIN and using
@@ -556,23 +588,23 @@ extern slurm_fd _slurm_create_socket ( slurm_socket_type_t type )
  * descriptors for them in FDS[0] and FDS[1].  If PROTOCOL is zero,
  * one will be chosen automatically.  Returns 0 on success, -1 for errors.  */
 extern int _slurm_socketpair (int __domain, int __type, 
-                                int __protocol, int __fds[2])
+			      int __protocol, int __fds[2])
 {
-        return SLURM_PROTOCOL_FUNCTION_NOT_IMPLEMENTED ;
+	return SLURM_PROTOCOL_FUNCTION_NOT_IMPLEMENTED ;
 }
 
 /* Give the socket FD the local address ADDR (which is LEN bytes long).  */
 extern int _slurm_bind (int __fd, struct sockaddr const * __addr, 
-                                socklen_t __len)
+				socklen_t __len)
 {
-        return bind ( __fd , __addr , __len ) ;
+	return bind ( __fd , __addr , __len ) ;
 }
 
 /* Put the local address of FD into *ADDR and its length in *LEN.  */
 extern int _slurm_getsockname (int __fd, struct sockaddr * __addr, 
-                                socklen_t *__restrict __len)
+			       socklen_t *__restrict __len)
 {
-        return getsockname ( __fd , __addr , __len ) ;        
+	return getsockname ( __fd , __addr , __len ) ;	
 }
 
 /* Open a connection on socket FD to peer at ADDR (which LEN bytes long).
@@ -580,7 +612,7 @@ extern int _slurm_getsockname (int __fd, struct sockaddr * __addr,
  * and the only address from which to accept transmissions.
  * Return 0 on success, -1 for errors.  */
 extern int _slurm_connect (int __fd, struct sockaddr const * __addr, 
-                                socklen_t __len)
+			   socklen_t __len)
 {
 #if 0
 	return connect ( __fd , __addr , __len ) ;
@@ -614,7 +646,7 @@ again:	rc = poll(&ufds, 1, 5000);
 		/* poll failed */
 		if (errno == EINTR) {
 			/* NOTE: connect() is non-interruptible in Linux */
-			debug3("_slurm_connect poll failed: %m");
+			debug2("_slurm_connect poll failed: %m");
 			goto again;
 		} else
 			error("_slurm_connect poll failed: %m");
@@ -655,76 +687,77 @@ done:
 /* Put the address of the peer connected to socket FD into *ADDR
  * (which is *LEN bytes long), and its actual length into *LEN.  */
 extern int _slurm_getpeername (int __fd, struct sockaddr * __addr, 
-                                socklen_t *__restrict __len)
+			       socklen_t *__restrict __len)
 {
-        return getpeername ( __fd , __addr , __len ) ;
+	return getpeername ( __fd , __addr , __len ) ;
 }
 
 /* Send N bytes of BUF to socket FD.  Returns the number sent or -1.  */
 extern ssize_t _slurm_send (int __fd, __const void *__buf, size_t __n, 
-                                int __flags)
+			    int __flags)
 {
-        return send ( __fd , __buf , __n , __flags ) ;
+	return send ( __fd , __buf , __n , __flags ) ;
 }
 
 /* Read N bytes into BUF from socket FD.
  * Returns the number read or -1 for errors.  */
 extern ssize_t _slurm_recv (int __fd, void *__buf, size_t __n, int __flags)
 {
-        return recv ( __fd , __buf , __n , __flags ) ;
+	return recv ( __fd , __buf , __n , __flags ) ;
 }
 
 /* Send N bytes of BUF on socket FD to peer at address ADDR (which is
  * ADDR_LEN bytes long).  Returns the number sent, or -1 for errors.  */
-extern ssize_t _slurm_sendto (int __fd, __const void *__buf, size_t __n, int __flags, struct sockaddr const * __addr, 
-                                socklen_t __addr_len)
+extern ssize_t _slurm_sendto (int __fd, __const void *__buf, size_t __n, 
+			      int __flags, struct sockaddr const * __addr, 
+			      socklen_t __addr_len)
 {
-        return sendto ( __fd , __buf , __n , __flags , __addr, __addr_len) ;
+	return sendto ( __fd , __buf , __n , __flags , __addr, __addr_len) ;
 }
 /* Read N bytes into BUF through socket FD.
  * If ADDR is not NULL, fill in *ADDR_LEN bytes of it with tha address of
  * the sender, and store the actual size of the address in *ADDR_LEN.
  * Returns the number of bytes read or -1 for errors.  */
 extern ssize_t _slurm_recvfrom (int __fd, void *__restrict __buf, 
-                                size_t __n, int __flags, 
-                                struct sockaddr * __addr, 
-                                socklen_t *__restrict __addr_len)
+				size_t __n, int __flags, 
+				struct sockaddr * __addr, 
+				socklen_t *__restrict __addr_len)
 {
-        return recvfrom ( __fd , __buf , __n , __flags , __addr, __addr_len) ;
+	return recvfrom ( __fd , __buf , __n , __flags , __addr, __addr_len) ;
 }
 
 /* Send a msg described MESSAGE on socket FD.
  * Returns the number of bytes sent, or -1 for errors.  */
 extern ssize_t _slurm_sendmsg (int __fd, __const struct msghdr *__msg, 
-                                int __flags)
+				int __flags)
 {
-        return sendmsg ( __fd , __msg , __flags ) ;
+	return sendmsg ( __fd , __msg , __flags ) ;
 }
 
 /* Send a msg described MESSAGE on socket FD.
  * Returns the number of bytes read or -1 for errors.  */
 extern ssize_t _slurm_recvmsg (int __fd, struct msghdr *__msg, int __flags)
 {
-        return recvmsg ( __fd , __msg , __flags );
+	return recvmsg ( __fd , __msg , __flags );
 }
 
 /* Put the current value for socket FD's option OPTNAME at protocol level LEVEL
  * into OPTVAL (which is *OPTLEN bytes long), and set *OPTLEN to the value's
  * actual length.  Returns 0 on success, -1 for errors.  */
 extern int _slurm_getsockopt (int __fd, int __level, int __optname, 
-                                void *__restrict __optval, 
-                                socklen_t *__restrict __optlen)
+				void *__restrict __optval, 
+				socklen_t *__restrict __optlen)
 {
-        return getsockopt ( __fd , __level , __optname , __optval , __optlen ) ;
+	return getsockopt ( __fd , __level , __optname , __optval , __optlen ) ;
 }
 
 /* Set socket FD's option OPTNAME at protocol level LEVEL
  * to *OPTVAL (which is OPTLEN bytes long).
  * Returns 0 on success, -1 for errors.  */
 extern int _slurm_setsockopt (int __fd, int __level, int __optname, 
-                                __const void *__optval, socklen_t __optlen)
+				__const void *__optval, socklen_t __optlen)
 {
-        return setsockopt ( __fd , __level , __optname , __optval , __optlen ) ;
+	return setsockopt ( __fd , __level , __optname , __optval , __optlen ) ;
 }
 
 
@@ -733,7 +766,7 @@ extern int _slurm_setsockopt (int __fd, int __level, int __optname,
  * Returns 0 on success, -1 for errors.  */
 extern int _slurm_listen (int __fd, int __n)
 {
-        return listen ( __fd , __n ) ;
+	return listen ( __fd , __n ) ;
 }
 
 /* Await a connection on socket FD.
@@ -742,9 +775,9 @@ extern int _slurm_listen (int __fd, int __n)
  * peer and *ADDR_LEN to the address's actual length, and return the
  * new socket's descriptor, or -1 for errors.  */
 extern int _slurm_accept (int __fd, struct sockaddr * __addr, 
-                                socklen_t *__restrict __addr_len)
+				socklen_t *__restrict __addr_len)
 {
-        return accept ( __fd , __addr , __addr_len ) ;
+	return accept ( __fd , __addr , __addr_len ) ;
 }
 
 /* Shut down all or part of the connection open on socket FD.
@@ -755,134 +788,134 @@ extern int _slurm_accept (int __fd, struct sockaddr * __addr,
  * Returns 0 on success, -1 for errors.  */
 extern int _slurm_shutdown (int __fd, int __how)
 {
-        return shutdown ( __fd , __how );
+	return shutdown ( __fd , __how );
 }
 
 extern int _slurm_close (int __fd )
 {
-        return close ( __fd ) ;
+	return close ( __fd ) ;
 }
 
 extern int _slurm_fcntl(int fd, int cmd, ... )
 {
-        int rc ;
-        va_list va ;
+	int rc ;
+	va_list va ;
 
-        va_start ( va , cmd ) ;
-        rc =_slurm_vfcntl ( fd , cmd , va ) ;
-        va_end ( va ) ;
-        return rc ;
+	va_start ( va , cmd ) ;
+	rc =_slurm_vfcntl ( fd , cmd , va ) ;
+	va_end ( va ) ;
+	return rc ;
 }
 
 extern int _slurm_vfcntl(int fd, int cmd, va_list va )
 {
-        long arg ;
+	long arg ;
 
-        switch ( cmd )
-        {
-                case F_GETFL :
-                        return fcntl ( fd , cmd ) ;
-                        break ;
-                case F_SETFL :
-                        arg = va_arg ( va , long ) ;
-                        return fcntl ( fd , cmd , arg) ;
-                        break ;
-                default :
-                        return SLURM_PROTOCOL_ERROR ;
-                        break ;
-        }
+	switch ( cmd )
+	{
+		case F_GETFL :
+			return fcntl ( fd , cmd ) ;
+			break ;
+		case F_SETFL :
+			arg = va_arg ( va , long ) ;
+			return fcntl ( fd , cmd , arg) ;
+			break ;
+		default :
+			return SLURM_PROTOCOL_ERROR ;
+			break ;
+	}
 }
 
 /* sets the fields of a slurm_addr */
 void _slurm_set_addr_uint (slurm_addr *addr, uint16_t port, uint32_t ipaddr)
 {
-        addr->sin_family      = AF_SLURM ;
-        addr->sin_port        = htons(port);
-        addr->sin_addr.s_addr = htonl(ipaddr);
+	addr->sin_family      = AF_SLURM ;
+	addr->sin_port	= htons(port);
+	addr->sin_addr.s_addr = htonl(ipaddr);
 }
 
 /* resets the address field of a slurm_addr, port and family are unchanged */
 void _reset_slurm_addr (slurm_addr *addr, slurm_addr new_addr)
 {
-        addr->sin_addr.s_addr = new_addr.sin_addr.s_addr;
+	addr->sin_addr.s_addr = new_addr.sin_addr.s_addr;
 }
 
 void _slurm_set_addr_char (slurm_addr * addr, uint16_t port, char *host)
 {
-        struct hostent * he    = NULL;
-        int              h_err = 0;
-        char *           h_buf[4096];
+	struct hostent * he    = NULL;
+	int	   h_err = 0;
+	char *	   h_buf[4096];
 
-        /* 
-         * If NULL hostname passed in, we only update the port
-         *   of addr
-         */
-        addr->sin_family = AF_SLURM;
-        addr->sin_port   = htons(port);
-        if (host == NULL)
-                return;
+	/* 
+	 * If NULL hostname passed in, we only update the port
+	 *   of addr
+	 */
+	addr->sin_family = AF_SLURM;
+	addr->sin_port   = htons(port);
+	if (host == NULL)
+		return;
 
-        he = get_host_by_name(host, (void *)&h_buf, sizeof(h_buf), &h_err);
+	he = get_host_by_name(host, (void *)&h_buf, sizeof(h_buf), &h_err);
 
-        if (he != NULL)
-                memcpy (&addr->sin_addr.s_addr, he->h_addr, he->h_length);
-        else {
-                error("Unable to resolve \"%s\": %s", host, hstrerror(h_err));
-                addr->sin_family = 0;
-                addr->sin_port = 0;
-        } 
-        return;
+	if (he != NULL)
+		memcpy (&addr->sin_addr.s_addr, he->h_addr, he->h_length);
+	else {
+		error("Unable to resolve \"%s\": %s", host, hstrerror(h_err));
+		addr->sin_family = 0;
+		addr->sin_port = 0;
+	} 
+	return;
 }
 
 void _slurm_get_addr (slurm_addr *addr, uint16_t *port, char *host, 
-                      unsigned int buflen )
+		      unsigned int buflen )
 {
-        struct hostent *he;
-        char   h_buf[4096];
-        int    h_err  = 0;
-        char * s_addr = (char *) &addr->sin_addr.s_addr;
-        int    len    = sizeof(addr->sin_addr.s_addr);
+	struct hostent *he;
+	char   h_buf[4096];
+	int    h_err  = 0;
+	char * s_addr = (char *) &addr->sin_addr.s_addr;
+	int    len    = sizeof(addr->sin_addr.s_addr);
 
-        he = get_host_by_addr( s_addr, len, AF_SLURM, 
-                               (void *) &h_buf, sizeof(h_buf), &h_err );
+	he = get_host_by_addr( s_addr, len, AF_SLURM, 
+			       (void *) &h_buf, sizeof(h_buf), &h_err );
 
-        if (he != NULL) {
-                *port = ntohs(addr->sin_port);
-                strncpy(host, he->h_name, buflen);
-        } else {
-                error("Lookup failed: %s", host_strerror(h_err));
-                *port = 0;
-                strncpy(host, "", buflen);
-        } 
-        return;
+	if (he != NULL) {
+		*port = ntohs(addr->sin_port);
+		strncpy(host, he->h_name, buflen);
+	} else {
+		error("Lookup failed: %s", host_strerror(h_err));
+		*port = 0;
+		strncpy(host, "", buflen);
+	} 
+	return;
 }
 
 void _slurm_print_slurm_addr ( slurm_addr * address, char *buf, size_t n )
 {
-        char addrbuf[INET_ADDRSTRLEN];
-        inet_ntop(AF_INET, &address->sin_addr, addrbuf, INET_ADDRSTRLEN);
-        /* warning: silently truncates */
-        snprintf(buf, n, "%s:%d", addrbuf, ntohs(address->sin_port));
+	char addrbuf[INET_ADDRSTRLEN];
+	inet_ntop(AF_INET, &address->sin_addr, addrbuf, INET_ADDRSTRLEN);
+	/* warning: silently truncates */
+	snprintf(buf, n, "%s:%d", addrbuf, ntohs(address->sin_port));
 }
-        
+	
 void _slurm_pack_slurm_addr(slurm_addr *addr, Buf buffer)
 {
-        pack32( ntohl( addr->sin_addr.s_addr ), buffer );
-        pack16( ntohs( addr->sin_port ), buffer );
+	pack32( ntohl( addr->sin_addr.s_addr ), buffer );
+	pack16( ntohs( addr->sin_port ), buffer );
 }
 
 int _slurm_unpack_slurm_addr_no_alloc(slurm_addr *addr, Buf buffer)
 {
-        addr->sin_family = AF_SLURM ;
-        safe_unpack32(&addr->sin_addr.s_addr, buffer);
-        safe_unpack16(&addr->sin_port, buffer);
+	addr->sin_family = AF_SLURM ;
+	safe_unpack32(&addr->sin_addr.s_addr, buffer);
+	safe_unpack16(&addr->sin_port, buffer);
 
-        addr->sin_addr.s_addr = htonl(addr->sin_addr.s_addr);
-        addr->sin_port = htons(addr->sin_port);
-        return SLURM_SUCCESS;
+	addr->sin_addr.s_addr = htonl(addr->sin_addr.s_addr);
+	addr->sin_port = htons(addr->sin_port);
+	return SLURM_SUCCESS;
 
     unpack_error:
-        return SLURM_ERROR;
+	return SLURM_ERROR;
 }
 
 /*
