@@ -4,7 +4,7 @@
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -38,6 +38,7 @@
 #include "./crypto.h"
 #include "./msg.h"
 #include "src/common/uid.h"
+#include "src/slurmctld/locks.h"
 #include <sys/poll.h>
 
 #define _DEBUG 0
@@ -56,6 +57,7 @@ char     e_host[E_HOST_SIZE] = "";
 char     e_host_bu[E_HOST_SIZE] = "";
 uint16_t e_port = 0;
 struct   part_record *exclude_part_ptr[EXC_PART_CNT];
+struct   part_record *hide_part_ptr[HIDE_PART_CNT];
 uint32_t first_job_id;
 uint16_t job_aggregation_time = 10;	/* Default value is 10 seconds */
 int      init_prio_mode = PRIO_HOLD;
@@ -141,12 +143,20 @@ static void *_msg_thread(void *no_data)
 	slurm_fd sock_fd = -1, new_fd;
 	slurm_addr cli_addr;
 	char *msg;
-	slurm_ctl_conf_t *conf = slurm_conf_lock();
+	slurm_ctl_conf_t *conf;
 	int i;
+	/* Locks: Write configuration, job, node, and partition */
+	slurmctld_lock_t config_write_lock = {
+		WRITE_LOCK, WRITE_LOCK, WRITE_LOCK, WRITE_LOCK };
 
+	conf = slurm_conf_lock();
 	sched_port = conf->schedport;
 	slurm_conf_unlock();
 
+	/* Wait until configuration is completely loaded */
+	lock_slurmctld(config_write_lock);
+	unlock_slurmctld(config_write_lock);
+
 	/* If SchedulerPort is already taken, keep trying to open it
 	 * once per minute. Slurmctld will continue to function
 	 * during this interval even if nothing can be scheduled. */
@@ -180,8 +190,10 @@ static void *_msg_thread(void *no_data)
 		err_code = 0;
 		err_msg = "";
 		msg = _recv_msg(new_fd);
-		_proc_msg(new_fd, msg);
-		xfree(msg);
+		if (msg) {
+			_proc_msg(new_fd, msg);
+			xfree(msg);
+		}
 		slurm_close_accepted_conn(new_fd);
 	}
 	verbose("wiki: message engine shutdown");
@@ -205,7 +217,7 @@ static char * _get_wiki_conf_path(void)
 		val = default_slurm_config_file;
 
 	/* Replace file name on end of path */
-	i = strlen(val) + 1;
+	i = strlen(val) + 10;
 	path = xmalloc(i);
 	strcpy(path, val);
 	val = strrchr(path, (int)'/');
@@ -232,12 +244,13 @@ extern int parse_wiki_config(void)
 		{"EHostBackup", S_P_STRING},
 		{"EPort", S_P_UINT16},
 		{"ExcludePartitions", S_P_STRING},
+		{"HidePartitionJobs", S_P_STRING},
 		{"HostFormat", S_P_UINT16},
 		{"JobAggregationTime", S_P_UINT16},
 		{"JobPriority", S_P_STRING},
 		{NULL} };
 	s_p_hashtbl_t *tbl;
-	char *exclude_partitions;
+	char *exclude_partitions, *hide_partitions;
 	char *key = NULL, *priority_mode = NULL, *wiki_conf;
 	struct stat buf;
 	slurm_ctl_conf_t *conf;
@@ -246,6 +259,8 @@ extern int parse_wiki_config(void)
 	/* Set default values */
 	for (i=0; i<EXC_PART_CNT; i++)
 		exclude_part_ptr[i] = NULL;
+	for (i=0; i<HIDE_PART_CNT; i++)
+		hide_part_ptr[i] = NULL;
 	conf = slurm_conf_lock();
 	strncpy(e_host, conf->control_addr, sizeof(e_host));
 	if (conf->backup_addr) {
@@ -258,7 +273,7 @@ extern int parse_wiki_config(void)
 
 	wiki_conf = _get_wiki_conf_path();
 	if ((wiki_conf == NULL) || (stat(wiki_conf, &buf) == -1)) {
-		debug("No wiki.conf file (%s)", wiki_conf);
+		fatal("No wiki.conf file (%s)", wiki_conf);
 		xfree(wiki_conf);
 		return SLURM_SUCCESS;
 	}
@@ -269,7 +284,7 @@ extern int parse_wiki_config(void)
 		fatal("something wrong with opening/reading wiki.conf file");
 
 	if (! s_p_get_string(&key, "AuthKey", tbl))
-		debug("Warning: No wiki_conf AuthKey specified");
+		fatal("No wiki_conf AuthKey specified");
 	else {
 		strncpy(auth_key, key, sizeof(auth_key));
 		xfree(key);
@@ -305,6 +320,25 @@ extern int parse_wiki_config(void)
 		}
 	}
 
+	if (s_p_get_string(&hide_partitions, "HidePartitionJobs", tbl)) {
+		char *tok = NULL, *tok_p = NULL;
+		tok = strtok_r(hide_partitions, ",", &tok_p);
+		i = 0;
+		while (tok) {
+			if (i >= HIDE_PART_CNT) {
+				error("HidePartitionJobs has too many entries "
+				      "skipping %s and later entries");
+				break;
+			}	
+			hide_part_ptr[i] = find_part_record(tok);
+			if (hide_part_ptr[i])
+				i++;
+			else
+				error("HidePartitionJobs %s not found", tok);
+			tok = strtok_r(NULL, ",", &tok_p);
+		}
+	}
+
 	if (s_p_get_string(&priority_mode, "JobPriority", tbl)) {
 		if (strcasecmp(priority_mode, "hold") == 0)
 			init_prio_mode = PRIO_HOLD;
@@ -328,11 +362,61 @@ extern int parse_wiki_config(void)
 	info("HostFormat         = %u", use_host_exp);
 	info("JobAggregationTime = %u sec", job_aggregation_time);
 	info("JobPriority        = %s", init_prio_mode ? "run" : "hold");
-	info("KillWait           = %u sec", kill_wait);      
+	info("KillWait           = %u sec", kill_wait);
+	for (i=0; i<EXC_PART_CNT; i++) {
+		if (!exclude_part_ptr[i])
+			continue;
+		info("ExcludePartitions  = %s", exclude_part_ptr[i]->name);
+	}
+	for (i=0; i<HIDE_PART_CNT; i++) {
+		if (!hide_part_ptr[i])
+			continue;
+		info("HidePartitionJobs  = %s", hide_ptr_ptr[i]->name);
+	}
 #endif
 	return SLURM_SUCCESS;
 }
 
+/*
+ * Return a string containing any scheduling plugin configuration information 
+ * that we want to expose via "scontrol show configuration".
+ * NOTE: the caller must xfree the returned pointer
+ */
+extern char *	get_wiki_conf(void)
+{
+	int i, first;
+	char buf[20], *conf = NULL;
+
+	snprintf(buf, sizeof(buf), "HostFormat=%u", use_host_exp);
+	xstrcat(conf, buf);
+
+	first = 1;
+	for (i=0; i<EXC_PART_CNT; i++) {
+		if (!exclude_part_ptr[i])
+			continue;
+		if (first) {
+			xstrcat(conf, ";ExcludePartitions=");
+			first = 0;
+		} else
+			xstrcat(conf, ",");
+		xstrcat(conf, exclude_part_ptr[i]->name);
+	}
+
+	first = 1;
+	for (i=0; i<HIDE_PART_CNT; i++) {
+		if (!hide_part_ptr[i])
+			continue;
+		if (first) {
+			xstrcat(conf, ";HidePartitionJobs=");
+			first = 0;
+		} else
+			xstrcat(conf, ",");
+		xstrcat(conf, hide_part_ptr[i]->name);
+	}
+
+	return conf;
+}
+
 static size_t	_read_bytes(int fd, char *buf, size_t size)
 {
 	size_t bytes_remaining, bytes_read;
@@ -597,7 +681,8 @@ static void	_proc_msg(slurm_fd new_fd, char *msg)
 		job_release_task(cmd_ptr, &err_code, &err_msg);
 	} else if (strncmp(cmd_ptr, "JOBWILLRUN", 10) == 0) {
 		msg_type = "wiki:JOBWILLRUN";
-		job_will_run(cmd_ptr, &err_code, &err_msg);
+		if (!job_will_run(cmd_ptr, &err_code, &err_msg))
+			goto free_resp_msg;
 	} else if (strncmp(cmd_ptr, "MODIFYJOB", 9) == 0) {
 		msg_type = "wiki:MODIFYJOB";
 		job_modify_wiki(cmd_ptr, &err_code, &err_msg);
