@@ -6,7 +6,7 @@
  *  Copyright (C) 2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -40,7 +40,7 @@
 #include "sacct.h"
 #include "src/common/parse_time.h"
 #include "slurm.h"
-#define FORMAT_STRING_SIZE 50
+#define FORMAT_STRING_SIZE 34
 
 void _elapsed_time(long secs, long usecs, char *str);
 
@@ -97,98 +97,129 @@ void print_fields(type_t type, void *object)
 
 void print_cpu(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char str[FORMAT_STRING_SIZE];
 	
 	switch(type) {
 	case HEADLINE:
-		printf("%15s", "Cpu");
+		printf("%-15s", "Cpu");
 		break;
 	case UNDERSCORE:
-		printf("%15s", "---------------");
+		printf("%-15s", "---------------");
 		break;
 	case JOB:
 		_elapsed_time(job->tot_cpu_sec, job->tot_cpu_usec, str);
-		printf("%15s", str);
+		printf("%-15s", str);
 		break;
 	case JOBSTEP:
 		_elapsed_time(step->tot_cpu_sec, step->tot_cpu_usec, str);
-		printf("%15s", str);
+		printf("%-15s", str);
+		break;
+	default:
+		printf("%-15s", "n/a");
 		break;
 	} 
 }
 
 void print_elapsed(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char str[FORMAT_STRING_SIZE];
 
 	switch(type) {
 	case HEADLINE:
-		printf("%15s", "Elapsed");
+		printf("%-15s", "Elapsed");
 		break;
 	case UNDERSCORE:
-		printf("%15s", "---------------");
+		printf("%-15s", "---------------");
 		break;
 	case JOB:
 		_elapsed_time(job->elapsed, 0, str);
-		printf("%15s", str);
+		printf("%-15s", str);
 		break;
 	case JOBSTEP:
 		_elapsed_time(step->elapsed, 0, str);
-		printf("%15s", str);
+		printf("%-15s", str);
+		break;
+	default:
+		printf("%-15s", "n/a");
 		break;
 	} 
 }
 
 void print_exitcode(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
+	char tmp[9];
+	uint16_t term_sig = 0;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%8s", "ExitCode");
+		printf("%-8s", "ExitCode");
 		break;
 	case UNDERSCORE:
-		printf("%8s", "--------");
+		printf("%-8s", "--------");
 		break;
 	case JOB:
-		printf("%8d", job->exitcode);
+		if (WIFSIGNALED(job->exitcode))
+			term_sig = WTERMSIG(job->exitcode);
+	
+		snprintf(tmp, sizeof(tmp), "%u:%u",
+			 WEXITSTATUS(job->exitcode), term_sig);
+		printf("%-8s", tmp);
 		break;
 	case JOBSTEP:
-		printf("%8d", step->exitcode);
+		if (WIFSIGNALED(step->exitcode))
+			term_sig = WTERMSIG(step->exitcode);
+	
+		snprintf(tmp, sizeof(tmp), "%u:%u",
+			 WEXITSTATUS(step->exitcode), term_sig);
+		printf("%-8s", tmp);
+		break;
+	default:
+		printf("%-8s", "n/a");
 		break;
 	} 
 }
 
 void print_gid(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	int32_t gid = -1;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%5s", "Gid");
+		printf("%-5s", "Gid");
 		break;
 	case UNDERSCORE:
-		printf("%5s", "-----");
+		printf("%-5s", "-----");
 		break;
 	case JOB:
-		printf("%5d", job->header.gid);
+		gid = job->gid;
+		break;
+	case JOBCOMP:
+		printf("%-5u", jobcomp->gid);
 		break;
 	case JOBSTEP:
-		printf("s%5d", step->header.gid);
+		printf("%-5s", " ");
+		break;
+	default:
+		printf("%-5s", "n/a");
 		break;
 	} 
+
+	if(gid != -1) 
+		printf("%-5d", gid);
 }
 
 void print_group(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
 	int gid = -1;
 	char	*tmp="(unknown)";
 	struct	group *gr = NULL;
@@ -201,10 +232,16 @@ void print_group(type_t type, void *object)
 		printf("%-9s", "---------");
 		break;
 	case JOB:
-		gid = job->header.gid;
+		gid = job->gid;
+		break;
+	case JOBCOMP:
+		printf("%-9s", jobcomp->gid_name);
 		break;
 	case JOBSTEP:
-		gid = step->header.gid;
+		printf("%-9s", " ");
+		break;
+	default:
+		printf("%-9s", "n/a");
 		break;
 	}
 	if(gid != -1) {
@@ -214,125 +251,34 @@ void print_group(type_t type, void *object)
 	} 
 }
 
-void print_idrss(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-	struct rusage rusage;
-	char outbuf[FORMAT_STRING_SIZE];
-	rusage.ru_idrss = 0;
-	
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Idrss");
-		return;
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		return;
-		break;
-	case JOB:
-		rusage = job->rusage;
-		break;
-	case JOBSTEP:
-		rusage = step->rusage;
-		break;
-	} 
-	convert_num_unit((float)rusage.ru_idrss, outbuf, sizeof(outbuf),
-			 UNIT_NONE);
-	printf("%8s", outbuf);
-}
-
-void print_inblocks(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Inblocks");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_inblock);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_inblock);
-		break;
-	} 
-}
-
-void print_isrss(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Isrss");
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		break;
-	case JOB:
-		printf("%8ld", job->rusage.ru_isrss);
-		break;
-	case JOBSTEP:
-		printf("%8ld", step->rusage.ru_isrss);
-		break;
-	} 
-
-}
-
-void print_ixrss(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Ixrss");
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		break;
-	case JOB:
-		printf("%8ld", job->rusage.ru_ixrss);
-		break;
-	case JOBSTEP:
-		printf("%8ld", step->rusage.ru_ixrss);
-		break;
-	} 
-
-}
-
 void print_job(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%8s", "Job");
+		printf("%-8s", "Job");
 		break;
 	case UNDERSCORE:
-		printf("%8s", "--------");
+		printf("%-8s", "--------");
 		break;
 	case JOB:
-		printf("%8d", job->header.jobnum);
+		printf("%-8u", job->jobid);
 		break;
 	case JOBSTEP:
-		printf("%8d", step->header.jobnum);
+		printf("%-8s", " ");
+		break;
+	default:
+		printf("%-8s", "n/a");
 		break;
 	} 
 }
 
 void print_name(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
@@ -349,6 +295,15 @@ void print_name(type_t type, void *object)
 		else
 			printf("%-15.15s...", job->jobname);
 			
+		break;
+	case JOBCOMP:
+		if(!jobcomp->jobname)
+			printf("%-18s", "unknown");			     
+		else if(strlen(jobcomp->jobname)<19)
+			printf("%-18s", jobcomp->jobname);
+		else
+			printf("%-15.15s...", jobcomp->jobname);
+			
 		break;
 	case JOBSTEP:
 		if(!step->stepname)
@@ -358,13 +313,17 @@ void print_name(type_t type, void *object)
 		else
 			printf("%-15.15s...", step->stepname);
 		break;
+	default:
+		printf("%-18s", "n/a");
+		break;
 	} 
 }
 
 void print_jobid(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char outbuf[10];
 
 	switch(type) {
@@ -375,148 +334,54 @@ void print_jobid(type_t type, void *object)
 		printf("%-10s", "----------");
 		break;
 	case JOB:
-		printf("%-10d", job->header.jobnum);
+		printf("%-10u", job->jobid);
+		break;
+	case JOBCOMP:
+		printf("%-10u", jobcomp->jobid);
 		break;
 	case JOBSTEP:
 		snprintf(outbuf, sizeof(outbuf), "%u.%u",
-			 step->header.jobnum,
-			 step->stepnum);
+			 step->jobid,
+			 step->stepid);
 		printf("%-10s", outbuf);
 		break;
-	} 
-
-}
-
-void print_majflt(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Majflt");
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		break;
-	case JOB:
-		printf("%8ld", job->rusage.ru_majflt);
-		break;
-	case JOBSTEP:
-		printf("%8ld", step->rusage.ru_majflt);
+	default:
+		printf("%-10s", "n/a");
 		break;
 	} 
-}
 
-void print_minflt(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Minflt");
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		break;
-	case JOB:
-		printf("%8ld", job->rusage.ru_minflt);
-		break;
-	case JOBSTEP:
-		printf("%8ld", step->rusage.ru_minflt);
-		break;
-	} 
-}
-
-void print_msgrcv(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Msgrcv");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_msgrcv);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_msgrcv);
-		break;
-	} 
-}
-
-void print_msgsnd(type_t type, void *object)
-{
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Msgsnd");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_msgsnd);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_msgsnd);
-		break;
-	} 
 }
 
 void print_ncpus(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%7s", "Ncpus");
+		printf("%-7s", "Ncpus");
 		break;
 	case UNDERSCORE:
-		printf("%7s", "-------");
+		printf("%-7s", "-------");
 		break;
 	case JOB:
-		printf("%7d", job->ncpus);
+		printf("%-7u", job->alloc_cpus);
 		break;
 	case JOBSTEP:
-		printf("%7d", step->ncpus);
+		printf("%-7u", step->ncpus);
 		break;
-	} 
-}
-
-void print_nivcsw(type_t type, void *object)
-{ 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Nivcsw");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_nivcsw);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_nivcsw);
+	default:
+		printf("%-7s", "n/a");
 		break;
 	} 
 }
 
 void print_nodes(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
+	
 	switch(type) {
 	case HEADLINE:
 		printf("%-30s", "Nodes");
@@ -527,121 +392,69 @@ void print_nodes(type_t type, void *object)
 	case JOB:
 		printf("%-30s", job->nodes);
 		break;
+	case JOBCOMP:
+		printf("%-30s", jobcomp->nodelist);
+		break;
 	case JOBSTEP:
-		printf("%-30s", "                              ");
+		printf("%-30s", step->nodes);
+		break;
+	default:
+		printf("%-30s", "n/a");
 		break;
 	} 
 }
 
-void print_nsignals(type_t type, void *object)
+void print_nnodes(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	char temp[FORMAT_STRING_SIZE];
 
 	switch(type) {
 	case HEADLINE:
-		printf("%9s", "Nsignals");
+		printf("%-8s", "Node Cnt");
 		break;
 	case UNDERSCORE:
-		printf("%9s", "---------");
+		printf("%-8s", "--------");
 		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_nsignals);
+	case JOBCOMP:
+		convert_num_unit((float)jobcomp->node_cnt, temp, 
+				 sizeof(temp), UNIT_NONE);
+		printf("%-8s", temp);
 		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_nsignals);
-		break;
-	} 
-}
-
-void print_nswap(type_t type, void *object)
-{ 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%8s", "Nswap");
-		break;
-	case UNDERSCORE:
-		printf("%8s", "------");
-		break;
-	case JOB:
-		printf("%8ld", job->rusage.ru_nswap);
-		break;
-	case JOBSTEP:
-		printf("%8ld", step->rusage.ru_nswap);
+	default:
+		printf("%-8s", "n/a");
 		break;
 	} 
 }
 
 void print_ntasks(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%7s", "Ntasks");
+		printf("%-7s", "Ntasks");
 		break;
 	case UNDERSCORE:
-		printf("%7s", "-------");
+		printf("%-7s", "-------");
 		break;
 	case JOB:
-		printf("%7d", job->ntasks);
+		printf("%-7u", job->alloc_cpus);
 		break;
 	case JOBSTEP:
-		printf("%7d", step->ntasks);
+		printf("%-7u", step->ncpus);
 		break;
-	} 
-}
-
-void print_nvcsw(type_t type, void *object)
-{ 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Nvcsw");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_nvcsw);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_nvcsw);
-		break;
-	} 
-}
-
-void print_outblocks(type_t type, void *object)
-{ 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
-
-	switch(type) {
-	case HEADLINE:
-		printf("%9s", "Outblocks");
-		break;
-	case UNDERSCORE:
-		printf("%9s", "---------");
-		break;
-	case JOB:
-		printf("%9ld", job->rusage.ru_oublock);
-		break;
-	case JOBSTEP:
-		printf("%9ld", step->rusage.ru_oublock);
+	default:
+		printf("%-7s", "n/a");
 		break;
 	} 
 }
 
 void print_partition(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
@@ -651,30 +464,37 @@ void print_partition(type_t type, void *object)
 		printf("%-10s", "----------");
 		break;
 	case JOB:
-		if(!job->header.partition)
+		if(!job->partition)
 			printf("%-10s", "unknown");			     
-		else if(strlen(job->header.partition)<11)
-			printf("%-10s", job->header.partition);
+		else if(strlen(job->partition)<11)
+			printf("%-10s", job->partition);
 		else
-			printf("%-7.7s...", job->header.partition);
+			printf("%-7.7s...", job->partition);
+		
+		break;
+	case JOBCOMP:
+		if(!jobcomp->partition)
+			printf("%-10s", "unknown");			     
+		else if(strlen(jobcomp->partition)<11)
+			printf("%-10s", jobcomp->partition);
+		else
+			printf("%-7.7s...", jobcomp->partition);
 		
 		break;
 	case JOBSTEP:
-		if(!step->header.partition)
-			printf("%-10s", "unknown");			     
-		else if(strlen(step->header.partition)<11)
-			printf("%-10s", step->header.partition);
-		else
-			printf("%-7.7s...", step->header.partition);
-	
+		printf("%-10s", " ");
+		break;
+	default:
+		printf("%-10s", "n/a");
 		break;
 	} 
 }
 
+#ifdef HAVE_BG
 void print_blockid(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
@@ -684,44 +504,51 @@ void print_blockid(type_t type, void *object)
 		printf("%-16s", "----------------");
 		break;
 	case JOB:
-		if(!job->header.blockid)
+		if(!job->blockid)
 			printf("%-16s", "unknown");			     
-		else if(strlen(job->header.blockid)<17)
-			printf("%-16s", job->header.blockid);
+		else if(strlen(job->blockid)<17)
+			printf("%-16s", job->blockid);
 		else
-			printf("%-13.13s...", job->header.blockid);
+			printf("%-13.13s...", job->blockid);
+		
+		break;
+	case JOBCOMP:
+		if(!jobcomp->blockid)
+			printf("%-16s", "unknown");			     
+		else if(strlen(jobcomp->blockid)<17)
+			printf("%-16s", jobcomp->blockid);
+		else
+			printf("%-13.13s...", jobcomp->blockid);
 		
 		break;
 	case JOBSTEP:
-		if(!step->header.blockid)
-			printf("%-16s", "unknown");			     
-		else if(strlen(step->header.blockid)<17)
-			printf("%-16s", step->header.blockid);
-		else
-			printf("%-13.13s...", step->header.blockid);
-	
+		printf("%-16s", " ");
+		break;
+	default:
+		printf("%-16s", "n/a");
 		break;
 	} 
 }
+#endif
 
 void print_pages(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char outbuf[FORMAT_STRING_SIZE];
 	char buf1[FORMAT_STRING_SIZE];
 	char buf2[FORMAT_STRING_SIZE];
-	char buf3[50];
+	char buf3[FORMAT_STRING_SIZE];
 	sacct_t sacct;
 	char *nodes = NULL;
 	uint32_t pos;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%-50s", "MaxPages/Node:Task - Ave");
+		printf("%-34s", "MaxPages/Node:Task - Ave");
 		break;
 	case UNDERSCORE:
-		printf("%-50s", "----------------------------------");
+		printf("%-34s", "----------------------------------");
 		break;
 	case JOB:
 		sacct = job->sacct;
@@ -742,7 +569,7 @@ void print_pages(type_t type, void *object)
 				 sacct.max_pages_id.taskid, 
 				 buf2);
 		}
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
 		break;
 	case JOBSTEP:
 		sacct = step->sacct;
@@ -758,42 +585,45 @@ void print_pages(type_t type, void *object)
 			 buf3,
 			 sacct.max_pages_id.taskid, 
 			 buf2);
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
+		break;
+	default:
+		printf("%-34s", "n/a");
 		break;
 	} 
 }
 
 void print_rss(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char outbuf[FORMAT_STRING_SIZE];
 	char buf1[FORMAT_STRING_SIZE];
 	char buf2[FORMAT_STRING_SIZE];
-	char buf3[50];
+	char buf3[FORMAT_STRING_SIZE];
 	sacct_t sacct;
 	char *nodes = NULL;
 	uint32_t pos;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%-50s", "MaxRSS/Node:Task - Ave");
+		printf("%-34s", "MaxRSS/Node:Task - Ave");
 		break;
 	case UNDERSCORE:
-		printf("%-50s", "--------------------------------");
+		printf("%-34s", "----------------------------------");
 		break;
 	case JOB:
 		sacct = job->sacct;
 		nodes = job->nodes;
 		pos = sacct.min_cpu_id.nodeid;				 
 		convert_num_unit((float)sacct.max_rss, buf1, sizeof(buf1),
-				 UNIT_NONE);
+				 UNIT_KILO);
 
 		if(job->track_steps)
 			snprintf(outbuf, FORMAT_STRING_SIZE, "%s/- - -", buf1);
 		else {
 			convert_num_unit((float)sacct.ave_rss, 
-					 buf2, sizeof(buf2), UNIT_NONE);
+					 buf2, sizeof(buf2), UNIT_KILO);
 			find_hostname(pos, nodes, buf3);
 			snprintf(outbuf, FORMAT_STRING_SIZE, "%s/%s:%u - %s", 
 				 buf1,
@@ -801,64 +631,74 @@ void print_rss(type_t type, void *object)
 				 sacct.max_rss_id.taskid, 
 				 buf2);
 		}
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
 		break;
 	case JOBSTEP:
 		sacct = step->sacct;
 		nodes = step->nodes;
 		pos = sacct.min_cpu_id.nodeid;				 
 		convert_num_unit((float)sacct.max_rss, buf1, sizeof(buf1),
-				 UNIT_NONE);
+				 UNIT_KILO);
 		convert_num_unit((float)sacct.ave_rss, buf2, sizeof(buf2),
-				 UNIT_NONE);
+				 UNIT_KILO);
 		find_hostname(pos, nodes, buf3);
 		snprintf(outbuf, FORMAT_STRING_SIZE, "%s/%s:%u - %s", 
 			 buf1,
 			 buf3, 
 			 sacct.max_rss_id.taskid, 
 			 buf2);
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
+		break;
+	default:
+		printf("%-34s", "n/a");
 		break;
 	} 
 }
 
-void print_status(type_t type, void *object)
+void print_state(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%-20s", "Status");
+		printf("%-20s", "State");
 		break;
 	case UNDERSCORE:
 		printf("%-20s", "--------------------");
 		break;
 	case JOB:
-		if ( job->status == JOB_CANCELLED) {
+		if ( job->state == JOB_CANCELLED) {
 			printf ("%-10s by %6d",
-				decode_status_int(job->status), job->requid);
+				job_state_string(job->state), job->requid);
 		}
 		else {
-			printf("%-20s", decode_status_int(job->status));
+			printf("%-20s", job_state_string(job->state));
 		}
 		break;
+	case JOBCOMP:
+		printf("%-20s", jobcomp->state);
+		break;
 	case JOBSTEP:
-		if ( step->status == JOB_CANCELLED) {
+		if ( step->state == JOB_CANCELLED) {
 			printf ("%-10s by %6d",
-				decode_status_int(step->status), step->requid);
+				job_state_string(step->state), step->requid);
 		}
 		else {
-			printf("%-20s", decode_status_int(step->status));
+			printf("%-20s", job_state_string(step->state));
 		}
 		break;
+	default:
+		printf("%-20s", "n/a");
+		break;
 	} 
 }
 
 void print_submit(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char time_str[32];
 		
 	switch(type) {
@@ -869,132 +709,183 @@ void print_submit(type_t type, void *object)
 		printf("%-14s", "--------------");
 		break;
 	case JOB:
-		slurm_make_time_str(&job->header.job_submit, 
+		slurm_make_time_str(&job->submit, 
 				    time_str, 
 				    sizeof(time_str));
 		printf("%-14s", time_str);
 		break;
 	case JOBSTEP:
-		slurm_make_time_str(&step->header.timestamp, 
+		slurm_make_time_str(&step->start, 
 				    time_str, 
 				    sizeof(time_str));
 		printf("%-14s", time_str);
 		break;
+	default:
+		printf("%-14s", "n/a");
+		break;
 	} 
 }
 
 void print_start(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char time_str[32];
 	
 	switch(type) {
 	case HEADLINE:
-		printf("%-14s", "Start Time");
+		printf("%-19s", "Start Time");
 		break;
 	case UNDERSCORE:
-		printf("%-14s", "--------------");
+		printf("%-19s", "--------------------");
 		break;
 	case JOB:
-		slurm_make_time_str(&job->header.timestamp, 
+		slurm_make_time_str(&job->start, 
 				    time_str, 
 				    sizeof(time_str));
-		printf("%-14s", time_str);
+		printf("%-19s", time_str);
+		break;
+	case JOBCOMP:
+		printf("%-19s", jobcomp->start_time);
 		break;
 	case JOBSTEP:
-		slurm_make_time_str(&step->header.timestamp, 
+		slurm_make_time_str(&step->start, 
 				    time_str, 
 				    sizeof(time_str));
-		printf("%-14s", time_str);
+		printf("%-19s", time_str);
+		break;
+	default:
+		printf("%-19s", "n/a");
+		break;
+	} 
+}
+
+void print_timelimit(type_t type, void *object)
+{ 
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	
+	switch(type) {
+	case HEADLINE:
+		printf("%-10s", "Time Limit");
+		break;
+	case UNDERSCORE:
+		printf("%-10s", "----------");
+		break;
+	case JOBCOMP:
+		printf("%-10s", jobcomp->timelimit);
+		break;
+	default:
+		printf("%-10s", "n/a");
 		break;
 	} 
 }
 
 void print_end(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char time_str[32];
 	
 	switch(type) {
 	case HEADLINE:
-		printf("%-14s", "End Time");
+		printf("%-19s", "End Time");
 		break;
 	case UNDERSCORE:
-		printf("%-14s", "--------------");
+		printf("%-19s", "--------------------");
 		break;
 	case JOB:
 		slurm_make_time_str(&job->end, 
 				    time_str, 
 				    sizeof(time_str));
-		printf("%-14s", time_str);
+		printf("%-19s", time_str);
+		break;
+	case JOBCOMP:
+		printf("%-19s", jobcomp->end_time);
 		break;
 	case JOBSTEP:
 		slurm_make_time_str(&step->end, 
 				    time_str, 
 				    sizeof(time_str));
-		printf("%-14s", time_str);
+		printf("%-19s", time_str);
+		break;
+	default:
+		printf("%-19s", "n/a");
 		break;
 	} 
 }
 
 void print_systemcpu(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char str[FORMAT_STRING_SIZE];
 
 	switch(type) {
 	case HEADLINE:
-		printf("%15s", "SystemCpu");
+		printf("%-15s", "SystemCpu");
 		break;
 	case UNDERSCORE:
-		printf("%15s", "---------------");
+		printf("%-15s", "---------------");
 		break;
 	case JOB:
-		_elapsed_time(job->rusage.ru_stime.tv_sec,
-			      job->rusage.ru_stime.tv_usec, str);
-		printf("%15s", str);
+		_elapsed_time(job->sys_cpu_sec,
+			      job->sys_cpu_usec, str);
+		printf("%-15s", str);
 		break;
 	case JOBSTEP:
-		_elapsed_time(step->rusage.ru_stime.tv_sec,
-			      step->rusage.ru_stime.tv_usec, str);
-		printf("%15s", str);
+		_elapsed_time(step->sys_cpu_sec,
+			      step->sys_cpu_usec, str);
+		printf("%-15s", str);
+		break;
+	default:
+		printf("%-15s", "n/a");
 		break;
 	} 
-
 }
 
 void print_uid(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
+	int32_t uid = -1;
+	struct passwd *pw = NULL;
 	
 	switch(type) {
 	case HEADLINE:
-		printf("%5s", "Uid");
+		printf("%-5s", "Uid");
 		break;
 	case UNDERSCORE:
-		printf("%5s", "-----");
+		printf("%-5s", "-----");
 		break;
 	case JOB:
-		printf("%5d", job->header.uid);
+		if(job->user) {
+			if ((pw=getpwnam(job->user)))
+				uid = pw->pw_uid;
+		} else 
+			uid = job->uid;
+		break;
+	case JOBCOMP:
+		printf("%-5u", jobcomp->uid);
 		break;
 	case JOBSTEP:
-		printf("%5d", step->header.uid);
+		printf("%-5s", " ");
 		break;
 	} 
+
+	if(uid != -1) 
+		printf("%-5d", uid);
 }
 
 void print_user(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobcomp_job_rec_t *jobcomp = (jobcomp_job_rec_t *)object;
 	int uid = -1;
 	char	*tmp="(unknown)";
-	struct	passwd *pw = NULL;
-	
+	struct	passwd *pw = NULL;		 
+
 	switch(type) {
 	case HEADLINE:
 		printf("%-9s", "User");
@@ -1003,10 +894,19 @@ void print_user(type_t type, void *object)
 		printf("%-9s", "---------");
 		break;
 	case JOB:
-		uid = job->header.uid;
+		if(job->user) 
+			printf("%-9s", job->user);
+		else
+			uid = job->uid;
+		break;
+	case JOBCOMP:
+		printf("%-9s", jobcomp->uid_name);
 		break;
 	case JOBSTEP:
-		uid = step->header.uid;
+		printf("%-9s", " ");
+		break;
+	default:
+		printf("%-9s", "n/a");
 		break;
 	} 
 	if(uid != -1) {
@@ -1018,26 +918,29 @@ void print_user(type_t type, void *object)
 
 void print_usercpu(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char str[FORMAT_STRING_SIZE];
 	
 	switch(type) {
 	case HEADLINE:
-		printf("%15s", "UserCpu");
+		printf("%-15s", "UserCpu");
 		break;
 	case UNDERSCORE:
-		printf("%15s", "---------------");
+		printf("%-15s", "---------------");
 		break;
 	case JOB:
-		_elapsed_time(job->rusage.ru_utime.tv_sec,
-			      job->rusage.ru_utime.tv_usec, str);
-		printf("%15s", str);
+		_elapsed_time(job->user_cpu_sec,
+			      job->user_cpu_usec, str);
+		printf("%-15s", str);
 		break;
 	case JOBSTEP:
-		_elapsed_time(step->rusage.ru_utime.tv_sec,
-			      step->rusage.ru_utime.tv_usec, str);
-		printf("%15s", str);
+		_elapsed_time(step->user_cpu_sec,
+			      step->user_cpu_usec, str);
+		printf("%-15s", str);
+		break;
+	default:
+		printf("%-15s", "n/a");
 		break;
 	} 
 
@@ -1045,34 +948,34 @@ void print_usercpu(type_t type, void *object)
 
 void print_vsize(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char outbuf[FORMAT_STRING_SIZE];
 	char buf1[FORMAT_STRING_SIZE];
 	char buf2[FORMAT_STRING_SIZE];
-	char buf3[50];
+	char buf3[FORMAT_STRING_SIZE];
 	sacct_t sacct;
 	char *nodes = NULL;
 	uint32_t pos;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%-50s", "MaxVSIZE/Node:Task - Ave");
+		printf("%-34s", "MaxVSIZE/Node:Task - Ave");
 		break;
 	case UNDERSCORE:
-		printf("%-50s", "----------------------------------");
+		printf("%-34s", "----------------------------------");
 		break;
 	case JOB:
 		sacct = job->sacct;
 		nodes = job->nodes;
 		pos = sacct.min_cpu_id.nodeid;				 
 		convert_num_unit((float)sacct.max_vsize, 
-				 buf1, sizeof(buf1),UNIT_NONE);
+				 buf1, sizeof(buf1), UNIT_KILO);
 		if(job->track_steps)
 			snprintf(outbuf, FORMAT_STRING_SIZE, "%s/- - -", buf1);
 		else {
 			convert_num_unit((float)sacct.ave_vsize,
-					 buf2, sizeof(buf2), UNIT_NONE);
+					 buf2, sizeof(buf2), UNIT_KILO);
 			find_hostname(pos, nodes, buf3);
 			snprintf(outbuf, FORMAT_STRING_SIZE, "%s/%s:%u - %s", 
 				 buf1,
@@ -1080,45 +983,48 @@ void print_vsize(type_t type, void *object)
 				 sacct.max_vsize_id.taskid, 
 				 buf2);
 		}
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
 		break;
 	case JOBSTEP:
 		sacct = step->sacct;
 		nodes = step->nodes;
 		pos = sacct.min_cpu_id.nodeid;				 
 		convert_num_unit((float)sacct.max_vsize, buf1, sizeof(buf1), 
-				 UNIT_NONE);
+				 UNIT_KILO);
 		convert_num_unit((float)sacct.ave_vsize, buf2, sizeof(buf2),
-				 UNIT_NONE);
+				 UNIT_KILO);
 		find_hostname(pos, nodes, buf3);
 		snprintf(outbuf, FORMAT_STRING_SIZE, "%s/%s:%u - %s", 
 			 buf1,
 			 buf3, 
 			 sacct.max_vsize_id.taskid, 
 			 buf2);
-		printf("%-50s", outbuf);
+		printf("%-34s", outbuf);
+		break;
+	default:
+		printf("%-34s", "n/a");
 		break;
 	} 
 }
 
 void print_cputime(type_t type, void *object)
 { 
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 	char outbuf[FORMAT_STRING_SIZE];
 	char buf1[FORMAT_STRING_SIZE];
 	char buf2[FORMAT_STRING_SIZE];
-	char buf3[50];
+	char buf3[FORMAT_STRING_SIZE];
 	sacct_t sacct;
 	char *nodes = NULL;
 	uint32_t pos;
 
 	switch(type) {
 	case HEADLINE:
-		printf("%-50s", "MinCPUtime/Node:Task - Ave");
+		printf("%-37s", "MinCPUtime/Node:Task - Ave");
 		break;
 	case UNDERSCORE:
-		printf("%-50s", "------------------------------------");
+		printf("%-37s", "-------------------------------------");
 		break;
 	case JOB:
 		sacct = job->sacct;
@@ -1138,7 +1044,7 @@ void print_cputime(type_t type, void *object)
 				 sacct.min_cpu_id.taskid, 
 				 buf2);
 		}
-		printf("%-50s", outbuf);
+		printf("%-37s", outbuf);
 		break;
 	case JOBSTEP:
 		sacct = step->sacct;
@@ -1153,15 +1059,18 @@ void print_cputime(type_t type, void *object)
 			 buf3, 
 			 sacct.min_cpu_id.taskid, 
 			 buf2);
-		printf("%-50s", outbuf);
+		printf("%-37s", outbuf);
+		break;
+	default:
+		printf("%-37s", "n/a");
 		break;
 	} 
 }
 
 void print_account(type_t type, void *object)
 {
-	job_rec_t *job = (job_rec_t *)object;
-	step_rec_t *step = (step_rec_t *)object;
+	jobacct_job_rec_t *job = (jobacct_job_rec_t *)object;
+	jobacct_step_rec_t *step = (jobacct_step_rec_t *)object;
 
 	switch(type) {
 	case HEADLINE:
@@ -1186,6 +1095,128 @@ void print_account(type_t type, void *object)
 		else
 			printf("%-13.13s...", step->account);
 		break;
+	default:
+		printf("%-16s", "n/a");
+		break;
+		break;
 	}
 }
 
+
+#ifdef HAVE_BG
+void print_connection(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-10s", "Connection");
+		break;
+	case UNDERSCORE:
+		printf("%-10s", "----------");
+		break;
+	case JOBCOMP:
+		printf("%-10s", job->connection);
+		break;
+	default:
+		printf("%-10s", "n/a");
+		break;
+	}
+}
+void print_geo(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-8s", "Geometry");
+		break;
+	case UNDERSCORE:
+		printf("%-8s", "--------");
+		break;
+	case JOBCOMP:
+		printf("%-8s", job->geo);
+		break;
+	default:
+		printf("%-8s", "n/a");
+		break;
+	}
+}
+void print_max_procs(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-9s", "Max Procs");
+		break;
+	case UNDERSCORE:
+		printf("%-9s", "---------");
+		break;
+	case JOBCOMP:
+		printf("%-9d", job->max_procs);
+		break;
+	default:
+		printf("%-9s", "n/a");
+		break;
+	}
+}
+void print_reboot(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-6s", "Reboot");
+		break;
+	case UNDERSCORE:
+		printf("%-6s", "------");
+		break;
+	case JOBCOMP:
+		printf("%-6s", job->reboot);
+		break;
+	default:
+		printf("%-6s", "n/a");
+		break;
+	}
+}
+void print_rotate(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-6s", "Rotate");
+		break;
+	case UNDERSCORE:
+		printf("%-6s", "------");
+		break;
+	case JOBCOMP:
+		printf("%-6s", job->rotate);
+		break;
+	default:
+		printf("%-6s", "n/a");
+		break;
+	}
+}
+void print_bg_start_point(type_t type, void *object)
+{
+	jobcomp_job_rec_t *job = (jobcomp_job_rec_t *)object;
+
+	switch(type) {
+	case HEADLINE:
+		printf("%-14s", "BG Start Point");
+		break;
+	case UNDERSCORE:
+		printf("%-14s", "--------------");
+		break;
+	case JOBCOMP:
+		printf("%-14s", job->bg_start_point);
+		break;
+	default:
+		printf("%-14s", "n/a");
+		break;
+	}
+}
+#endif
+
