@@ -1,10 +1,11 @@
 /*****************************************************************************\
  *  opt.c - options processing for salloc
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <grondona1@llnl.gov>, et. al.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -60,6 +61,7 @@
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/parse_time.h"
+#include "src/common/proc_args.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
@@ -86,6 +88,7 @@
 #define OPT_JOBID       0x11
 #define OPT_EXCLUSIVE   0x12
 #define OPT_OVERCOMMIT  0x13
+#define OPT_ACCTG_FREQ  0x14
 
 /* generic getopt_long flags, integers and *not* valid characters */
 #define LONG_OPT_JOBID       0x105
@@ -113,6 +116,18 @@
 #define LONG_OPT_LINUX_IMAGE     0x121
 #define LONG_OPT_MLOADER_IMAGE   0x122
 #define LONG_OPT_RAMDISK_IMAGE   0x123
+#define LONG_OPT_NOSHELL         0x124
+#define LONG_OPT_GET_USER_ENV    0x125
+#define LONG_OPT_NETWORK         0x126
+#define LONG_OPT_SOCKETSPERNODE  0x130
+#define LONG_OPT_CORESPERSOCKET  0x131
+#define LONG_OPT_THREADSPERCORE  0x132
+#define LONG_OPT_NTASKSPERNODE   0x136
+#define LONG_OPT_NTASKSPERSOCKET 0x137
+#define LONG_OPT_NTASKSPERCORE   0x138
+#define LONG_OPT_MEM_PER_CPU     0x13a
+#define LONG_OPT_HINT            0x13b
+#define LONG_OPT_ACCTG_FREQ      0x13c
 
 /*---- global variables, defined in opt.h ----*/
 opt_t opt;
@@ -121,9 +136,6 @@ opt_t opt;
 
 typedef struct env_vars env_vars_t;
 
-/* return command name from its full path name */
-static char * _base_name(char* command);
-
 /* Get a decimal integer from arg */
 static int  _get_int(const char *arg, const char *what);
 
@@ -143,16 +155,10 @@ static void  _opt_list(void);
 /* verify options sanity  */
 static bool _opt_verify(void);
 
-static void  _print_version(void);
+static void  _proc_get_user_env(char *optarg);
 static void _process_env_var(env_vars_t *e, const char *val);
-static uint16_t _parse_mail_type(const char *arg);
-static char *_print_mail_type(const uint16_t type);
 static int _parse_signal(const char *signal_name);
-static long  _to_bytes(const char *arg);
 static void  _usage(void);
-static bool  _verify_node_count(const char *arg, int *min, int *max);
-static int   _verify_geometry(const char *arg, uint16_t *geometry);
-static int   _verify_conn_type(const char *arg);
 
 /*---[ end forward declarations of static functions ]---------------------*/
 
@@ -174,216 +180,6 @@ int initialize_and_process_args(int argc, char *argv[])
 
 }
 
-static void _print_version(void)
-{
-	printf("%s %s\n", PACKAGE, SLURM_VERSION);
-}
-
-/*
- * verify that a connection type in arg is of known form
- * returns the connection_type or -1 if not recognized
- */
-static int _verify_conn_type(const char *arg)
-{
-	int len = strlen(arg);
-
-	if (!strncasecmp(arg, "MESH", len))
-		return SELECT_MESH;
-	else if (!strncasecmp(arg, "TORUS", len))
-		return SELECT_TORUS;
-	else if (!strncasecmp(arg, "NAV", len))
-		return SELECT_NAV;
-
-	error("invalid --conn-type argument %s ignored.", arg);
-	return -1;
-}
-
-/*
- * verify geometry arguments, must have proper count
- * returns -1 on error, 0 otherwise
- */
-static int _verify_geometry(const char *arg, uint16_t *geometry)
-{
-	char* token, *delimiter = ",x", *next_ptr;
-	int i, rc = 0;
-	char* geometry_tmp = xstrdup(arg);
-	char* original_ptr = geometry_tmp;
-
-	token = strtok_r(geometry_tmp, delimiter, &next_ptr);
-	for (i=0; i<SYSTEM_DIMENSIONS; i++) {
-		if (token == NULL) {
-			error("insufficient dimensions in --geometry");
-			rc = -1;
-			break;
-		}
-		geometry[i] = (uint16_t)atoi(token);
-		if (geometry[i] == 0 || geometry[i] == (uint16_t)NO_VAL) {
-			error("invalid --geometry argument");
-			rc = -1;
-			break;
-		}
-		geometry_tmp = next_ptr;
-		token = strtok_r(geometry_tmp, delimiter, &next_ptr);
-	}
-	if (token != NULL) {
-		error("too many dimensions in --geometry");
-		rc = -1;
-	}
-
-	if (original_ptr)
-		xfree(original_ptr);
-
-	return rc;
-}
-
-/* Convert a string into a node count */
-static int
-_str_to_nodes(const char *num_str, char **leftover)
-{
-	long int num;
-	char *endptr;
-
-	num = strtol(num_str, &endptr, 10);
-	if (endptr == num_str) { /* no valid digits */
-		*leftover = (char *)num_str;
-		return 0;
-	} 
-	if (*endptr != '\0' && (*endptr == 'k' || *endptr == 'K')) {
-		num *= 1024;
-		endptr++;
-	}
-	*leftover = endptr;
-
-	return (int)num;
-}
-
-/* 
- * verify that a node count in arg is of a known form (count or min-max)
- * OUT min, max specified minimum and maximum node counts
- * RET true if valid
- */
-static bool 
-_verify_node_count(const char *arg, int *min_nodes, int *max_nodes)
-{
-	char *ptr, *min_str, *max_str;
-	char *leftover;
-	
-	/* Does the string contain a "-" character?  If so, treat as a range.
-	 * otherwise treat as an absolute node count. */
-	if ((ptr = index(arg, '-')) != NULL) {
-		min_str = xstrndup(arg, ptr-arg);
-		*min_nodes = _str_to_nodes(min_str, &leftover);
-		if (!xstring_is_whitespace(leftover)) {
-			error("\"%s\" is not a valid node count", min_str);
-			xfree(min_str);
-			return false;
-		}
-		xfree(min_str);
-		if (*min_nodes == 0)
-			*min_nodes = 1;
-
-		max_str = xstrndup(ptr+1, strlen(arg)-((ptr+1)-arg));
-		*max_nodes = _str_to_nodes(max_str, &leftover);
-		if (!xstring_is_whitespace(leftover)) {
-			error("\"%s\" is not a valid node count", max_str);
-			xfree(max_str);
-			return false;
-		}
-		xfree(max_str);
-	} else {
-		*min_nodes = *max_nodes = _str_to_nodes(arg, &leftover);
-		if (!xstring_is_whitespace(leftover)) {
-			error("\"%s\" is not a valid node count", arg);
-			return false;
-		}
-		if (*min_nodes == 0) {
-			/* whitespace does not a valid node count make */
-			error("\"%s\" is not a valid node count", arg);
-			return false;
-		}
-	}
-
-	if ((*max_nodes != 0) && (*max_nodes < *min_nodes)) {
-		error("Maximum node count %d is less than"
-		      " minimum node count %d",
-		      *max_nodes, *min_nodes);
-		return false;
-	}
-
-	return true;
-}
-
-/* return command name from its full path name */
-static char * _base_name(char* command)
-{
-	char *char_ptr, *name;
-	int i;
-
-	if (command == NULL)
-		return NULL;
-
-	char_ptr = strrchr(command, (int)'/');
-	if (char_ptr == NULL)
-		char_ptr = command;
-	else
-		char_ptr++;
-
-	i = strlen(char_ptr);
-	name = xmalloc(i+1);
-	strcpy(name, char_ptr);
-	return name;
-}
-
-/*
- * _to_bytes(): verify that arg is numeric with optional "G" or "M" at end
- * if "G" or "M" is there, multiply by proper power of 2 and return
- * number in bytes
- */
-static long _to_bytes(const char *arg)
-{
-	char *buf;
-	char *endptr;
-	int end;
-	int multiplier = 1;
-	long result;
-
-	buf = xstrdup(arg);
-
-	end = strlen(buf) - 1;
-
-	if (isdigit(buf[end])) {
-		result = strtol(buf, &endptr, 10);
-
-		if (*endptr != '\0')
-			result = -result;
-
-	} else {
-
-		switch (toupper(buf[end])) {
-
-		case 'G':
-			multiplier = 1024;
-			break;
-
-		case 'M':
-			/* do nothing */
-			break;
-
-		default:
-			multiplier = -1;
-		}
-
-		buf[end] = '\0';
-
-		result = multiplier * strtol(buf, &endptr, 10);
-
-		if (*endptr != '\0')
-			result = -result;
-	}
-
-	return result;
-}
-
 /*
  * print error message to stderr with opt.progname prepended
  */
@@ -421,6 +217,7 @@ static void _opt_default()
 
 	opt.gid = getgid();
 
+	opt.cwd = NULL;
 	opt.progname = NULL;
 
 	opt.nprocs = 1;
@@ -430,16 +227,29 @@ static void _opt_default()
 	opt.min_nodes = 1;
 	opt.max_nodes = 0;
 	opt.nodes_set = false;
+	opt.min_sockets_per_node = NO_VAL; /* requested min/maxsockets */
+	opt.max_sockets_per_node = NO_VAL;
+	opt.min_cores_per_socket = NO_VAL; /* requested min/maxcores */
+	opt.max_cores_per_socket = NO_VAL;
+	opt.min_threads_per_core = NO_VAL; /* requested min/maxthreads */
+	opt.max_threads_per_core = NO_VAL;
+	opt.ntasks_per_node      = NO_VAL; /* ntask max limits */
+	opt.ntasks_per_socket    = NO_VAL;
+	opt.ntasks_per_core      = NO_VAL;
+	opt.cpu_bind_type = 0;		/* local dummy variable for now */
 	opt.time_limit = NO_VAL;
 	opt.time_limit_str = NULL;
 	opt.partition = NULL;
 
 	opt.job_name = NULL;
 	opt.jobid = NO_VAL;
-	opt.dependency = NO_VAL;
+	opt.dependency = NULL;
 	opt.account  = NULL;
 	opt.comment  = NULL;
 
+	opt.distribution = SLURM_DIST_UNKNOWN;
+	opt.plane_size   = NO_VAL;
+
 	opt.shared = (uint16_t)NO_VAL;
 	opt.no_kill = false;
 	opt.kill_command_signal = SIGTERM;
@@ -457,6 +267,7 @@ static void _opt_default()
 	opt.minsockets      = -1;
 	opt.mincores        = -1;
 	opt.minthreads      = -1;
+	opt.mem_per_cpu	    = -1;
 	opt.realmem	    = -1;
 	opt.tmpdisk	    = -1;
 
@@ -470,12 +281,16 @@ static void _opt_default()
 		opt.geometry[i]	    = (uint16_t) NO_VAL;
 	opt.reboot          = false;
 	opt.no_rotate	    = false;
-	opt.conn_type	    = -1;
+	opt.conn_type	    = (uint16_t) NO_VAL;
 
 	opt.euid	    = (uid_t) -1;
 	opt.egid	    = (gid_t) -1;
 	
 	opt.bell            = BELL_AFTER_DELAY;
+	opt.acctg_freq      = -1;
+	opt.no_shell	    = false;
+	opt.get_user_env_time = -1;
+	opt.get_user_env_mode = -1;
 }
 
 /*---[ env var processing ]-----------------------------------------------*/
@@ -511,6 +326,8 @@ env_vars_t env_vars[] = {
   {"SALLOC_NO_BELL",       OPT_NO_BELL,    NULL,               NULL           },
   {"SALLOC_EXCLUSIVE",     OPT_EXCLUSIVE,  NULL,               NULL           },
   {"SALLOC_OVERCOMMIT",    OPT_OVERCOMMIT, NULL,               NULL           },
+  {"SALLOC_ACCTG_FREQ",    OPT_INT,        &opt.acctg_freq,    NULL           },
+  {"SALLOC_NETWORK",       OPT_STRING    , &opt.network,       NULL           },
   {NULL, 0, NULL, NULL}
 };
 
@@ -583,16 +400,16 @@ _process_env_var(env_vars_t *e, const char *val)
 		break;
 
 	case OPT_NODES:
-		opt.nodes_set = _verify_node_count( val, 
-						    &opt.min_nodes, 
-						    &opt.max_nodes );
+		opt.nodes_set = verify_node_count( val, 
+						   &opt.min_nodes, 
+						   &opt.max_nodes );
 		if (opt.nodes_set == false) {
 			error("invalid node count in env variable, ignoring");
 		}
 		break;
 
 	case OPT_CONN_TYPE:
-		opt.conn_type = _verify_conn_type(val);
+		opt.conn_type = verify_conn_type(val);
 		break;
 
 	case OPT_NO_ROTATE:
@@ -600,7 +417,7 @@ _process_env_var(env_vars_t *e, const char *val)
 		break;
 
 	case OPT_GEOMETRY:
-		if (_verify_geometry(val, opt.geometry)) {
+		if (verify_geometry(val, opt.geometry)) {
 			error("\"%s=%s\" -- invalid geometry, ignoring...",
 			      e->var, val);
 		}
@@ -657,9 +474,10 @@ void set_options(const int argc, char **argv)
 	int opt_char, option_index = 0;
 	char *tmp;
 	static struct option long_options[] = {
+		{"extra-node-info", required_argument, 0, 'B'},
 		{"cpus-per-task", required_argument, 0, 'c'},
 		{"constraint",    required_argument, 0, 'C'},
-		{"dependency",    required_argument, 0, 'd'},
+		{"chdir",         required_argument, 0, 'D'},
 		{"nodefile",      required_argument, 0, 'F'},
 		{"geometry",      required_argument, 0, 'g'},
 		{"help",          no_argument,       0, 'h'},
@@ -668,10 +486,14 @@ void set_options(const int argc, char **argv)
 		{"job-name",      required_argument, 0, 'J'},
 		{"no-kill",       no_argument,       0, 'k'},
 		{"kill-command",  optional_argument, 0, 'K'},
+		{"licenses",      required_argument, 0, 'L'},
+		{"distribution",  required_argument, 0, 'm'},
 		{"tasks",         required_argument, 0, 'n'},
+		{"ntasks",        required_argument, 0, 'n'},
 		{"nodes",         required_argument, 0, 'N'},
 		{"overcommit",    no_argument,       0, 'O'},
 		{"partition",     required_argument, 0, 'p'},
+		{"dependency",    required_argument, 0, 'P'},
 		{"quiet",         no_argument,       0, 'q'},
 		{"no-rotate",     no_argument,       0, 'R'},
 		{"share",         no_argument,       0, 's'},
@@ -690,6 +512,17 @@ void set_options(const int argc, char **argv)
 		{"mincores",      required_argument, 0, LONG_OPT_MINCORES},
 		{"minthreads",    required_argument, 0, LONG_OPT_MINTHREADS},
 		{"mem",           required_argument, 0, LONG_OPT_MEM},
+		{"job-mem",       required_argument, 0, LONG_OPT_MEM_PER_CPU},
+		{"task-mem",      required_argument, 0, LONG_OPT_MEM_PER_CPU},
+		{"mem-per-cpu",   required_argument, 0, LONG_OPT_MEM_PER_CPU},
+		{"hint",          required_argument, 0, LONG_OPT_HINT},
+		{"sockets-per-node", required_argument, 0, LONG_OPT_SOCKETSPERNODE},
+		{"cores-per-socket", required_argument, 0, LONG_OPT_CORESPERSOCKET},
+		{"threads-per-core", required_argument, 0, LONG_OPT_THREADSPERCORE},
+		{"ntasks-per-node",  required_argument, 0, LONG_OPT_NTASKSPERNODE},
+		{"ntasks-per-socket",required_argument, 0, LONG_OPT_NTASKSPERSOCKET},
+		{"ntasks-per-core",  required_argument, 0, LONG_OPT_NTASKSPERCORE},
+		{"tasks-per-node",  required_argument, 0, LONG_OPT_NTASKSPERNODE},
 		{"tmp",           required_argument, 0, LONG_OPT_TMP},
 		{"uid",           required_argument, 0, LONG_OPT_UID},
 		{"gid",           required_argument, 0, LONG_OPT_GID},
@@ -707,9 +540,13 @@ void set_options(const int argc, char **argv)
 		{"linux-image",   required_argument, 0, LONG_OPT_LINUX_IMAGE},
 		{"mloader-image", required_argument, 0, LONG_OPT_MLOADER_IMAGE},
 		{"ramdisk-image", required_argument, 0, LONG_OPT_RAMDISK_IMAGE},
+		{"acctg-freq",    required_argument, 0, LONG_OPT_ACCTG_FREQ},
+		{"no-shell",      no_argument,       0, LONG_OPT_NOSHELL},
+		{"get-user-env",  optional_argument, 0, LONG_OPT_GET_USER_ENV},
+		{"network",       required_argument, 0, LONG_OPT_NETWORK},
 		{NULL,            0,                 0, 0}
 	};
-	char *opt_string = "+a:c:C:d:F:g:hHIJ:kK::n:N:Op:qR:st:uU:vVw:W:x:";
+	char *opt_string = "+a:B:c:C:d:D:F:g:hHIJ:kK:L:m:n:N:Op:P:qR:st:uU:vVw:W:x:";
 
 	opt.progname = xbasename(argv[0]);
 	optind = 0;		
@@ -722,6 +559,24 @@ void set_options(const int argc, char **argv)
 				"information\n");
 			exit(1);
 			break;
+		case 'B':
+			opt.extra_set = verify_socket_core_thread_count(
+				optarg,
+				&opt.min_sockets_per_node,
+				&opt.max_sockets_per_node,
+				&opt.min_cores_per_socket,
+				&opt.max_cores_per_socket,
+				&opt.min_threads_per_core,
+				&opt.max_threads_per_core,
+				&opt.cpu_bind_type);
+
+
+			if (opt.extra_set == false) {
+				error("invalid resource allocation -B `%s'",
+					optarg);
+				exit(1);
+			}
+			break;
 		case 'c':
 			opt.cpus_set = true;
 			opt.cpus_per_task = 
@@ -731,8 +586,10 @@ void set_options(const int argc, char **argv)
 			xfree(opt.constraints);
 			opt.constraints = xstrdup(optarg);
 			break;
-		case 'd':
-			opt.dependency = _get_int(optarg, "dependency");
+/* 		case 'd':	see 'P' below */
+		case 'D':
+			xfree(opt.cwd);
+			opt.cwd = xstrdup(optarg);
 			break;
 		case 'F':
 			xfree(opt.nodelist);
@@ -746,7 +603,7 @@ void set_options(const int argc, char **argv)
 			}
 			break;
 		case 'g':
-			if (_verify_geometry(optarg, opt.geometry))
+			if (verify_geometry(optarg, opt.geometry))
 				exit(1);
 			break;
 		case 'h':
@@ -773,6 +630,19 @@ void set_options(const int argc, char **argv)
 			}
 			opt.kill_command_signal_set = true;
 			break;
+		case 'L':
+			xfree(opt.licenses);
+			opt.licenses = xstrdup(optarg);
+			break;
+		case 'm':
+			opt.distribution = verify_dist_type(optarg, 
+							    &opt.plane_size);
+			if (opt.distribution == SLURM_DIST_UNKNOWN) {
+				error("distribution type `%s' " 
+				      "is not recognized", optarg);
+				exit(1);
+			}
+			break;
 		case 'n':
 			opt.nprocs_set = true;
 			opt.nprocs = 
@@ -780,9 +650,9 @@ void set_options(const int argc, char **argv)
 			break;
 		case 'N':
 			opt.nodes_set = 
-				_verify_node_count(optarg, 
-						   &opt.min_nodes,
-						   &opt.max_nodes);
+				verify_node_count(optarg, 
+						  &opt.min_nodes,
+						  &opt.max_nodes);
 			if (opt.nodes_set == false) {
 				exit(1);
 			}
@@ -794,6 +664,11 @@ void set_options(const int argc, char **argv)
 			xfree(opt.partition);
 			opt.partition = xstrdup(optarg);
 			break;
+		case 'd':
+		case 'P':
+			xfree(opt.dependency);
+			opt.dependency = xstrdup(optarg);
+			break;
 		case 'q':
 			opt.quiet++;
 			break;
@@ -818,7 +693,7 @@ void set_options(const int argc, char **argv)
 			opt.verbose++;
 			break;
 		case 'V':
-			_print_version();
+			print_slurm_version();
 			exit(0);
 			break;
 		case 'w':
@@ -827,6 +702,9 @@ void set_options(const int argc, char **argv)
 #ifdef HAVE_BG
 			info("\tThe nodelist option should only be used if\n"
 			     "\tthe block you are asking for can be created.\n"
+			     "\tIt should also include all the midplanes you\n"
+			     "\twant to use, partial lists may not\n"
+			     "\twork correctly.\n"
 			     "\tPlease consult smap before using this option\n"
 			     "\tor your job may be stuck with no way to run.");
 #endif
@@ -877,32 +755,44 @@ void set_options(const int argc, char **argv)
 			}
 			break;
 		case LONG_OPT_MEM:
-			opt.realmem = (int) _to_bytes(optarg);
+			opt.realmem = (int) str_to_bytes(optarg);
 			if (opt.realmem < 0) {
 				error("invalid memory constraint %s", 
 				      optarg);
 				exit(1);
 			}
 			break;
+		case LONG_OPT_MEM_PER_CPU:
+			opt.mem_per_cpu = (int) str_to_bytes(optarg);
+			if (opt.mem_per_cpu < 0) {
+				error("invalid memory constraint %s", 
+				      optarg);
+				exit(1);
+			}
+			break;
 		case LONG_OPT_TMP:
-			opt.tmpdisk = _to_bytes(optarg);
+			opt.tmpdisk = str_to_bytes(optarg);
 			if (opt.tmpdisk < 0) {
 				error("invalid tmp value %s", optarg);
 				exit(1);
 			}
 			break;
 		case LONG_OPT_UID:
+			if (opt.euid != (uid_t) -1)
+				fatal ("duplicate --uid option");
 			opt.euid = uid_from_string (optarg);
 			if (opt.euid == (uid_t) -1)
 				fatal ("--uid=\"%s\" invalid", optarg);
 			break;
 		case LONG_OPT_GID:
+			if (opt.egid != (gid_t) -1)
+				fatal ("duplicate --gid option");
 			opt.egid = gid_from_string (optarg);
 			if (opt.egid == (gid_t) -1)
 				fatal ("--gid=\"%s\" invalid", optarg);
 			break;
 		case LONG_OPT_CONNTYPE:
-			opt.conn_type = _verify_conn_type(optarg);
+			opt.conn_type = verify_conn_type(optarg);
 			break;
 		case LONG_OPT_BEGIN:
 			opt.begin = parse_time(optarg);
@@ -912,7 +802,7 @@ void set_options(const int argc, char **argv)
 			}
 			break;
 		case LONG_OPT_MAIL_TYPE:
-			opt.mail_type |= _parse_mail_type(optarg);
+			opt.mail_type |= parse_mail_type(optarg);
 			if (opt.mail_type == 0)
 				fatal("--mail-type=%s invalid", optarg);
 			break;
@@ -953,6 +843,48 @@ void set_options(const int argc, char **argv)
 			xfree(opt.comment);
 			opt.comment = xstrdup(optarg);
 			break;
+		case LONG_OPT_SOCKETSPERNODE:
+			get_resource_arg_range( optarg, "sockets-per-node",
+						&opt.min_sockets_per_node,
+						&opt.max_sockets_per_node,
+						true );
+			break;
+		case LONG_OPT_CORESPERSOCKET:
+			get_resource_arg_range( optarg, "cores-per-socket",
+						&opt.min_cores_per_socket,
+						&opt.max_cores_per_socket,
+						true);
+			break;
+		case LONG_OPT_THREADSPERCORE:
+			get_resource_arg_range( optarg, "threads-per-core",
+						&opt.min_threads_per_core,
+						&opt.max_threads_per_core,
+						true );
+			break;
+		case LONG_OPT_HINT:
+			if (verify_hint(optarg,
+				&opt.min_sockets_per_node,
+				&opt.max_sockets_per_node,
+				&opt.min_cores_per_socket,
+				&opt.max_cores_per_socket,
+				&opt.min_threads_per_core,
+				&opt.max_threads_per_core,
+				&opt.cpu_bind_type)) {
+				exit(1);
+			}
+			break;
+		case LONG_OPT_NTASKSPERNODE:
+			opt.ntasks_per_node = _get_int(optarg,
+				"ntasks-per-node");
+			break;
+		case LONG_OPT_NTASKSPERSOCKET:
+			opt.ntasks_per_socket = _get_int(optarg, 
+				"ntasks-per-socket");
+			break;
+		case LONG_OPT_NTASKSPERCORE:
+			opt.ntasks_per_core = _get_int(optarg,
+				"ntasks-per-core");
+			break;
 		case LONG_OPT_REBOOT:
 			opt.reboot = true;
 			break;
@@ -972,6 +904,22 @@ void set_options(const int argc, char **argv)
 			xfree(opt.ramdiskimage);
 			opt.ramdiskimage = xstrdup(optarg);
 			break;
+		case LONG_OPT_ACCTG_FREQ:
+			opt.acctg_freq = _get_int(optarg, "acctg-freq");
+			break;
+		case LONG_OPT_NOSHELL:
+			opt.no_shell = true;
+			break;
+		case LONG_OPT_GET_USER_ENV:
+			if (optarg)
+				_proc_get_user_env(optarg);
+			else
+				opt.get_user_env_time = 0;
+			break;
+		case LONG_OPT_NETWORK:
+			xfree(opt.network);
+			opt.network = xstrdup(optarg);
+			break;
 		default:
 			fatal("Unrecognized command line parameter %c",
 			      opt_char);
@@ -979,6 +927,24 @@ void set_options(const int argc, char **argv)
 	}
 }
 
+static void _proc_get_user_env(char *optarg)
+{
+	char *end_ptr;
+
+	if ((optarg[0] >= '0') && (optarg[0] <= '9'))
+		opt.get_user_env_time = strtol(optarg, &end_ptr, 10);
+	else {
+		opt.get_user_env_time = 0;
+		end_ptr = optarg;
+	}
+ 
+	if ((end_ptr == NULL) || (end_ptr[0] == '\0'))
+		return;
+	if      ((end_ptr[0] == 's') || (end_ptr[0] == 'S'))
+		opt.get_user_env_mode = 1;
+	else if ((end_ptr[0] == 'l') || (end_ptr[0] == 'L'))
+		opt.get_user_env_mode = 2;
+}
 
 /*
  * _opt_args() : set options via commandline args and popt
@@ -1022,9 +988,9 @@ static bool _opt_verify(void)
 		opt.mincpus = opt.cpus_per_task;
 
 	if ((opt.job_name == NULL) && (command_argc > 0))
-		opt.job_name = _base_name(command_argv[0]);
+		opt.job_name = base_name(command_argv[0]);
 
-	if (command_argc == 0) {
+	if ((opt.no_shell == false) && (command_argc == 0)) {
 		error("A local command is a required parameter!");
 		verified = false;
 	}
@@ -1050,11 +1016,84 @@ static bool _opt_verify(void)
 		verified = false;
 	}
 
+	if ((opt.realmem > -1) && (opt.mem_per_cpu > -1)) {
+		if (opt.realmem < opt.mem_per_cpu) {
+			info("mem < mem-per-cpu - resizing mem to be equal "
+			     "to mem-per-cpu");
+			opt.realmem = opt.mem_per_cpu;
+		}
+	}
+	
+        /* Check to see if user has specified enough resources to
+	 * satisfy the plane distribution with the specified
+	 * plane_size.  
+	 * if (n/plane_size < N) and ((N-1) * plane_size >= n) -->
+	 * problem Simple check will not catch all the problem/invalid
+	 * cases.
+	 * The limitations of the plane distribution in the cons_res
+	 * environment are more extensive and are documented in the
+	 * SLURM reference guide.  */
+	if (opt.distribution == SLURM_DIST_PLANE && opt.plane_size) {
+		if ((opt.nprocs/opt.plane_size) < opt.min_nodes) {
+			if (((opt.min_nodes-1)*opt.plane_size) >= opt.nprocs) {
+#if(0)
+				info("Too few processes ((n/plane_size) %d < N %d) "
+				     "and ((N-1)*(plane_size) %d >= n %d)) ",
+				     opt.nprocs/opt.plane_size, opt.min_nodes, 
+				     (opt.min_nodes-1)*opt.plane_size, opt.nprocs);
+#endif
+				error("Too few processes for the requested "
+				      "{plane,node} distribution");
+				exit(1);
+			}
+		}
+	}
+
+	/* bound max_threads/cores from ntasks_cores/sockets */ 
+	if ((opt.max_threads_per_core <= 0) &&
+	    (opt.ntasks_per_core > 0)) {
+		opt.max_threads_per_core = opt.ntasks_per_core;
+		/* if cpu_bind_type doesn't already have a auto pref,
+		 * choose the level based on the level of ntasks
+		 */
+		if (!(opt.cpu_bind_type & (CPU_BIND_TO_SOCKETS |
+					   CPU_BIND_TO_CORES |
+					   CPU_BIND_TO_THREADS))) {
+			opt.cpu_bind_type |= CPU_BIND_TO_CORES;
+		}
+	}
+	if ((opt.max_cores_per_socket <= 0) &&
+	    (opt.ntasks_per_socket > 0)) {
+		opt.max_cores_per_socket = opt.ntasks_per_socket;
+		/* if cpu_bind_type doesn't already have a auto pref,
+		 * choose the level based on the level of ntasks
+		 */
+		if (!(opt.cpu_bind_type & (CPU_BIND_TO_SOCKETS |
+					   CPU_BIND_TO_CORES |
+					   CPU_BIND_TO_THREADS))) {
+			opt.cpu_bind_type |= CPU_BIND_TO_SOCKETS;
+		}
+	}
+
 	/* massage the numbers */
 	if (opt.nodes_set && !opt.nprocs_set) {
 		/* 1 proc / node default */
 		opt.nprocs = opt.min_nodes;
 
+		/* 1 proc / min_[socket * core * thread] default */
+		if (opt.min_sockets_per_node > 0) {
+			opt.nprocs *= opt.min_sockets_per_node;
+			opt.nprocs_set = true;
+		}
+		if (opt.min_cores_per_socket > 0) {
+			opt.nprocs *= opt.min_cores_per_socket;
+			opt.nprocs_set = true;
+		}
+		if (opt.min_threads_per_core > 0) {
+			opt.nprocs *= opt.min_threads_per_core;
+			opt.nprocs_set = true;
+		}
+
 	} else if (opt.nodes_set && opt.nprocs_set) {
 
 		/* 
@@ -1076,7 +1115,7 @@ static bool _opt_verify(void)
 
 	if (opt.time_limit_str) {
 		opt.time_limit = time_str2mins(opt.time_limit_str);
-		if (opt.time_limit < 0) {
+		if ((opt.time_limit < 0) && (opt.time_limit != INFINITE)) {
 			error("Invalid time limit specification");
 			exit(1);
 		}
@@ -1090,9 +1129,6 @@ static bool _opt_verify(void)
 	if ((opt.egid != (gid_t) -1) && (opt.egid != opt.gid)) 
 		opt.gid = opt.egid;
 
-        if ((opt.egid != (gid_t) -1) && (opt.egid != opt.gid))
-	        opt.gid = opt.egid;
-
 	if (opt.immediate) {
 		char *sched_name = slurm_get_sched_type();
 		if (strcmp(sched_name, "sched/wiki") == 0) {
@@ -1103,43 +1139,14 @@ static bool _opt_verify(void)
 		xfree(sched_name);
 	}
 
+#ifdef HAVE_AIX
+	if (opt.network == NULL)
+		opt.network = "us,sn_all,bulk_xfer";
+#endif
+
 	return verified;
 }
 
-static uint16_t _parse_mail_type(const char *arg)
-{
-	uint16_t rc;
-
-	if (strcasecmp(arg, "BEGIN") == 0)
-		rc = MAIL_JOB_BEGIN;
-	else if  (strcasecmp(arg, "END") == 0)
-		rc = MAIL_JOB_END;
-	else if (strcasecmp(arg, "FAIL") == 0)
-		rc = MAIL_JOB_FAIL;
-	else if (strcasecmp(arg, "ALL") == 0)
-		rc = MAIL_JOB_BEGIN |  MAIL_JOB_END |  MAIL_JOB_FAIL;
-	else
-		rc = 0;		/* failure */
-
-	return rc;
-}
-static char *_print_mail_type(const uint16_t type)
-{
-	if (type == 0)
-		return "NONE";
-
-	if (type == MAIL_JOB_BEGIN)
-		return "BEGIN";
-	if (type == MAIL_JOB_END)
-		return "END";
-	if (type == MAIL_JOB_FAIL)
-		return "FAIL";
-	if (type == (MAIL_JOB_BEGIN |  MAIL_JOB_END |  MAIL_JOB_FAIL))
-		return "ALL";
-
-	return "MULTIPLE";
-}
-
 /* helper function for printing options
  * 
  * warning: returns pointer to memory allocated on the stack.
@@ -1151,9 +1158,21 @@ static char *print_constraints()
 	if (opt.mincpus > 0)
 		xstrfmtcat(buf, "mincpus=%d ", opt.mincpus);
 
+	if (opt.minsockets > 0)
+		xstrfmtcat(buf, "minsockets=%d ", opt.minsockets);
+
+	if (opt.mincores > 0)
+		xstrfmtcat(buf, "mincores=%d ", opt.mincores);
+
+	if (opt.minthreads > 0)
+		xstrfmtcat(buf, "minthreads=%d ", opt.minthreads);
+
 	if (opt.realmem > 0)
 		xstrfmtcat(buf, "mem=%dM ", opt.realmem);
 
+	if (opt.mem_per_cpu > 0)
+		xstrfmtcat(buf, "mem-per-cpu=%dM ", opt.mem_per_cpu);
+
 	if (opt.tmpdisk > 0)
 		xstrfmtcat(buf, "tmp=%ld ", opt.tmpdisk);
 
@@ -1172,39 +1191,6 @@ static char *print_constraints()
 	return buf;
 }
 
-static char * 
-print_commandline()
-{
-	int i;
-	char buf[256];
-
-	buf[0] = '\0';
-	for (i = 0; i < command_argc; i++)
-		snprintf(buf, 256,  "%s", command_argv[i]);
-	return xstrdup(buf);
-}
-
-static char *
-print_geometry()
-{
-	int i;
-	char buf[32], *rc = NULL;
-
-	if ((SYSTEM_DIMENSIONS == 0)
-	||  (opt.geometry[0] == (uint16_t)NO_VAL))
-		return NULL;
-
-	for (i=0; i<SYSTEM_DIMENSIONS; i++) {
-		if (i > 0)
-			snprintf(buf, sizeof(buf), "x%u", opt.geometry[i]);
-		else
-			snprintf(buf, sizeof(buf), "%u", opt.geometry[i]);
-		xstrcat(rc, buf);
-	}
-
-	return rc;
-}
-
 /*
  * Takes a string containing the number or name of a signal and returns
  * the signal number.  The signal name is case insensitive, and may be of
@@ -1287,6 +1273,9 @@ static void _opt_list()
 	info("job name       : `%s'", opt.job_name);
 	if (opt.jobid != NO_VAL)
 		info("jobid          : %u", opt.jobid);
+	info("distribution   : %s", format_task_dist_states(opt.distribution));
+	if(opt.distribution == SLURM_DIST_PLANE)
+		info("plane size   : %u", opt.plane_size);
 	info("verbose        : %d", opt.verbose);
 	info("immediate      : %s", tf_(opt.immediate));
 	info("overcommit     : %s", tf_(opt.overcommit));
@@ -1299,16 +1288,14 @@ static void _opt_list()
 		info("nice           : %d", opt.nice);
 	info("account        : %s", opt.account);
 	info("comment        : %s", opt.comment);
-	if (opt.dependency == NO_VAL)
-		info("dependency     : none");
-	else
-		info("dependency     : %u", opt.dependency);
+	info("dependency     : %s", opt.dependency);
+	info("network        : %s", opt.network);
 	str = print_constraints();
 	info("constraints    : %s", str);
 	xfree(str);
-	if (opt.conn_type >= 0)
+	if (opt.conn_type != (uint16_t) NO_VAL)
 		info("conn_type      : %u", opt.conn_type);
-	str = print_geometry();
+	str = print_geometry(opt.geometry);
 	info("geometry       : %s", str);
 	xfree(str);
 	info("reboot         : %s", opt.reboot ? "no" : "yes");
@@ -1326,9 +1313,19 @@ static void _opt_list()
 		slurm_make_time_str(&opt.begin, time_str, sizeof(time_str));
 		info("begin          : %s", time_str);
 	}
-	info("mail_type      : %s", _print_mail_type(opt.mail_type));
+	info("mail_type      : %s", print_mail_type(opt.mail_type));
 	info("mail_user      : %s", opt.mail_user);
-	str = print_commandline();
+	info("sockets-per-node  : %d - %d", opt.min_sockets_per_node,
+					    opt.max_sockets_per_node);
+	info("cores-per-socket  : %d - %d", opt.min_cores_per_socket,
+					    opt.max_cores_per_socket);
+	info("threads-per-core  : %d - %d", opt.min_threads_per_core,
+					    opt.max_threads_per_core);
+	info("ntasks-per-node   : %d", opt.ntasks_per_node);
+	info("ntasks-per-socket : %d", opt.ntasks_per_socket);
+	info("ntasks-per-core   : %d", opt.ntasks_per_core);
+	info("plane_size        : %u", opt.plane_size);
+	str = print_commandline(command_argc, command_argv);
 	info("user command   : `%s'", str);
 	xfree(str);
 
@@ -1339,12 +1336,12 @@ static void _usage(void)
  	printf(
 "Usage: salloc [-N numnodes|[min nodes]-[max nodes]] [-n num-processors]\n"
 "              [[-c cpus-per-node] [-r n] [-p partition] [--hold] [-t minutes]\n"
-"              [--immediate] [--no-kill] [--overcommit]\n"
+"              [--immediate] [--no-kill] [--overcommit] [-D path]\n"
 "              [--share] [-J jobname] [--jobid=id]\n"
-"              [--verbose] [--gid=group] [--uid=user]\n"
+"              [--verbose] [--gid=group] [--uid=user] [--licenses=names]\n"
 "              [-W sec] [--minsockets=n] [--mincores=n] [--minthreads=n]\n"
 "              [--contiguous] [--mincpus=n] [--mem=MB] [--tmp=MB] [-C list]\n"
-"              [--account=name] [--dependency=jobid] [--comment=name]\n"
+"              [--account=name] [--dependency=type:jobid] [--comment=name]\n"
 #ifdef HAVE_BG		/* Blue gene specific options */
 "              [--geometry=XxYxZ] [--conn-type=type] [--no-rotate] [ --reboot]\n"
 "              [--blrts-image=path] [--linux-image=path]\n"
@@ -1353,11 +1350,14 @@ static void _usage(void)
 "              [--mail-type=type] [--mail-user=user][--nice[=value]]\n"
 "              [--bell] [--no-bell] [--kill-command[=signal]]\n"
 "              [--nodefile=file] [--nodelist=hosts] [--exclude=hosts]\n"
+"              [--network=type] [--mem-per-cpu=MB]\n"
 "              executable [args...]\n");
 }
 
 static void _help(void)
 {
+	slurm_ctl_conf_t *conf;
+
         printf (
 "Usage: salloc [OPTIONS...] executable [args...]\n"
 "\n"
@@ -1365,31 +1365,37 @@ static void _help(void)
 "  -N, --nodes=N               number of nodes on which to run (N = min[-max])\n"
 "  -n, --tasks=N               number of processors required\n"
 "  -c, --cpus-per-task=ncpus   number of cpus required per task\n"
+"      --ntasks-per-node=n     number of tasks to invoke on each node\n"
 "  -p, --partition=partition   partition requested\n"
 "  -H, --hold                  submit job in held state\n"
 "  -t, --time=minutes          time limit\n"
+"  -D, --chdir=path            change working directory\n"
 "  -I, --immediate             exit if resources are not immediately available\n"
 "  -k, --no-kill               do not kill job on node failure\n"
 "  -K, --kill-command[=signal] signal to send terminating job\n"
 "  -O, --overcommit            overcommit resources\n"
 "  -s, --share                 share nodes with other jobs\n"
+"  -m, --distribution=type     distribution method for processes to nodes\n"
+"                              (type = block|cyclic|arbitrary)\n"
 "  -J, --job-name=jobname      name of job\n"
 "      --jobid=id              specify jobid to use\n"
 "  -W, --wait=sec              seconds to wait for allocation if not\n"
 "                              immediately available\n"
 "  -v, --verbose               verbose mode (multiple -v's increase verbosity)\n"
 "  -q, --quiet                 quiet mode (suppress informational messages)\n"
-"  -d, --dependency=jobid      defer job until specified jobid completes\n"
+"  -P, --dependency=type:jobid defer job until condition on jobid is satisfied\n"
 "      --nice[=value]          decrease secheduling priority by value\n"
 "  -U, --account=name          charge job to specified account\n"
 "      --begin=time            defer job until HH:MM DD/MM/YY\n"
 "      --comment=name          arbitrary comment\n"
+"  -L, --licenses=names        required license, comma separated\n"
 "      --mail-type=type        notify on state change: BEGIN, END, FAIL or ALL\n"
 "      --mail-user=user        who to send email notification for job state changes\n"
 "      --bell                  ring the terminal bell when the job is allocated\n"
 "      --no-bell               do NOT ring the terminal bell\n"
 "      --gid=group_id          group ID to run job as (user root only)\n"
 "      --uid=user_id           user ID to run job as (user root only)\n"
+"      --get-user-env          used by Moab.  See srun man page.\n"
 "\n"
 "Constraint options:\n"
 "      --mincpus=n             minimum number of cpus per node\n"
@@ -1407,8 +1413,37 @@ static void _help(void)
 "Consumable resources related options:\n" 
 "      --exclusive             allocate nodes in exclusive mode when\n" 
 "                              cpu consumable resource is enabled\n"
+"      --mem-per-cpu=MB        maximum amount of real memory per allocated\n"
+"                              cpu required by the job.\n" 
+"                              --mem >= --job-mem if --mem is specified.\n" 
 "\n"
+"Affinity/Multi-core options: (when the task/affinity plugin is enabled)\n" 
+"  -B --extra-node-info=S[:C[:T]]            Expands to:\n"
+"      --sockets-per-node=S    number of sockets per node to allocate\n"
+"      --cores-per-socket=C    number of cores per socket to allocate\n"
+"      --threads-per-core=T    number of threads per core to allocate\n"
+"                              each field can be 'min[-max]' or wildcard '*'\n"
+"                              total cpus requested = (N x S x C x T)\n"
+"\n"
+"      --ntasks-per-socket=n   number of tasks to invoke on each socket\n"
+"      --ntasks-per-core=n     number of tasks to invoke on each core\n");
+	conf = slurm_conf_lock();
+	if (conf->task_plugin != NULL
+	    && strcasecmp(conf->task_plugin, "task/affinity") == 0) {
+		printf(
+"      --hint=                 Bind tasks according to application hints\n"
+"                              (see \"--hint=help\" for options)\n");
+	}
+	slurm_conf_unlock();
+
+        printf("\n"
+#ifdef HAVE_AIX				/* AIX/Federation specific options */
+  "AIX related options:\n"
+  "  --network=type              communication protocol to be used\n"
+  "\n"
+#endif
 #ifdef HAVE_BG				/* Blue gene specific options */
+"\n"
   "Blue Gene related options:\n"
   "  -g, --geometry=XxYxZ        geometry constraints of the job\n"
   "  -R, --no-rotate             disable geometry rotation\n"
