@@ -1,12 +1,12 @@
 /*****************************************************************************\
  *  sinfo.c - Report overall state the system
  *
- *  $Id: sinfo.c 10744 2007-01-11 20:09:18Z da $
+ *  $Id: sinfo.c 14203 2008-06-06 16:58:44Z jette $
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>, Morris Jette <jette1@llnl.gov>
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -101,8 +101,12 @@ int main(int argc, char *argv[])
 	List sinfo_list = NULL;
 	int rc = 0;
 
-	log_init(xbasename(argv[0]), opts, SYSLOG_FACILITY_DAEMON, NULL);
+	log_init(xbasename(argv[0]), opts, SYSLOG_FACILITY_USER, NULL);
 	parse_command_line(argc, argv);
+	if (params.verbose) {
+		opts.stderr_level += params.verbose;
+		log_alter(opts, SYSLOG_FACILITY_USER, NULL);
+	}
 
 	while (1) {
 		if ((!params.no_header)
@@ -254,7 +258,7 @@ _query_server(partition_info_msg_t ** part_pptr,
 		    slurm_load_partitions((time_t) NULL, &new_part_ptr,
 					  show_flags);
 	if (error_code) {
-		slurm_perror("slurm_load_part");
+		slurm_perror("slurm_load_partitions");
 		return error_code;
 	}
 
@@ -552,9 +556,13 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr,
 	    (_strcmp(node_ptr->reason, sinfo_ptr->reason)))
 		return false;
 
-	if (params.match_flags.state_flag &&
-	    (node_ptr->node_state != sinfo_ptr->node_state))
-		return false;
+	if (params.match_flags.state_flag) {
+		char *state1, *state2;
+		state1 = node_state_string(node_ptr->node_state);
+		state2 = node_state_string(sinfo_ptr->node_state);
+		if (strcmp(state1, state2))
+			return false;
+	}
 
 	/* If no need to exactly match sizes, just return here 
 	 * otherwise check cpus, disk, memory and weigth individually */
@@ -629,7 +637,11 @@ static bool _match_part_data(sinfo_data_t *sinfo_ptr,
 		return false;
 
 	if (params.match_flags.share_flag &&
-	    (part_ptr->shared != sinfo_ptr->part_info->shared))
+	    (part_ptr->max_share != sinfo_ptr->part_info->max_share))
+		return false;
+
+	if (params.match_flags.priority_flag &&
+	    (part_ptr->priority != sinfo_ptr->part_info->priority))
 		return false;
 
 	return true;
@@ -724,7 +736,7 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 
 	sinfo_ptr->cpus_alloc += node_ptr->used_cpus;
 	sinfo_ptr->cpus_total += node_ptr->cpus;
-	if ((sinfo_ptr->node_state & NODE_STATE_DRAIN) ||
+	if ((node_ptr->node_state & NODE_STATE_DRAIN) ||
 	    (base_state == NODE_STATE_DOWN)) {
 		sinfo_ptr->cpus_other += node_ptr->cpus -
 					 node_ptr->used_cpus;
