@@ -1,11 +1,11 @@
 /*****************************************************************************\
  *  controller.c - main control machine daemon for slurm
- *  $Id: controller.c 12452 2007-10-05 19:07:07Z da $
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>, Kevin Tew <tew1@llnl.gov>
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -68,7 +68,8 @@
 #include "src/common/node_select.h"
 #include "src/common/pack.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_jobacct.h"
+#include "src/common/slurm_jobacct_gather.h"
+#include "src/common/slurm_accounting_storage.h"
 #include "src/common/slurm_auth.h"
 #include "src/common/slurm_jobcomp.h"
 #include "src/common/slurm_protocol_api.h"
@@ -76,8 +77,11 @@
 #include "src/common/uid.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
+#include "src/common/assoc_mgr.h"
 
 #include "src/slurmctld/agent.h"
+#include "src/slurmctld/job_scheduler.h"
+#include "src/slurmctld/licenses.h"
 #include "src/slurmctld/locks.h"
 #include "src/slurmctld/ping_nodes.h"
 #include "src/slurmctld/proc_req.h"
@@ -100,6 +104,12 @@
 				 * check-in before we ping them */
 #define SHUTDOWN_WAIT     2	/* Time to wait for backup server shutdown */
 
+#if (0)
+/* If defined and FastSchedule=0 in slurm.conf, then report the CPU count that a 
+ * node registers with rather than the CPU count defined for the node in slurm.conf */
+#define SLURM_NODE_ACCT_REGISTER 1
+#endif
+
 /**************************************************************************\
  * To test for memory leaks, set MEMORY_LEAK_DEBUG to 1 using 
  * "configure --enable-memory-leak-debug" then execute
@@ -115,6 +125,8 @@
  *    (640 bytes), it is really not lost.
  * The _keyvalue_regex_init() function will generate two blocks "definitely
  *    lost", both of size zero. We haven't bothered to address this.
+ * On some systems dlopen() will generate a small number of "definitely
+ *    lost" blocks that are not cleared by dlclose().
  * On some systems, pthread_create() will generated a small number of 
  *    "possibly lost" blocks.
  * Otherwise the report should be free of errors. Remember to reset 
@@ -128,6 +140,9 @@ log_options_t log_opts = LOG_OPTS_INITIALIZER;
 /* Global variables */
 slurmctld_config_t slurmctld_config;
 int bg_recover = DEFAULT_RECOVER;
+char *slurmctld_cluster_name = NULL; /* name of cluster */
+void *acct_db_conn = NULL;
+int accounting_enforce = 0;
 
 /* Local variables */
 static int	daemonize = DEFAULT_DAEMONIZE;
@@ -152,10 +167,13 @@ static int controller_sigarray[] = {
 
 static void         _default_sigaction(int sig);
 inline static void  _free_server_thread(void);
+static int          _accounting_cluster_ready();
+static int          _accounting_mark_all_nodes_down(char *reason);
 static void         _init_config(void);
 static void         _init_pidfile(void);
 static void         _kill_old_slurmctld(void);
 static void         _parse_commandline(int argc, char *argv[]);
+static void         _remove_assoc(acct_association_rec_t *rec);
 inline static int   _report_locks_set(void);
 static void *       _service_connection(void *arg);
 static int          _shutdown_backup_controller(int wait_time);
@@ -173,9 +191,13 @@ typedef struct connection_arg {
 /* main - slurmctld main function, start various threads and process RPCs */
 int main(int argc, char *argv[])
 {
-	int error_code;
+	int cnt, error_code, i;
 	pthread_attr_t thread_attr;
 	struct stat stat_buf;
+	/* Locks: Write configuration, job, node, and partition */
+	slurmctld_lock_t config_write_lock = {
+		WRITE_LOCK, WRITE_LOCK, WRITE_LOCK, WRITE_LOCK };
+	assoc_init_args_t assoc_init_arg;
 
 	/*
 	 * Establish initial configuration
@@ -214,6 +236,22 @@ int main(int argc, char *argv[])
 	if (stat(slurmctld_conf.mail_prog, &stat_buf) != 0)
 		error("Configured MailProg is invalid");
 
+	if (!strcmp(slurmctld_conf.accounting_storage_type, 
+		    "accounting_storage/none")) {
+		if (strcmp(slurmctld_conf.job_acct_gather_type, 
+			   "jobacct_gather/none"))
+			error("Job accounting information gathered, "
+			      "but not stored"); 
+	} else {
+		if (!strcmp(slurmctld_conf.job_acct_gather_type, 
+			    "jobacct_gather/none"))
+			info("Job accounting information stored, "
+			     "but details not gathered");
+	}
+
+	if (license_init(slurmctld_conf.licenses) != SLURM_SUCCESS)
+		fatal("Invalid Licenses value: %s", slurmctld_conf.licenses);
+
 #ifndef NDEBUG
 #  ifdef PR_SET_DUMPABLE
 	if (prctl(PR_SET_DUMPABLE, 1) < 0)
@@ -228,11 +266,11 @@ int main(int argc, char *argv[])
 		fatal("Unable to initialize StateSaveLocation");
 
 	if (daemonize) {
-		error_code = daemon(1, 1);
+		slurmctld_config.daemonize = 1;
+		if (daemon(1, 1))
+			error("daemon(): %m");
 		log_alter(log_opts, LOG_DAEMON, 
 			  slurmctld_conf.slurmctld_logfile);
-		if (error_code)
-			error("daemon error %d", error_code);
 		if (slurmctld_conf.slurmctld_logfile
 		&&  (slurmctld_conf.slurmctld_logfile[0] == '/')) {
 			char *slash_ptr, *work_dir;
@@ -251,8 +289,27 @@ int main(int argc, char *argv[])
 					slurmctld_conf.state_save_location);
 			}
 		}
+	} else {
+		slurmctld_config.daemonize = 0;
 	}
-	info("slurmctld version %s started", SLURM_VERSION);
+
+	/* This needs to be copied for other modules to access the
+	 * memory, it will report 'HashBase' if it is not duped
+	 */
+	slurmctld_cluster_name = xstrdup(slurmctld_conf.cluster_name);
+	accounting_enforce = slurmctld_conf.accounting_storage_enforce;
+	acct_db_conn = acct_storage_g_get_connection(true, false);
+	assoc_init_arg.enforce = accounting_enforce;
+	assoc_init_arg.remove_assoc_notify = _remove_assoc;
+	if (assoc_mgr_init(acct_db_conn, &assoc_init_arg) &&
+	    accounting_enforce) {
+		error("assoc_mgr_init failure");
+		fatal("slurmdbd and/or database must be up at "
+		      "slurmctld start time");
+	}
+
+	info("slurmctld version %s started on cluster %s",
+	     SLURM_VERSION, slurmctld_cluster_name);
 
 	if ((error_code = gethostname_short(node_name, MAX_SLURM_NAME)))
 		fatal("getnodename error %s", slurm_strerror(error_code));
@@ -260,8 +317,10 @@ int main(int argc, char *argv[])
 	/* init job credential stuff */
 	slurmctld_config.cred_ctx = slurm_cred_creator_ctx_create(
 			slurmctld_conf.job_credential_private_key);
-	if (!slurmctld_config.cred_ctx)
-		fatal("slurm_cred_creator_ctx_create: %m");
+	if (!slurmctld_config.cred_ctx) {
+		fatal("slurm_cred_creator_ctx_create(%s): %m",
+			slurmctld_conf.job_credential_private_key);
+	}
 
 
 	/* Not used in creator
@@ -282,7 +341,12 @@ int main(int argc, char *argv[])
 			SLURM_SUCCESS )
 		fatal( "failed to initialize checkpoint plugin" );
 	if (slurm_select_init() != SLURM_SUCCESS )
-		fatal( "failed to initialize node selection plugin state");
+		fatal( "failed to initialize node selection plugin");
+	if (slurm_acct_storage_init(NULL) != SLURM_SUCCESS )
+		fatal( "failed to initialize accounting_storage plugin");
+
+	if (slurm_jobacct_gather_init() != SLURM_SUCCESS )
+		fatal( "failed to initialize jobacct_gather plugin");
 
 	while (1) {
 		/* initialization for each primary<->backup switch */
@@ -301,20 +365,46 @@ int main(int argc, char *argv[])
 			(void) _shutdown_backup_controller(SHUTDOWN_WAIT);
 			/* Now recover the remaining state information */
 			if (switch_restore(slurmctld_conf.state_save_location,
-					 recover ? true : false))
+					   recover ? true : false))
 				fatal(" failed to initialize switch plugin" );
+			lock_slurmctld(config_write_lock);
 			if ((error_code = read_slurm_conf(recover))) {
 				fatal("read_slurm_conf reading %s: %s",
 					slurmctld_conf.slurm_conf,
 					slurm_strerror(error_code));
 			}
+			unlock_slurmctld(config_write_lock);
+			
+			if ((recover == 0) || 
+			    (!stat("/tmp/slurm_accounting_first", &stat_buf))) {
+				/* When first starting to write node state
+				 * information to Gold or SlurmDBD, create 
+				 * a file called "/tmp/slurm_accounting_first"  
+				 * to capture node initialization information */
+				
+				_accounting_mark_all_nodes_down("cold-start");
+				unlink("/tmp/slurm_accounting_first");
+			}
 		} else {
 			error("this host (%s) not valid controller (%s or %s)",
 				node_name, slurmctld_conf.control_machine,
 				slurmctld_conf.backup_controller);
 			exit(0);
 		}
+
+		if(!acct_db_conn) {
+			acct_db_conn = 
+				acct_storage_g_get_connection(true, false);
+			if (assoc_mgr_init(acct_db_conn, &assoc_init_arg) &&
+			    accounting_enforce) {
+				error("assoc_mgr_init failure");
+				fatal("slurmdbd and/or database must be up at "
+				      "slurmctld start time");
+			}
+		}
+
 		info("Running as primary controller");
+		_accounting_cluster_ready();
 		if (slurm_sched_init() != SLURM_SUCCESS)
 			fatal("failed to initialize scheduling plugin");
 
@@ -326,9 +416,12 @@ int main(int argc, char *argv[])
 		slurm_mutex_unlock(&slurmctld_config.thread_count_lock);
 		slurm_attr_init(&thread_attr);
 		if (pthread_create(&slurmctld_config.thread_id_rpc, 
-				&thread_attr,_slurmctld_rpc_mgr, NULL))
+				&thread_attr, _slurmctld_rpc_mgr, NULL))
 			fatal("pthread_create error %m");
 		slurm_attr_destroy(&thread_attr);
+		clusteracct_storage_g_register_ctld(
+					slurmctld_conf.cluster_name, 
+					slurmctld_conf.slurmctld_port);
 
 		/*
 		 * create attached thread for signal handling
@@ -378,6 +471,10 @@ int main(int argc, char *argv[])
 		if (slurmctld_config.resume_backup == false)
 			break;
 		recover = 2;
+
+		/* Save any pending state save RPCs */
+		acct_storage_g_close_connection(&acct_db_conn);
+		assoc_mgr_fini();
 	}
 
 	/* Since pidfile is created as user root (its owner is
@@ -386,12 +483,10 @@ int main(int argc, char *argv[])
 	if (unlink(slurmctld_conf.slurmctld_pidfile) < 0)
 		verbose("Unable to remove pidfile '%s': %m",
 			slurmctld_conf.slurmctld_pidfile);
-
+	
 #ifdef MEMORY_LEAK_DEBUG
-{
 	/* This should purge all allocated memory,   *\
 	\*   Anything left over represents a leak.   */
-	int i, cnt;
 
 	/* Give running agents a chance to complete and free memory.
 	 * Wait up to 30 seconds (3 seconds * 10) */
@@ -405,31 +500,47 @@ int main(int argc, char *argv[])
 	if (i >= 10)
 		error("Left %d agent threads active", cnt);
 
+	/* Plugins are needed to purge job/node data structures,
+	 * unplug after other data structures are purged */
+	g_slurm_jobcomp_fini();
+	slurm_acct_storage_fini();
+	slurm_jobacct_gather_fini();
+	slurm_sched_fini();
+	slurm_select_fini();
+	checkpoint_fini();
+	slurm_auth_fini();
+	switch_fini();
+	assoc_mgr_fini();
+
 	/* Purge our local data structures */
 	job_fini();
 	part_fini();	/* part_fini() must preceed node_fini() */
 	node_fini();
 	trigger_fini();
 
-	/* Plugins are needed to purge job/node data structures,
-	 * unplug after other data structures are purged */
-	g_slurm_jobcomp_fini();
-	jobacct_g_fini_slurmctld();
-	slurm_sched_fini();
-	slurm_select_fini();
-	checkpoint_fini();
-	slurm_auth_fini();
-	switch_fini();
-
 	/* purge remaining data structures */
 	slurm_cred_ctx_destroy(slurmctld_config.cred_ctx);
+	slurm_crypto_fini();	/* must be after ctx_destroy */
 	slurm_conf_destroy();
 	slurm_api_clear_config();
 	sleep(2);
-}
+#else
+	/* Give REQUEST_SHUTDOWN a chance to get propagated, 
+	 * up to 3 seconds. */
+	for (i=0; i<3; i++) {
+		agent_purge();
+		cnt = get_agent_count();
+		if (cnt == 0)
+			break;
+		sleep(1);
+	}
 #endif
 
-	info("Slurmctld shutdown completing");
+	xfree(slurmctld_cluster_name);
+	if (cnt) {
+		info("Slurmctld shutdown completing with %d active agent "
+			"threads\n\n", cnt);
+	}
 	log_fini();
 	
 	if (dump_core)
@@ -464,6 +575,7 @@ static void  _init_config(void)
 		(void) setrlimit(RLIMIT_DATA, &rlim);
 	}
 
+	slurmctld_config.boot_time      = time(NULL);
 	slurmctld_config.daemonize      = DEFAULT_DAEMONIZE;
 	slurmctld_config.resume_backup  = false;
 	slurmctld_config.server_thread_count = 0;
@@ -482,6 +594,35 @@ static void  _init_config(void)
 #endif
 }
 
+/* Read configuration file.
+ * Same name as API function for use in accounting_storage plugin */
+extern int slurm_reconfigure(void)
+{
+	/* Locks: Write configuration, job, node, and partition */
+	slurmctld_lock_t config_write_lock = { 
+		WRITE_LOCK, WRITE_LOCK, WRITE_LOCK, WRITE_LOCK };
+	int rc;
+
+	/*
+	 * XXX - need to shut down the scheduler
+	 * plugin, re-read the configuration, and then
+	 * restart the (possibly new) plugin.
+	 */
+	lock_slurmctld(config_write_lock);
+	rc = read_slurm_conf(0);
+	if (rc)
+		error("read_slurm_conf: %s", slurm_strerror(rc));
+	else {
+		_update_cred_key();
+		set_slurmctld_state_loc();
+	}
+	unlock_slurmctld(config_write_lock);
+	trigger_reconfig();
+	slurm_sched_partition_change();	/* notify sched plugin */
+	select_g_reconfigure();		/* notify select plugin too */
+	return rc;
+}
+
 /* _slurmctld_signal_hand - Process daemon-wide signals */
 static void *_slurmctld_signal_hand(void *no_data)
 {
@@ -492,9 +633,6 @@ static void *_slurmctld_signal_hand(void *no_data)
 	/* Locks: Read configuration */
 	slurmctld_lock_t config_read_lock = { 
 		READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
-	/* Locks: Write configuration, job, node, and partition */
-	slurmctld_lock_t config_write_lock = { 
-		WRITE_LOCK, WRITE_LOCK, WRITE_LOCK, WRITE_LOCK };
 
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
@@ -528,23 +666,7 @@ static void *_slurmctld_signal_hand(void *no_data)
 			break;
 		case SIGHUP:	/* kill -1 */
 			info("Reconfigure signal (SIGHUP) received");
-			/*
-			 * XXX - need to shut down the scheduler
-			 * plugin, re-read the configuration, and then
-			 * restart the (possibly new) plugin.
-			 */
-			lock_slurmctld(config_write_lock);
-			rc = read_slurm_conf(0);
-			if (rc)
-				error("read_slurm_conf: %s",
-				      slurm_strerror(rc));
-			else {
-				_update_cred_key();
-				set_slurmctld_state_loc();
-			}
-			unlock_slurmctld(config_write_lock);
-			trigger_reconfig();
-			slurm_sched_partition_change();
+			slurm_reconfigure();
 			break;
 		case SIGABRT:	/* abort */
 			info("SIGABRT received");
@@ -762,6 +884,89 @@ static void _free_server_thread(void)
 	pthread_cond_broadcast(&server_thread_cond);
 }
 
+static int _accounting_cluster_ready()
+{
+	uint32_t procs = 0;
+	struct node_record *node_ptr;
+	int i;
+	int rc = SLURM_ERROR;
+	time_t event_time = time(NULL);
+
+	node_ptr = node_record_table_ptr;
+	for (i = 0; i < node_record_count; i++, node_ptr++) {
+		if (node_ptr->name == '\0')
+			continue;
+#ifdef SLURM_NODE_ACCT_REGISTER
+		if (slurmctld_conf.fast_schedule)
+			procs += node_ptr->config_ptr->cpus;
+		else
+			procs += node_ptr->cpus;
+#else
+		procs += node_ptr->config_ptr->cpus;
+#endif
+	}
+
+	rc = clusteracct_storage_g_cluster_procs(acct_db_conn,
+						 slurmctld_cluster_name,
+						 procs, event_time);
+
+	return rc;
+}
+
+static int _accounting_mark_all_nodes_down(char *reason)
+{
+	char *state_file;
+	struct stat stat_buf;
+	struct node_record *node_ptr;
+	int i;
+	time_t event_time;
+	int rc = SLURM_ERROR;
+
+	state_file = xstrdup (slurmctld_conf.state_save_location);
+	xstrcat (state_file, "/node_state");
+	if (stat(state_file, &stat_buf)) {
+		debug("_accounting_mark_all_nodes_down: could not stat(%s) "
+		      "to record node down time", state_file);
+		event_time = time(NULL);
+	} else {
+		event_time = stat_buf.st_mtime;
+	}
+	xfree(state_file);
+
+	if((rc = acct_storage_g_flush_jobs_on_cluster(acct_db_conn,
+						      slurmctld_cluster_name,
+						      event_time))
+	   == SLURM_ERROR)
+		return rc;
+
+	node_ptr = node_record_table_ptr;
+	for (i = 0; i < node_record_count; i++, node_ptr++) {
+		if (node_ptr->name == '\0')
+			continue;
+		if((rc = clusteracct_storage_g_node_down(acct_db_conn,
+							 slurmctld_cluster_name,
+							 node_ptr, event_time,
+							 reason))
+		   == SLURM_ERROR) 
+			break;
+	}
+	return rc;
+}
+
+static void _remove_assoc(acct_association_rec_t *rec)
+{
+	int cnt = 0;
+
+	if (accounting_enforce)
+		cnt = job_cancel_by_assoc_id(rec->id);
+
+	if (cnt) {
+		info("Removed association id:%u user:%s, cancelled %u jobs",
+		     rec->id, rec->user, cnt);
+	} else
+		debug("Removed association id:%u user:%s", rec->id, rec->user);
+}
+
 /*
  * _slurmctld_background - process slurmctld background activities
  *	purge defunct job records, save state, schedule jobs, and 
@@ -778,6 +983,7 @@ static void *_slurmctld_background(void *no_data)
 	static time_t last_timelimit_time;
 	static time_t last_assert_primary_time;
 	static time_t last_trigger;
+	static time_t last_node_acct;
 	time_t now;
 	int ping_interval;
 	DEF_TIMERS;
@@ -795,6 +1001,9 @@ static void *_slurmctld_background(void *no_data)
 	 * (Might kill jobs on nodes set DOWN) */
 	slurmctld_lock_t node_write_lock = { 
 		READ_LOCK, WRITE_LOCK, WRITE_LOCK, NO_LOCK };
+	/* Locks: Read node */
+	slurmctld_lock_t node_read_lock = { 
+		NO_LOCK, NO_LOCK, READ_LOCK, NO_LOCK };
 	/* Locks: Write partition */
 	slurmctld_lock_t part_write_lock = { 
 		NO_LOCK, NO_LOCK, NO_LOCK, WRITE_LOCK };
@@ -813,12 +1022,14 @@ static void *_slurmctld_background(void *no_data)
 		ping_interval = 60 * 60 * 24 * 356;	/* one year */
 	last_ping_node_time = now + (time_t)MIN_CHECKIN_TIME - ping_interval;
 	last_ping_srun_time = now;
+	last_node_acct = now;
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 	debug3("_slurmctld_background pid = %u", getpid());
 
-	while (slurmctld_config.shutdown_time == 0) {
-		sleep(1);
+	while (1) {
+		if (slurmctld_config.shutdown_time == 0)
+			sleep(1);
 
 		now = time(NULL);
 		START_TIMER;
@@ -844,9 +1055,10 @@ static void *_slurmctld_background(void *no_data)
 
 		if (difftime(now, last_timelimit_time) >= PERIODIC_TIMEOUT) {
 			last_timelimit_time = now;
-			debug2("Performing job time limit check");
+			debug2("Performing job time limit and checkpoint test");
 			lock_slurmctld(job_write_lock);
 			job_time_limit();
+			step_checkpoint();
 			unlock_slurmctld(job_write_lock);
 		}
 
@@ -875,8 +1087,8 @@ static void *_slurmctld_background(void *no_data)
 			unlock_slurmctld(job_read_lock);
 		}
 
-		/* Process pending agent work */
-		agent_retry(RPC_RETRY_INTERVAL);
+		/* Process any pending agent work */
+		agent_retry(RPC_RETRY_INTERVAL, true);
 
 		if (difftime(now, last_group_time) >= PERIODIC_GROUP_CHECK) {
 			last_group_time = now;
@@ -897,6 +1109,7 @@ static void *_slurmctld_background(void *no_data)
 			last_sched_time = now;
 			if (schedule())
 				last_checkpoint_time = 0;  /* force state save */
+			set_job_elig_time();
 		}
 
 		if (difftime(now, last_trigger) > TRIGGER_INTERVAL) {
@@ -911,6 +1124,14 @@ static void *_slurmctld_background(void *no_data)
 			save_all_state();
 		}
 
+		if (difftime(now, last_node_acct) >= PERIODIC_NODE_ACCT) {
+			/* Report current node state to account for added 
+			 * or reconfigured nodes */
+			last_node_acct = now;
+			lock_slurmctld(node_read_lock);
+			_accounting_cluster_ready();
+			unlock_slurmctld(node_read_lock);
+		}
 		/* Reassert this machine as the primary controller.
 		 * A network or security problem could result in 
 		 * the backup controller assuming control even 
