@@ -4,7 +4,7 @@
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -39,6 +39,10 @@
 #  include "config.h"
 #endif
 
+#ifdef HAVE_SYS_SYSLOG_H
+#  include <sys/syslog.h>
+#endif
+
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -51,6 +55,7 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "src/common/assoc_mgr.h"
 #include "src/common/hostlist.h"
 #include "src/common/list.h"
 #include "src/common/macros.h"
@@ -58,12 +63,13 @@
 #include "src/common/parse_spec.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_jobcomp.h"
+#include "src/common/slurm_rlimits_info.h"
 #include "src/common/switch.h"
 #include "src/common/xstring.h"
-#include "src/common/node_select.h"
-#include "src/common/slurm_jobacct.h"
-#include "src/common/slurm_rlimits_info.h"
 
+#include "src/slurmctld/acct_policy.h"
+#include "src/slurmctld/job_scheduler.h"
+#include "src/slurmctld/licenses.h"
 #include "src/slurmctld/locks.h"
 #include "src/slurmctld/node_scheduler.h"
 #include "src/slurmctld/proc_req.h"
@@ -72,18 +78,21 @@
 #include "src/slurmctld/slurmctld.h"
 #include "src/slurmctld/trigger_mgr.h"
 
+static void _acct_restore_active_jobs(void);
 static int  _build_bitmaps(void);
+static void _build_bitmaps_pre_select(void);
 static int  _init_all_slurm_conf(void);
 static void _purge_old_node_state(struct node_record *old_node_table_ptr, 
 				int old_node_record_count);
-static void _restore_node_state(struct node_record *old_node_table_ptr, 
+static int  _restore_job_dependencies(void);
+static int  _restore_node_state(struct node_record *old_node_table_ptr, 
 				int old_node_record_count);
 static int  _preserve_select_type_param(slurm_ctl_conf_t * ctl_conf_ptr, 
 					select_type_plugin_info_t old_select_type_p);
 static int  _preserve_plugins(slurm_ctl_conf_t * ctl_conf_ptr, 
 				char *old_auth_type, char *old_checkpoint_type,
-				char *old_sched_type, char *old_select_type,
-				char *old_switch_type);
+				char *old_crypto_type, char *old_sched_type, 
+				char *old_select_type, char *old_switch_type);
 static int  _sync_nodes_to_comp_job(void);
 static int  _sync_nodes_to_jobs(void);
 static int  _sync_nodes_to_active_job(struct job_record *job_ptr);
@@ -91,12 +100,60 @@ static int  _sync_nodes_to_active_job(struct job_record *job_ptr);
 static void _validate_node_proc_count(void);
 #endif
 
-static char highest_node_name[MAX_SLURM_NAME] = "";
+static char *highest_node_name = NULL;
 int node_record_count = 0;
 
 /* FIXME - declarations for temporarily moved functions */
 #define MULTIPLE_VALUE_MSG "Multiple values for %s, latest one used"
 
+/*
+ * _build_bitmaps_pre_select - recover some state for jobs and nodes prior to 
+ *	calling the select_* functions
+ */
+static void _build_bitmaps_pre_select(void)
+{
+	struct part_record   *part_ptr;
+	struct node_record   *node_ptr;
+	ListIterator part_iterator;
+	int i;
+	
+
+	/* scan partition table and identify nodes in each */
+	part_iterator = list_iterator_create(part_list);
+	if (part_iterator == NULL)
+		fatal ("memory allocation failure");
+
+	while ((part_ptr = (struct part_record *) list_next(part_iterator))) {
+		FREE_NULL_BITMAP(part_ptr->node_bitmap);
+
+		if ((part_ptr->nodes == NULL) || (part_ptr->nodes[0] == '\0'))
+			continue;
+
+		if (node_name2bitmap(part_ptr->nodes, false, 
+				     &part_ptr->node_bitmap)) {
+			fatal("Invalid node names in partition %s",
+			      part_ptr->name);
+		}
+
+		for (i=0; i<node_record_count; i++) {
+			if (bit_test(part_ptr->node_bitmap, i) == 0)
+				continue;
+			node_ptr = &node_record_table_ptr[i];
+			part_ptr->total_nodes++;
+			if (slurmctld_conf.fast_schedule)
+				part_ptr->total_cpus += 
+					node_ptr->config_ptr->cpus;
+			else
+				part_ptr->total_cpus += node_ptr->cpus;
+			node_ptr->part_cnt++;
+			xrealloc(node_ptr->part_pptr, (node_ptr->part_cnt *
+				sizeof(struct part_record *)));
+			node_ptr->part_pptr[node_ptr->part_cnt-1] = part_ptr;
+		}
+	}
+	list_iterator_destroy(part_iterator);
+	return;	
+}
 
 /*
  * _build_bitmaps - build node bitmaps to define which nodes are in which 
@@ -110,16 +167,11 @@ int node_record_count = 0;
  */
 static int _build_bitmaps(void)
 {
-	int i, j, error_code = SLURM_SUCCESS;
-	char *this_node_name;
+	int i, error_code = SLURM_SUCCESS;
 	ListIterator config_iterator;
-	ListIterator part_iterator;
 	struct config_record *config_ptr;
-	struct part_record   *part_ptr;
-	struct node_record   *node_ptr;
 	struct job_record    *job_ptr;
 	ListIterator job_iterator;
-	hostlist_t host_list;
 
 	last_node_update = time(NULL);
 	last_part_update = time(NULL);
@@ -185,7 +237,7 @@ static int _build_bitmaps(void)
 		base_state = node_record_table_ptr[i].node_state & 
 				NODE_STATE_BASE;
 		drain_flag = node_record_table_ptr[i].node_state &
-				NODE_STATE_DRAIN; 
+				(NODE_STATE_DRAIN | NODE_STATE_FAIL);
 		no_resp_flag = node_record_table_ptr[i].node_state & 
 				NODE_STATE_NO_RESPOND;
 		job_cnt = node_record_table_ptr[i].run_job_cnt +
@@ -204,56 +256,6 @@ static int _build_bitmaps(void)
 			bit_set(node_record_table_ptr[i].config_ptr->
 				node_bitmap, i);
 	}
-
-	/* scan partition table and identify nodes in each */
-	part_iterator = list_iterator_create(part_list);
-	if (part_iterator == NULL)
-		fatal ("memory allocation failure");
-
-	while ((part_ptr = (struct part_record *) list_next(part_iterator))) {
-		FREE_NULL_BITMAP(part_ptr->node_bitmap);
-		part_ptr->node_bitmap =
-		    (bitstr_t *) bit_alloc(node_record_count);
-		if (part_ptr->node_bitmap == NULL)
-			fatal ("bit_alloc malloc failure");
-
-		/* check for each node in the partition */
-		if ((part_ptr->nodes == NULL) || (part_ptr->nodes[0] == '\0'))
-			continue;
-
-		if ((host_list = hostlist_create(part_ptr->nodes)) == NULL) {
-			fatal("hostlist_create error for %s, %m",
-			      part_ptr->nodes);
-			continue;
-		}
-
-		while ((this_node_name = hostlist_shift(host_list))) {
-			node_ptr = find_node_record(this_node_name);
-			if (node_ptr == NULL) {
-				fatal("_build_bitmaps: node %s is referenced "
-					"but not defined in slurm.conf "
-					"(no NodeName specification)", 
-					this_node_name);
-				free(this_node_name);
-				continue;
-			}
-			j = node_ptr - node_record_table_ptr;
-			bit_set(part_ptr->node_bitmap, j);
-			part_ptr->total_nodes++;
-			if (slurmctld_conf.fast_schedule)
-				part_ptr->total_cpus += 
-					node_ptr->config_ptr->cpus;
-			else
-				part_ptr->total_cpus += node_ptr->cpus;
-			node_ptr->part_cnt++;
-			xrealloc(node_ptr->part_pptr, (node_ptr->part_cnt *
-				sizeof(struct part_record *)));
-			node_ptr->part_pptr[node_ptr->part_cnt-1] = part_ptr;
-			free(this_node_name);
-		}
-		hostlist_destroy(host_list);
-	}
-	list_iterator_destroy(part_iterator);
 	return error_code;
 }
 
@@ -282,7 +284,7 @@ static int _init_all_slurm_conf(void)
 	if ((error_code = init_job_conf()))
 		return error_code;
 
-	strcpy(highest_node_name, "");
+	xfree(highest_node_name);
 	return 0;
 }
 
@@ -299,9 +301,12 @@ static int _state_str2int(const char *state_str)
 			break;
 		}
 	}
-	if ((i >= NODE_STATE_END)
-	&&  (strncasecmp("DRAIN", state_str, 5) == 0))
-		state_val = NODE_STATE_UNKNOWN | NODE_STATE_DRAIN;
+	if (i >= NODE_STATE_END) {
+		if (strncasecmp("DRAIN", state_str, 5) == 0)
+			state_val = NODE_STATE_UNKNOWN | NODE_STATE_DRAIN;
+		else if (strncasecmp("FAIL", state_str, 4) == 0)
+			state_val = NODE_STATE_IDLE | NODE_STATE_FAIL;
+	}
 	if (state_val == NO_VAL) {
 		error("invalid node state %s", state_str);
 		errno = EINVAL;
@@ -340,6 +345,7 @@ static void _set_node_prefix(const char *nodenames, slurm_ctl_conf_t *conf)
 	debug3("Prefix is %s %s %d", conf->node_prefix, nodenames, i);
 }
 #endif /* HAVE_BG */
+
 /* 
  * _build_single_nodeline_info - From the slurm.conf reader, build table,
  * 	and set values
@@ -368,26 +374,27 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 	}
 
 	if ((alias_list = hostlist_create(node_ptr->nodenames)) == NULL) {
-		error("Unable to create NodeName list from %s",
+		fatal("Unable to create NodeName list from %s",
 		      node_ptr->nodenames);
 		error_code = errno;
 		goto cleanup;
 	}
 	if ((hostname_list = hostlist_create(node_ptr->hostnames)) == NULL) {
-		error("Unable to create NodeHostname list from %s",
+		fatal("Unable to create NodeHostname list from %s",
 		      node_ptr->hostnames);
 		error_code = errno;
 		goto cleanup;
 	}
 	if ((address_list = hostlist_create(node_ptr->addresses)) == NULL) {
-		error("Unable to create NodeAddr list from %s",
+		fatal("Unable to create NodeAddr list from %s",
 		      node_ptr->addresses);
 		error_code = errno;
 		goto cleanup;
 	}
 
 #ifdef HAVE_BG
-	_set_node_prefix(node_ptr->nodenames, conf);
+	if (conf->node_prefix == NULL)
+		_set_node_prefix(node_ptr->nodenames, conf);
 #endif
 
 	/* some sanity checks */
@@ -418,14 +425,16 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 		hostname = hostlist_shift(hostname_list);
 		address = hostlist_shift(address_list);
 #endif		
-		if (strcmp(alias, highest_node_name) <= 0) {
+		if (highest_node_name &&
+		    (strcmp(alias, highest_node_name) <= 0)) {
 			/* find_node_record locks this to get the
 			   alias so we need to unlock */
 			slurm_conf_unlock();
 			node_rec = find_node_record(alias);
 			slurm_conf_lock();			
 		} else {
-			strncpy(highest_node_name, alias, MAX_SLURM_NAME);
+			xfree(highest_node_name);
+			highest_node_name = xstrdup(alias);
 			node_rec = NULL;
 		}
 
@@ -435,7 +444,7 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 			    (state_val != NODE_STATE_UNKNOWN))
 				node_rec->node_state = state_val;
 			node_rec->last_response = (time_t) 0;
-			strncpy(node_rec->comm_name, address, MAX_SLURM_NAME);
+			node_rec->comm_name = xstrdup(address);
 
 			node_rec->port = node_ptr->port;
 			node_rec->reason = xstrdup(node_ptr->reason);
@@ -563,6 +572,22 @@ static int _build_all_nodeline_info(slurm_ctl_conf_t *conf)
 
 		_build_single_nodeline_info(node, config_ptr, conf);
 	}
+	xfree(highest_node_name);
+#ifdef HAVE_BG
+{
+	char *node_000 = NULL;
+	struct node_record *node_rec = NULL;
+	if (conf->node_prefix)
+		node_000 = xstrdup(conf->node_prefix);
+	xstrcat(node_000, "000");
+	slurm_conf_unlock();
+	node_rec = find_node_record(node_000);
+	slurm_conf_lock();
+	if (node_rec == NULL)
+		fatal("No node %s configured", node_000);
+	xfree(node_000);
+}
+#endif
 	return SLURM_SUCCESS;
 }
 
@@ -578,37 +603,44 @@ static int _build_single_partitionline_info(slurm_conf_partition_t *part)
 {
 	struct part_record *part_ptr;
 
-	if (strlen(part->name) >= MAX_SLURM_NAME) {
-		error("_parse_part_spec: partition name %s too long",
-		      part->name);
-		return EINVAL;
-	}
-
 	part_ptr = list_find_first(part_list, &list_find_part, part->name);
 	if (part_ptr == NULL) {
 		part_ptr = create_part_record();
-		strcpy(part_ptr->name, part->name);
+		xfree(part_ptr->name);
+		part_ptr->name = xstrdup(part->name);
 	} else {
 		verbose("_parse_part_spec: duplicate entry for partition %s",
 			part->name);
 	}
 
 	if (part->default_flag) {
-		if ((strlen(default_part_name) > 0)
+		if (default_part_name
 		&&  strcmp(default_part_name, part->name))
 			info("_parse_part_spec: changing default partition "
 				"from %s to %s", 
 				default_part_name, part->name);
-		strcpy(default_part_name, part->name);
+		xfree(default_part_name);
+		default_part_name = xstrdup(part->name);
 		default_part_loc = part_ptr;
 	}
-	part_ptr->hidden    = part->hidden_flag ? 1 : 0;
-	part_ptr->max_time  = part->max_time;
-	part_ptr->max_nodes = part->max_nodes;
-	part_ptr->min_nodes = part->min_nodes;
-	part_ptr->root_only = part->root_only_flag ? 1 : 0;
-	part_ptr->state_up  = part->state_up_flag ? 1 : 0;
-	part_ptr->shared    = part->shared;
+	if(part->disable_root_jobs == (uint16_t)NO_VAL)
+		part_ptr->disable_root_jobs = slurmctld_conf.disable_root_jobs;
+	else
+		part_ptr->disable_root_jobs = part->disable_root_jobs;
+	
+	if(part_ptr->disable_root_jobs) 
+		debug2("partition %s does not allow root jobs", part_ptr->name);
+	
+	part_ptr->hidden         = part->hidden_flag ? 1 : 0;
+	part_ptr->max_time       = part->max_time;
+	part_ptr->max_share      = part->max_share;
+	part_ptr->max_nodes      = part->max_nodes;
+	part_ptr->max_nodes_orig = part->max_nodes;
+	part_ptr->min_nodes      = part->min_nodes;
+	part_ptr->min_nodes_orig = part->min_nodes;
+	part_ptr->priority       = part->priority;
+	part_ptr->root_only      = part->root_only_flag ? 1 : 0;
+	part_ptr->state_up       = part->state_up_flag ? 1 : 0;
 	if (part->allow_groups) {
 		xfree(part_ptr->allow_groups);
 		part_ptr->allow_groups = xstrdup(part->allow_groups);
@@ -673,19 +705,20 @@ static int _build_all_partitionline_info()
  *              state information depending upon value
  *              0 = use no saved state information
  *              1 = recover saved job and trigger state, 
- *                  node DOWN/DRAIN state and reason information
+ *                  node DOWN/DRAIN/FAIL state and reason information
  *              2 = recover all state saved from last slurmctld shutdown
- * RET 0 if no error, otherwise an error code
+ * RET SLURM_SUCCESS if no error, otherwise an error code
  * Note: Operates on common variables only
  */
 int read_slurm_conf(int recover)
 {
 	DEF_TIMERS;
-	int error_code, i;
+	int error_code, i, rc, load_job_ret = SLURM_SUCCESS;
 	int old_node_record_count;
 	struct node_record *old_node_table_ptr;
 	char *old_auth_type       = xstrdup(slurmctld_conf.authtype);
 	char *old_checkpoint_type = xstrdup(slurmctld_conf.checkpoint_type);
+	char *old_crypto_type     = xstrdup(slurmctld_conf.crypto_type);
 	char *old_sched_type      = xstrdup(slurmctld_conf.schedtype);
 	char *old_select_type     = xstrdup(slurmctld_conf.select_type);
 	char *old_switch_type     = xstrdup(slurmctld_conf.switch_type);
@@ -707,9 +740,16 @@ int read_slurm_conf(int recover)
 	old_node_record_count = node_record_count;
 	old_node_table_ptr    = node_record_table_ptr;
 	for (i=0; i<node_record_count; i++) {
+		xfree(old_node_table_ptr[i].arch);
 		xfree(old_node_table_ptr[i].features);
+		xfree(old_node_table_ptr[i].os);
 		old_node_table_ptr[i].features = xstrdup(
 			old_node_table_ptr[i].config_ptr->feature);
+		/* Store the original configured CPU count somewhere
+		 * (port is reused here for that purpose) so we can
+		 * report changes in its configuration. */
+		old_node_table_ptr[i].port = old_node_table_ptr[i].
+					     config_ptr->cpus;
 	}
 	node_record_table_ptr = NULL;
 	node_record_count = 0;
@@ -725,7 +765,6 @@ int read_slurm_conf(int recover)
 	_build_all_partitionline_info();
 
 	update_logging();
-	jobacct_g_init_slurmctld(slurmctld_conf.job_acct_logfile);
 	g_slurm_jobcomp_init(slurmctld_conf.job_comp_loc);
 	slurm_sched_init();
 	if (switch_init() < 0)
@@ -747,28 +786,30 @@ int read_slurm_conf(int recover)
 	if (recover > 1) {	/* Load node, part and job info */
 		(void) load_all_node_state(false);
 		(void) load_all_part_state();
-		(void) load_all_job_state();
+		load_job_ret = load_all_job_state();
 	} else if (recover == 1) {	/* Load job info only */
 		(void) load_all_node_state(true);
-		(void) load_all_job_state();
+		load_job_ret = load_all_job_state();
 	} else {	/* Load no info, preserve all state */
 		if (old_node_table_ptr) {
-			debug("restoring original state of nodes");
-			_restore_node_state(old_node_table_ptr, 
-					    old_node_record_count);
+			info("restoring original state of nodes");
+			rc = _restore_node_state(old_node_table_ptr, 
+						 old_node_record_count);
+			error_code = MAX(error_code, rc);  /* not fatal */
 		}
 		reset_first_job_id();
 		(void) slurm_sched_reconfig();
 		xfree(state_save_dir);
 	}
 
+	_build_bitmaps_pre_select();
 	if ((select_g_node_init(node_record_table_ptr, node_record_count)
-			!= SLURM_SUCCESS) 
+	     != SLURM_SUCCESS) 
 	    || (select_g_block_init(part_list) != SLURM_SUCCESS) 
 	    || (select_g_state_restore(state_save_dir) != SLURM_SUCCESS) 
 	    || (select_g_job_init(job_list) != SLURM_SUCCESS)) {
-		error("failed to initialize node selection plugin state");
-		abort();
+		fatal("failed to initialize node selection plugin state, "
+		      "Clean start required.");
 	}
 	xfree(state_save_dir);
 	reset_job_bitmaps();		/* must follow select_g_job_init() */
@@ -777,8 +818,14 @@ int read_slurm_conf(int recover)
 	(void) sync_job_files();
 	_purge_old_node_state(old_node_table_ptr, old_node_record_count);
 
-	if ((error_code = _build_bitmaps()))
-		return error_code;
+	if ((rc = _build_bitmaps()))
+		fatal("_build_bitmaps failure");
+
+	license_free();
+	if (license_init(slurmctld_conf.licenses) != SLURM_SUCCESS)
+		fatal("Invalid Licenses value: %s", slurmctld_conf.licenses);
+
+	_restore_job_dependencies();
 	restore_node_features();
 #ifdef 	HAVE_ELAN
 	_validate_node_proc_count();
@@ -793,15 +840,21 @@ int read_slurm_conf(int recover)
 	list_sort(config_list, &list_compare_config);
 
 	/* Update plugins as possible */
-	error_code = _preserve_plugins(&slurmctld_conf,
-			old_auth_type, old_checkpoint_type,
-			old_sched_type, old_select_type,
-			old_switch_type);
+	rc = _preserve_plugins(&slurmctld_conf,
+			       old_auth_type, old_checkpoint_type,
+			       old_crypto_type, old_sched_type, 
+			       old_select_type, old_switch_type);
+	error_code = MAX(error_code, rc);	/* not fatal */
 
 	/* Update plugin parameters as possible */
-	error_code = _preserve_select_type_param(
-		        &slurmctld_conf,
-			old_select_type_p);
+	rc = _preserve_select_type_param(&slurmctld_conf,
+					 old_select_type_p);
+	error_code = MAX(error_code, rc);	/* not fatal */
+
+	/* Restore job accounting info if file missing or corrupted,
+	 * an extremely rare situation */
+	if (load_job_ret)
+		_acct_restore_active_jobs();
 
 	slurmctld_conf.last_update = time(NULL);
 	END_TIMER2("read_slurm_conf");
@@ -811,11 +864,11 @@ int read_slurm_conf(int recover)
 
 /* Restore node state and size information from saved records.
  * If a node was re-configured to be down or drained, we set those states */
-static void _restore_node_state(struct node_record *old_node_table_ptr, 
+static int _restore_node_state(struct node_record *old_node_table_ptr, 
 				int old_node_record_count)
 {
 	struct node_record *node_ptr;
-	int i;
+	int i, rc = SLURM_SUCCESS;
 
 	for (i = 0; i < old_node_record_count; i++) {
 		uint16_t drain_flag = false, down_flag = false;
@@ -836,6 +889,12 @@ static void _restore_node_state(struct node_record *old_node_table_ptr,
 			node_ptr->node_state |= NODE_STATE_DRAIN; 
 			
 		node_ptr->last_response = old_node_table_ptr[i].last_response;
+		if (old_node_table_ptr[i].port != node_ptr->config_ptr->cpus) {
+			rc = ESLURM_NEED_RESTART;
+			error("Configured cpu count change on %s (%u to %u)", 
+			      node_ptr->name, old_node_table_ptr[i].port, 
+			      node_ptr->config_ptr->cpus);
+		}
 		node_ptr->cpus          = old_node_table_ptr[i].cpus;
 		node_ptr->sockets       = old_node_table_ptr[i].sockets;
 		node_ptr->cores         = old_node_table_ptr[i].cores;
@@ -852,7 +911,18 @@ static void _restore_node_state(struct node_record *old_node_table_ptr,
 			node_ptr->features = old_node_table_ptr[i].features;
 			old_node_table_ptr[i].features = NULL;
 		}
+		if (old_node_table_ptr[i].arch) {
+			xfree(node_ptr->arch);
+			node_ptr->arch = old_node_table_ptr[i].arch;
+			old_node_table_ptr[i].arch = NULL;
+		}
+		if (old_node_table_ptr[i].os) {
+			xfree(node_ptr->os);
+			node_ptr->os = old_node_table_ptr[i].os;
+			old_node_table_ptr[i].os = NULL;
+		}
 	}
+	return rc;
 }
 
 /* Purge old node state information */
@@ -862,8 +932,12 @@ static void _purge_old_node_state(struct node_record *old_node_table_ptr,
 	int i;
 
 	for (i = 0; i < old_node_record_count; i++) {
-		xfree(old_node_table_ptr[i].part_pptr);
+		xfree(old_node_table_ptr[i].arch);
+		xfree(old_node_table_ptr[i].comm_name);
 		xfree(old_node_table_ptr[i].features);
+		xfree(old_node_table_ptr[i].name);
+		xfree(old_node_table_ptr[i].os);
+		xfree(old_node_table_ptr[i].part_pptr);
 		xfree(old_node_table_ptr[i].reason);
 	}
 	xfree(old_node_table_ptr);
@@ -900,8 +974,8 @@ static int  _preserve_select_type_param(slurm_ctl_conf_t *ctl_conf_ptr,
  */
 static int  _preserve_plugins(slurm_ctl_conf_t * ctl_conf_ptr, 
 		char *old_auth_type, char *old_checkpoint_type,
-		char *old_sched_type, char *old_select_type, 
-		char *old_switch_type)
+		char *old_crypto_type, char *old_sched_type, 
+		char *old_select_type, char *old_switch_type)
 {
 	int rc = SLURM_SUCCESS;
 
@@ -924,6 +998,16 @@ static int  _preserve_plugins(slurm_ctl_conf_t * ctl_conf_ptr,
 			xfree(old_checkpoint_type);
 	}
 
+	if (old_crypto_type) {
+		if (strcmp(old_crypto_type,
+				ctl_conf_ptr->crypto_type)) {
+			xfree(ctl_conf_ptr->crypto_type);
+			ctl_conf_ptr->crypto_type = old_crypto_type;
+			rc = ESLURM_INVALID_CRYPTO_TYPE_CHANGE;
+		} else
+			xfree(old_crypto_type);
+	}
+
 	if (old_sched_type) {
 		if (strcmp(old_sched_type, ctl_conf_ptr->schedtype)) {
 			xfree(ctl_conf_ptr->schedtype);
@@ -1097,3 +1181,80 @@ static void _validate_node_proc_count(void)
 }
 #endif
 
+/*
+ * _restore_job_dependencies - Build depend_list and license_list for every job
+ *	also reset the runing job count for scheduling policy
+ */
+static int _restore_job_dependencies(void)
+{
+	int error_code = SLURM_SUCCESS, rc;
+	struct job_record *job_ptr;
+	ListIterator job_iterator;
+	char *new_depend;
+	bool valid;
+	List license_list;
+
+	assoc_mgr_clear_used_info();
+	job_iterator = list_iterator_create(job_list);
+	while ((job_ptr = (struct job_record *) list_next(job_iterator))) {
+		if (accounting_enforce &&
+		    ((job_ptr->job_state == JOB_RUNNING) ||
+		     (job_ptr->job_state == JOB_SUSPENDED)))
+			acct_policy_job_begin(job_ptr);
+
+		license_list = license_job_validate(job_ptr->licenses, &valid);
+		if (job_ptr->license_list)
+			list_destroy(job_ptr->license_list);
+		if (valid)
+			job_ptr->license_list = license_list;
+		if (job_ptr->job_state == JOB_RUNNING) 
+			license_job_get(job_ptr);
+
+		if ((job_ptr->details == NULL) ||
+		    (job_ptr->details->dependency == NULL))
+			continue;
+		new_depend = job_ptr->details->dependency;
+		job_ptr->details->dependency = NULL;
+		rc = update_job_dependency(job_ptr, new_depend);
+		if (rc != SLURM_SUCCESS) {
+			error("Invalid dependencies discarded for job %u: %s",
+				job_ptr->job_id, new_depend);
+			error_code = rc;
+		}
+		xfree(new_depend);
+	}
+	list_iterator_destroy(job_iterator);
+	return error_code;
+}
+
+/* Flush accounting information on this cluster, then for each running or 
+ * suspended job, restore its state in the accounting system */
+static void _acct_restore_active_jobs(void)
+{
+	struct job_record *job_ptr;
+	ListIterator job_iterator;
+	struct step_record *step_ptr;
+	ListIterator step_iterator;
+
+	info("Reinitializing job accounting state");
+	acct_storage_g_flush_jobs_on_cluster(acct_db_conn,
+					     slurmctld_cluster_name,
+					     time(NULL));
+	job_iterator = list_iterator_create(job_list);
+	while ((job_ptr = (struct job_record *) list_next(job_iterator))) {
+		if (job_ptr->job_state == JOB_SUSPENDED)
+			jobacct_storage_g_job_suspend(acct_db_conn, job_ptr);
+		if ((job_ptr->job_state == JOB_SUSPENDED) ||
+		    (job_ptr->job_state == JOB_RUNNING)) {
+			jobacct_storage_g_job_start(acct_db_conn, job_ptr);
+			step_iterator = list_iterator_create(job_ptr->step_list);
+			while ((step_ptr = (struct step_record *) 
+					   list_next(step_iterator))) {
+				jobacct_storage_g_step_start(acct_db_conn, 
+							     step_ptr);
+			}
+			list_iterator_destroy (step_iterator);
+		}
+	}
+	list_iterator_destroy(job_iterator);
+}
