@@ -1,11 +1,13 @@
 /*****************************************************************************\
  *  src/slurmd/slurmd/slurmd.c - main slurm node server daemon
- *  $Id: slurmd.c 11829 2007-07-12 16:14:24Z jette $
+ *  $Id: slurmd.c 14314 2008-06-23 20:57:56Z jette $
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Portions Copyright (C) 2008 Vijay Ramasubramanian.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -49,6 +51,7 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/resource.h>
+#include <sys/utsname.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/mman.h>
@@ -258,6 +261,8 @@ main (int argc, char *argv[])
 
 	_slurmd_fini();
 	_destroy_conf();
+	slurm_crypto_fini();	/* must be after _destroy_conf() */
+
 	info("Slurmd shutdown completing");
 	log_fini();
        	return 0;
@@ -447,20 +452,31 @@ _fill_registration_msg(slurm_node_registration_status_msg_t *msg)
 	List steps;
 	ListIterator i;
 	step_loc_t *stepd;
-	int          n;
+	int  n;
+	char *arch, *os;
+	struct utsname buf;
 
-	msg->node_name = xstrdup (conf->node_name);
-	msg->cpus	= conf->cpus;
-	msg->sockets	= conf->sockets;
-	msg->cores	= conf->cores;
-	msg->threads	= conf->threads;
-
-	msg->real_memory_size     = conf->real_memory_size;
-	msg->temporary_disk_space = conf->tmp_disk_space;
+	msg->node_name  = xstrdup (conf->node_name);
+	msg->cpus	 = conf->cpus;
+	msg->sockets	 = conf->sockets;
+	msg->cores	 = conf->cores;
+	msg->threads	 = conf->threads;
+	msg->real_memory = conf->real_memory_size;
+	msg->tmp_disk    = conf->tmp_disk_space;
 
 	debug3("Procs=%u Sockets=%u Cores=%u Threads=%u Memory=%u TmpDisk=%u",
 	       msg->cpus, msg->sockets, msg->cores, msg->threads,
-	       msg->real_memory_size, msg->temporary_disk_space);
+	       msg->real_memory, msg->tmp_disk);
+
+	uname(&buf);
+	if ((arch = getenv("SLURM_ARCH")))
+		msg->arch = xstrdup(arch);
+	else
+		msg->arch = xstrdup(buf.machine);
+	if ((os = getenv("SLURM_OS")))
+		msg->os   = xstrdup(os);
+	else
+		msg->os = xstrdup(buf.sysname);
 
 	if (msg->startup) {
 		if (switch_g_alloc_node_info(&msg->switch_nodeinfo))
@@ -559,8 +575,6 @@ _read_config()
 
 	conf->cr_type = cf->select_type_param;
 
-	conf->fast_schedule = cf->fast_schedule;
-
 	path_pubkey = xstrdup(cf->job_credential_public_certificate);
 
 	if (!conf->logfile)
@@ -570,6 +584,11 @@ _read_config()
 	/* node_name may already be set from a command line parameter */
 	if (conf->node_name == NULL)
 		conf->node_name = slurm_conf_get_nodename(conf->hostname);
+	/* if we didn't match the form of the hostname already
+	 * stored in conf->hostname, check to see if we match any
+	 * valid aliases */
+	if (conf->node_name == NULL)
+		conf->node_name = slurm_conf_get_aliased_nodename();
 	if (conf->node_name == NULL)
 		conf->node_name = slurm_conf_get_nodename("localhost");
 	if (conf->node_name == NULL)
@@ -597,17 +616,10 @@ _read_config()
 		    &conf->block_map_size,
 		    &conf->block_map, &conf->block_map_inv);
 
-	if (conf->fast_schedule) {
-	    	conf->cpus    = conf->conf_cpus;
-		conf->sockets = conf->conf_sockets;
-		conf->cores   = conf->conf_cores;
-		conf->threads = conf->conf_threads;
-	} else {
-	    	conf->cpus    = conf->actual_cpus;
-		conf->sockets = conf->actual_sockets;
-		conf->cores   = conf->actual_cores;
-		conf->threads = conf->actual_threads;
-	}
+	conf->cpus    = conf->actual_cpus;
+	conf->sockets = conf->actual_sockets;
+	conf->cores   = conf->actual_cores;
+	conf->threads = conf->actual_threads;
 
 	get_memory(&conf->real_memory_size);
 
@@ -616,6 +628,8 @@ _read_config()
 	_free_and_set(&conf->epilog,   xstrdup(cf->epilog));
 	_free_and_set(&conf->prolog,   xstrdup(cf->prolog));
 	_free_and_set(&conf->tmpfs,    xstrdup(cf->tmp_fs));
+	_free_and_set(&conf->health_check_program, 
+		      xstrdup(cf->health_check_program));
 	_free_and_set(&conf->spooldir, xstrdup(cf->slurmd_spooldir));
 	_massage_pathname(&conf->spooldir);
 	_free_and_set(&conf->pidfile,  xstrdup(cf->slurmd_pidfile));
@@ -625,7 +639,7 @@ _read_config()
 	_free_and_set(&conf->pubkey,   path_pubkey);
 	
 	conf->propagate_prio = cf->propagate_prio_process;
-	conf->job_acct_freq = cf->job_acct_freq;
+	conf->job_acct_gather_freq = cf->job_acct_gather_freq;
 
 	if ( (conf->node_name == NULL) ||
 	     (conf->node_name[0] == '\0') )
@@ -721,6 +735,7 @@ _print_conf()
 	debug3("TmpDisk     = %u",       conf->tmp_disk_space);
 	debug3("Epilog      = `%s'",     conf->epilog);
 	debug3("Logfile     = `%s'",     cf->slurmd_logfile);
+	debug3("HealthCheck = `%s'",     conf->health_check_program);
 	debug3("NodeName    = %s",       conf->node_name);
 	debug3("Port        = %u",       conf->port);
 	debug3("Prolog      = `%s'",     conf->prolog);
@@ -733,7 +748,6 @@ _print_conf()
 	debug3("TaskEpilog  = `%s'",     conf->task_epilog);
 	debug3("Use CPUSETS = %u",       conf->use_cpusets);
 	debug3("Use PAM     = %u",       conf->use_pam);
-	debug3("Fast Sched  = %u",       conf->fast_schedule);
 	slurm_conf_unlock();
 }
 
@@ -757,6 +771,7 @@ _init_conf()
 	conf->block_map_inv = NULL;
 	conf->conffile    = NULL;
 	conf->epilog      = NULL;
+	conf->health_check_program = NULL;
 	conf->logfile     = NULL;
 	conf->pubkey      = NULL;
 	conf->prolog      = NULL;
@@ -774,7 +789,6 @@ _init_conf()
 	conf->spooldir	  = xstrdup(DEFAULT_SPOOLDIR);
 	conf->use_pam	  =  0;
 	conf->use_cpusets =  0;
-	conf->fast_schedule = 0;
 
 	slurm_mutex_init(&conf->config_mutex);
 	return;
@@ -786,6 +800,7 @@ _destroy_conf()
 	if(conf) {
 		xfree(conf->block_map);
 		xfree(conf->block_map_inv);
+		xfree(conf->health_check_program);
 		xfree(conf->hostname);
 		xfree(conf->node_name);
 		xfree(conf->conffile);
@@ -903,7 +918,7 @@ _slurmd_init()
 		return SLURM_FAILURE;
 	if (slurmd_task_init() != SLURM_SUCCESS)
 		return SLURM_FAILURE;
-	if (slurm_auth_init() != SLURM_SUCCESS)
+	if (slurm_auth_init(NULL) != SLURM_SUCCESS)
 		return SLURM_FAILURE;
 
 	if (getrlimit(RLIMIT_NOFILE,&rlim) == 0) {
