@@ -1,11 +1,12 @@
 /*****************************************************************************\
  *  src/slurmd/slurmstepd/mgr.c - job manager functions for slurmstepd
- *  $Id: mgr.c 12647 2007-11-12 17:09:47Z da $
+ *  $Id: mgr.c 14504 2008-07-14 17:38:53Z jette $
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -82,7 +83,7 @@
 #include "src/common/node_select.h"
 #include "src/common/fd.h"
 #include "src/common/safeopen.h"
-#include "src/common/slurm_jobacct.h"
+#include "src/common/slurm_jobacct_gather.h"
 #include "src/common/switch.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
@@ -139,6 +140,7 @@ step_complete_t step_complete = {
 	{},
 	-1,
 	-1,
+	true,
 	(bitstr_t *)NULL,
 	0,
         NULL
@@ -157,6 +159,7 @@ typedef struct kill_thread {
 /* 
  * Job manager related prototypes
  */
+static int  _access(const char *path, int modes, uid_t uid, gid_t gid);
 static void _send_launch_failure(launch_tasks_request_msg_t *, 
                                  slurm_addr *, int);
 static int  _fork_all_tasks(slurmd_job_t *job);
@@ -172,6 +175,7 @@ static void _slurmd_job_log_init(slurmd_job_t *job);
 static void _wait_for_io(slurmd_job_t *job);
 static int  _send_exit_msg(slurmd_job_t *job, uint32_t *tid, int n, 
 		int status);
+static void _wait_for_children_slurmstepd(slurmd_job_t *job);
 static int  _send_pending_exit_msgs(slurmd_job_t *job);
 static void _send_step_complete_msgs(slurmd_job_t *job);
 static void _wait_for_all_tasks(slurmd_job_t *job);
@@ -228,7 +232,10 @@ mgr_launch_tasks_setup(launch_tasks_request_msg_t *msg, slurm_addr *cli,
 static void
 _batch_finish(slurmd_job_t *job, int rc)
 {
-	int status = job->task[0]->estatus;
+	int i;
+	for (i = 0; i < job->ntasks; i++)
+		step_complete.step_rc = MAX(step_complete.step_rc,
+					    WEXITSTATUS(job->task[i]->estatus));
 
 	if (job->argv[0] && (unlink(job->argv[0]) < 0))
 		error("unlink(%s): %m", job->argv[0]);
@@ -237,11 +244,12 @@ _batch_finish(slurmd_job_t *job, int rc)
 	xfree(job->batchdir);
 	if ((job->stepid == NO_VAL) || (job->stepid == SLURM_BATCH_SCRIPT)) {
 		verbose("job %u completed with slurm_rc = %d, job_rc = %d",
-			job->jobid, rc, status);
-		_send_complete_batch_script_msg(job, rc, status);
+			job->jobid, rc, step_complete.step_rc);
+		_send_complete_batch_script_msg(job, rc, job->task[0]->estatus);
 	} else {
+		_wait_for_children_slurmstepd(job);
 		verbose("job %u.%u completed with slurm_rc = %d, job_rc = %d",
-			job->jobid, job->stepid, rc, status);
+			job->jobid, job->stepid, rc, step_complete.step_rc);
 		_send_step_complete_msgs(job);
 	}
 }
@@ -280,7 +288,7 @@ mgr_launch_batch_job_setup(batch_job_launch_msg_t *msg, slurm_addr *cli)
 	}
 	
 	/* this is the new way of setting environment variables */
-	env_array_for_batch_job(&job->env, msg);
+	env_array_for_batch_job(&job->env, msg, conf->node_name);
 
 	/* this is the old way of setting environment variables */
 	job->envtp->nprocs = msg->nprocs;
@@ -336,7 +344,7 @@ _setup_normal_io(slurmd_job_t *job)
 
 	/*
 	 * Temporarily drop permissions, initialize task stdio file
-	 * decriptors (which may be connected to files), then
+	 * descriptors (which may be connected to files), then
 	 * reclaim privileges.
 	 */
 	if (_drop_privileges(job, true, &sprivs) < 0)
@@ -478,6 +486,8 @@ _wait_for_children_slurmstepd(slurmd_job_t *job)
 		step_complete.step_rc = MAX(step_complete.step_rc,
 					 WEXITSTATUS(job->task[i]->estatus));
 
+	step_complete.wait_children = false;
+
 	pthread_mutex_unlock(&step_complete.lock);
 }
 
@@ -495,6 +505,7 @@ _one_step_complete_msg(slurmd_job_t *job, int first, int last)
 	int rc = -1;
 	int retcode;
 	int i;
+	static bool acct_sent = false;
 
 	debug2("_one_step_complete_msg: first=%d, last=%d", first, last);
 	msg.job_id = job->jobid;
@@ -502,11 +513,14 @@ _one_step_complete_msg(slurmd_job_t *job, int first, int last)
 	msg.range_first = first;
 	msg.range_last = last;
 	msg.step_rc = step_complete.step_rc;
-	msg.jobacct = jobacct_g_alloc(NULL);
+	msg.jobacct = jobacct_gather_g_create(NULL);
 	/************* acct stuff ********************/
-	jobacct_g_aggregate(step_complete.jobacct, job->jobacct);
-	jobacct_g_getinfo(step_complete.jobacct, JOBACCT_DATA_TOTAL, 
-			  msg.jobacct);
+	if(!acct_sent) {
+		jobacct_gather_g_aggregate(step_complete.jobacct, job->jobacct);
+		jobacct_gather_g_getinfo(step_complete.jobacct, JOBACCT_DATA_TOTAL, 
+				  msg.jobacct);
+		acct_sent = true;
+	}
 	/*********************************************/	
 	slurm_msg_t_init(&req);
 	req.msg_type = REQUEST_STEP_COMPLETE;
@@ -548,7 +562,7 @@ _one_step_complete_msg(slurmd_job_t *job, int first, int last)
 		error("Rank %d failed sending step completion message"
 		      " directly to slurmctld", step_complete.rank);
 finished:
-	jobacct_g_free(msg.jobacct);
+	jobacct_gather_g_destroy(msg.jobacct);
 }
 
 /* Given a starting bit in the step_complete.bits bitstring, "start",
@@ -656,10 +670,13 @@ job_manager(slurmd_job_t *job)
 	 */
 	if (switch_init() != SLURM_SUCCESS
 	    || slurmd_task_init() != SLURM_SUCCESS
-	    || mpi_hook_slurmstepd_init(&job->env) != SLURM_SUCCESS
 	    || slurm_proctrack_init() != SLURM_SUCCESS
-	    || jobacct_init() != SLURM_SUCCESS) {
-		rc = SLURM_FAILURE;
+	    || slurm_jobacct_gather_init() != SLURM_SUCCESS) {
+		rc = SLURM_PLUGIN_NAME_INVALID;
+		goto fail1;
+	}
+	if (mpi_hook_slurmstepd_init(&job->env) != SLURM_SUCCESS) {
+		rc = SLURM_MPI_PLUGIN_NAME_INVALID;
 		goto fail1;
 	}
 	
@@ -716,12 +733,17 @@ job_manager(slurmd_job_t *job)
 	reattach_job = job;
 
 	job->state = SLURMSTEPD_STEP_RUNNING;
-
+	
+	/* if we are not polling then we need to make sure we get some
+	 * information here
+	 */
+	if(!conf->job_acct_gather_freq)
+		jobacct_gather_g_stat_task(0);
 	/* Send job launch response with list of pids */
 	_send_launch_resp(job, 0);
 
 	_wait_for_all_tasks(job);
-	jobacct_g_endpoll();
+	jobacct_gather_g_endpoll();
 		
 	job->state = SLURMSTEPD_STEP_ENDING;
 
@@ -901,7 +923,7 @@ _fork_all_tasks(slurmd_job_t *job)
 				if (j > i)
 					close(readfds[j]);
 			}
-			/* jobacct_g_endpoll();	
+			/* jobacct_gather_g_endpoll();	
 			 * closing jobacct files here causes deadlock */
 
 			if (conf->propagate_prio == 1)
@@ -954,19 +976,26 @@ _fork_all_tasks(slurmd_job_t *job)
 
 	for (i = 0; i < job->ntasks; i++) {
 		/*
-                 * Put this task in the step process group
-                 */
-                if (setpgid (job->task[i]->pid, job->pgid) < 0)
-                        error ("Unable to put task %d (pid %ld) into pgrp %ld",
-                               i, job->task[i]->pid, job->pgid);
+		 * Put this task in the step process group
+		 * login_tty() must put task zero in its own
+		 * session, causing setpgid() to fail, setsid()
+		 * has already set its process group as desired
+		 */
+		if ((job->pty == 0)
+		&&  (setpgid (job->task[i]->pid, job->pgid) < 0)) {
+			error("Unable to put task %d (pid %ld) into "
+				"pgrp %ld: %m",
+				i, job->task[i]->pid, job->pgid);
+		}
 
-                if (slurm_container_add(job, job->task[i]->pid) == SLURM_ERROR) {
-                        error("slurm_container_create: %m");
+                if (slurm_container_add(job, job->task[i]->pid)
+		    == SLURM_ERROR) {
+                        error("slurm_container_add: %m");
 			goto fail1;
                 }
 		jobacct_id.nodeid = job->nodeid;
 		jobacct_id.taskid = job->task[i]->gtid;
-		jobacct_g_add_task(job->task[i]->pid, 
+		jobacct_gather_g_add_task(job->task[i]->pid, 
 				   &jobacct_id);
 
 		if (spank_task_post_fork (job, i) < 0) {
@@ -974,8 +1003,8 @@ _fork_all_tasks(slurmd_job_t *job)
 			return SLURM_ERROR;
 		}
 	}
-	jobacct_g_set_proctrack_container_id(job->cont_id);
-
+	jobacct_gather_g_set_proctrack_container_id(job->cont_id);
+	
 	/*
 	 * Now it's ok to unblock the tasks, so they may call exec.
 	 */
@@ -1096,12 +1125,12 @@ _wait_for_any_task(slurmd_job_t *job, bool waitflag)
 		}
 
 		/************* acct stuff ********************/
-		jobacct = jobacct_g_remove_task(pid);
+		jobacct = jobacct_gather_g_remove_task(pid);
 		if(jobacct) {
-			jobacct_g_setinfo(jobacct, 
-					  JOBACCT_DATA_RUSAGE, &rusage);
-			jobacct_g_aggregate(job->jobacct, jobacct);
-			jobacct_g_free(jobacct);
+			jobacct_gather_g_setinfo(jobacct, 
+						 JOBACCT_DATA_RUSAGE, &rusage);
+			jobacct_gather_g_aggregate(job->jobacct, jobacct);
+			jobacct_gather_g_destroy(jobacct);
 		} 		
 		/*********************************************/	
 	
@@ -1129,7 +1158,7 @@ _wait_for_any_task(slurmd_job_t *job, bool waitflag)
 			if (job->task_epilog) {
 				_run_script_as_user("user task_epilog",
 						    job->task_epilog,
-						    job, 2, job->env);
+						    job, 5, job->env);
 			}
 			if (conf->task_epilog) {
 				char *my_epilog;
@@ -1659,6 +1688,34 @@ _initgroups(slurmd_job_t *job)
 	return 0;
 }
 
+/*
+ * Check this user's access rights to a file
+ * path IN: pathname of file to test
+ * modes IN: desired access
+ * uid IN: user ID to access the file
+ * gid IN: group ID to access the file
+ * RET 0 on success, -1 on failure
+ */
+static int _access(const char *path, int modes, uid_t uid, gid_t gid)
+{
+	struct stat buf;
+	int f_mode;
+
+	if (stat(path, &buf) != 0)
+		return -1;
+
+	if (buf.st_uid == uid)
+		f_mode = (buf.st_mode >> 6) & 07;
+	else if (buf.st_gid == gid)
+		f_mode = (buf.st_mode >> 3) & 07;
+	else
+		f_mode = buf.st_mode & 07;
+
+	if ((f_mode & modes) == modes)
+		return 0;
+	return -1;
+}
+
 /*
  * Run a script as a specific user, with the specified uid, gid, and 
  * extended groups.
@@ -1685,8 +1742,8 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 
 	debug("[job %u] attempting to run %s [%s]", job->jobid, name, path);
 
-	if (access(path, R_OK | X_OK) < 0) {
-		error("Could not run %s [%s]: %m", name, path);
+	if (_access(path, 5, job->pwd->pw_uid, job->pwd->pw_gid) < 0) {
+		error("Could not run %s [%s]: access denied", name, path);
 		return -1;
 	}
 
@@ -1717,7 +1774,11 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 		}
 
 		chdir(job->cwd);
+#ifdef SETPGRP_TWO_ARGS
+		setpgrp(0, 0);
+#else
 		setpgrp();
+#endif
 		execve(path, argv, env);
 		error("execve(): %m");
 		exit(127);
@@ -1740,7 +1801,7 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 			break;
 		} else if (rc == 0) {
 			sleep(1);
-			if ((--max_wait) == 0) {
+			if ((--max_wait) <= 0) {
 				killpg(cpid, SIGKILL);
 				opt = 0;
 			}
@@ -1749,7 +1810,7 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 			break;
 		}
 	}
-	/* Insure that all child processes get killed */
+	/* Insure that all child processes get killed, one last time */
 	killpg(cpid, SIGKILL);
 	slurm_container_signal(job->cont_id, SIGKILL);
 	
