@@ -1,12 +1,12 @@
 /*****************************************************************************\
  *  configure_functions.c - Functions related to configure mode of smap.
- *  $Id: configure_functions.c 11985 2007-08-09 23:07:08Z da $
+ *  $Id: configure_functions.c 14295 2008-06-19 23:58:28Z da $
  *****************************************************************************
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
  *
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -213,7 +213,8 @@ static int _create_allocation(char *com, List allocated_blocks)
 	request->nodecards = 0;
 	request->quarters = 0;
 	request->passthrough = false;
-	
+	request->avail_node_bitmap = NULL;
+
 	while(i<len) {				
 		if(!strncasecmp(com+i, "mesh", 4)) {
 			request->conn_type=SELECT_MESH;
@@ -584,11 +585,14 @@ static int _change_state_bps(char *com, int state)
 #ifdef HAVE_BG
 	if ((com[i+3] == 'x')
 	    || (com[i+3] == '-')) {
-		for(j=0; j<3; j++) 
-			if((i+j)>len 
-			   || (com[i+j] < '0' || com[i+j] > 'Z'
-			       || (com[i+j] > '9' && com[i+j] < 'A'))) 
-				goto error_message2;
+		for(j=0; j<3; j++) {
+			if (((i+j) <= len) &&
+			    (((com[i+j] >= '0') && (com[i+j] <= '9')) ||
+			     ((com[i+j] >= 'A') && (com[i+j] <= 'Z'))))
+				continue;
+			goto error_message2; 
+
+		}
 		number = xstrntol(com + i, NULL,
 				  BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);
 		start[X] = number / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -597,11 +601,13 @@ static int _change_state_bps(char *com, int state)
 		start[Z] = (number % HOSTLIST_BASE);
 		
 		i += 4;
-		for(j=0; j<3; j++) 
-			if((i+j)>len 
-			   || (com[i+j] < '0' || com[i+j] > 'Z'
-			       || (com[i+j] > '9' && com[i+j] < 'A'))) 
-				goto error_message2;
+		for(j=0; j<3; j++) {
+			if (((i+j) <= len) &&
+			    (((com[i+j] >= '0') && (com[i+j] <= '9')) ||
+			     ((com[i+j] >= 'A') && (com[i+j] <= 'Z'))))
+				continue; 
+			goto error_message2;
+		}
 		number = xstrntol(com + i, NULL,
 				  BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);
 		end[X] = number / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -609,11 +615,13 @@ static int _change_state_bps(char *com, int state)
 			/ HOSTLIST_BASE;
 		end[Z] = (number % HOSTLIST_BASE);			
 	} else {
-		for(j=0; j<3; j++) 
-			if((i+j)>len 
-			   || (com[i+j] < '0' || com[i+j] > 'Z'
-			       || (com[i+j] > '9' && com[i+j] < 'A')))
-				goto error_message2;
+		for(j=0; j<3; j++) {
+			if (((i+j) <= len) &&
+			    (((com[i+j] >= '0') && (com[i+j] <= '9')) ||
+			     ((com[i+j] >= 'A') && (com[i+j] <= 'Z'))))
+				continue;
+			goto error_message2;
+		}
 		number = xstrntol(com + i, NULL,
 				  BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);
 		start[X] = end[X] = number / (HOSTLIST_BASE * HOSTLIST_BASE);
@@ -648,7 +656,7 @@ static int _change_state_bps(char *com, int state)
 	if ((com[i+3] == 'x')
 	    || (com[i+3] == '-')) {
 		start[X] =  xstrntol(com + i, NULL,
-				    BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);;
+				    BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);
 		i += 4;
 		end[X] =  xstrntol(com + i, NULL, 
 				   BA_SYSTEM_DIMENSIONS, HOSTLIST_BASE);
@@ -855,7 +863,8 @@ static int _copy_allocation(char *com, List allocated_blocks)
 		request->rotate_count= 0;
 		request->elongate_count = 0;
 	       	request->elongate_geos = list_create(NULL);
-	
+		request->avail_node_bitmap = NULL;
+
 		results_i = list_iterator_create(request->elongate_geos);
 		while ((geo_ptr = list_next(results_i)) != NULL) {
 			geo = xmalloc(sizeof(int)*3);
@@ -977,6 +986,8 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 #ifdef HAVE_BG
 	char *nodes = NULL, *conn_type = NULL;
 	int bp_count = 0;
+	int diff=0;
+	int largest_diff=-1;
 	int start[BA_SYSTEM_DIMENSIONS];
 	int end[BA_SYSTEM_DIMENSIONS];
 	int start1[BA_SYSTEM_DIMENSIONS];
@@ -987,10 +998,6 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 	int len = 0;
 	int x,y,z;
 	
-	start1[X] = 0;
-	start1[Y] = 0;
-	start1[Z] = 0;
-	
 	geo[X] = 0;
 	geo[Y] = 0;
 	geo[Z] = 0;
@@ -1043,7 +1050,8 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 			end[Z] = (number % HOSTLIST_BASE);
 
 			j += 3;
-			if(!bp_count) {
+			diff = end[X]-start[X];
+			if(diff > largest_diff) {
 				start1[X] = start[X];
 				start1[Y] = start[Y];
 				start1[Z] = start[Z];
@@ -1078,7 +1086,8 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 			start[Z] = (number % HOSTLIST_BASE);
 			
 			j+=3;
-			if(!bp_count) {
+			diff = 0;
+			if(diff > largest_diff) {
 				start1[X] = start[X];
 				start1[Y] = start[Y];
 				start1[Z] = start[Z];
@@ -1098,6 +1107,7 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 			bp_count++;
 			if(nodes[j] != ',')
 				break;
+			j--;
 		}
 		j++;
 	}
@@ -1107,7 +1117,13 @@ static int _add_bg_record(blockreq_t *blockreq, List allocated_blocks)
 		geo[X], geo[Y], geo[Z], conn_type, 
 		start1[X], start1[Y], start1[Z],
 		blockreq->nodecards, blockreq->quarters);
+	if(!strcasecmp(layout_mode, "OVERLAP")) 
+		reset_ba_system(false);
+	
+	set_all_bps_except(nodes);
 	_create_allocation(com, allocated_blocks);
+	reset_all_removed_bps();
+	
 #endif
 	return SLURM_SUCCESS;
 }
