@@ -6,7 +6,7 @@
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
  * 
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -425,6 +425,7 @@ static int _marknodes(db2_block_info_t *block_ptr, int count)
 							0);
 			if(block_ptr->nodes[j] != ',')
 				break;
+			j--;
 		}
 		j++;
 	}
@@ -914,6 +915,7 @@ static int _make_nodelist(char *nodes, List nodelist)
 			_addto_nodelist(nodelist, start, start);
 			if(nodes[j] != ',')
 				break;
+			j--;
 		}
 		j++;
 	}
