@@ -1,11 +1,11 @@
 /*****************************************************************************\
  * src/srun/allocate.c - srun functions for managing node allocations
- * $Id: allocate.c 12700 2007-11-27 23:39:24Z jette $
+ * $Id: allocate.c 14570 2008-07-18 22:06:26Z da $
  *****************************************************************************
  *  Copyright (C) 2002-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -55,41 +55,138 @@
 #include "src/common/xstring.h"
 #include "src/common/forward.h"
 #include "src/common/env.h"
+#include "src/common/fd.h"
 
 #include "src/srun/allocate.h"
-#include "src/srun/msg.h"
 #include "src/srun/opt.h"
-#include "src/srun/attach.h"
+#include "src/srun/debugger.h"
 
 #define MAX_ALLOC_WAIT 60	/* seconds */
 #define MIN_ALLOC_WAIT  5	/* seconds */
 #define MAX_RETRIES    10
 
+pthread_mutex_t msg_lock = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t msg_cond = PTHREAD_COND_INITIALIZER;
+allocation_msg_thread_t *msg_thr = NULL;
+resource_allocation_response_msg_t *global_resp = NULL;
+struct pollfd global_fds[1];
+
 extern char **environ;
 
+static bool exit_flag = false;
+static uint32_t pending_job_id = 0;
+
 /*
  * Static Prototypes
  */
-static int   _accept_msg_connection(slurm_fd slurmctld_fd,
-		resource_allocation_response_msg_t **resp);
-static int   _handle_msg(slurm_msg_t *msg, \
-		resource_allocation_response_msg_t **resp);
-static int   _wait_for_alloc_rpc(int sleep_time,
-		resource_allocation_response_msg_t **resp);
-static void  _wait_for_resources(resource_allocation_response_msg_t **resp);
-static bool  _retry();
+static void _set_pending_job_id(uint32_t job_id);
+static void _exit_on_signal(int signo);
+static void _signal_while_allocating(int signo);
 static void  _intr_handler(int signo);
 
-static job_step_create_request_msg_t * _step_req_create(srun_job_t *j);
-
 static sig_atomic_t destroy_job = 0;
-static srun_job_t *allocate_job = NULL;
+
+static void _set_pending_job_id(uint32_t job_id)
+{
+	debug2("Pending job allocation %u", job_id);
+	pending_job_id = job_id;
+}
+
+static void _signal_while_allocating(int signo)
+{
+	destroy_job = 1;
+	if (pending_job_id != 0) {
+		slurm_complete_job(pending_job_id, 0);
+	}
+}
+
+static void _exit_on_signal(int signo)
+{
+	exit_flag = true;
+}
+
+/* This typically signifies the job was cancelled by scancel */
+static void _job_complete_handler(srun_job_complete_msg_t *msg)
+{
+	if((int)msg->step_id >= 0)
+		info("Force Terminated job %u.%u", msg->job_id, msg->step_id);
+	else
+		info("Force Terminated job %u", msg->job_id);
+}
+
+/*
+ * Job has been notified of it's approaching time limit. 
+ * Job will be killed shortly after timeout.
+ * This RPC can arrive multiple times with the same or updated timeouts.
+ * FIXME: We may want to signal the job or perform other action for this.
+ * FIXME: How much lead time do we want for this message? Some jobs may 
+ *	require tens of minutes to gracefully terminate.
+ */
+static void _timeout_handler(srun_timeout_msg_t *msg)
+{
+	static time_t last_timeout = 0;
+
+	if (msg->timeout != last_timeout) {
+		last_timeout = msg->timeout;
+		verbose("job time limit to be reached at %s", 
+			ctime(&msg->timeout));
+	}
+}
+
+static void _user_msg_handler(srun_user_msg_t *msg)
+{
+	info("%s", msg->msg);
+}
+
+static void _ping_handler(srun_ping_msg_t *msg) 
+{
+	/* the api will respond so there really isn't anything to do
+	   here */
+}
+
+static void _node_fail_handler(srun_node_fail_msg_t *msg)
+{
+	error("Node failure on %s", msg->nodelist);
+}
+
+
+
+static bool _retry()
+{
+	static int  retries = 0;
+	static char *msg = "Slurm controller not responding, "
+		           "sleeping and retrying.";
+
+	if (errno == ESLURM_ERROR_ON_DESC_TO_RECORD_COPY) {
+		if (retries == 0)
+			error (msg);
+		else if (retries < MAX_RETRIES)
+			debug (msg);
+		else
+			return false;
+		sleep (++retries);
+	} else {
+		error("Unable to allocate resources: %m");
+		return false;
+	}
+
+	return true;
+}
+
+/*
+ * SIGINT handler while waiting for resources to become available.
+ */
+static void
+_intr_handler(int signo)
+{
+	destroy_job = 1;
+}
 
 int
 allocate_test(void)
 {
 	int rc;
-	job_desc_msg_t *j = job_desc_msg_create_from_opts (NULL);
+	job_desc_msg_t *j = job_desc_msg_create_from_opts();
 	if(!j)
 		return SLURM_ERROR;
 	
@@ -101,23 +198,13 @@ allocate_test(void)
 resource_allocation_response_msg_t *
 allocate_nodes(void)
 {
-	int rc = 0;
-	static int sigarray[] = { SIGQUIT, SIGINT, SIGTERM, 0 };
-	SigFunc *oquitf, *ointf, *otermf;
-	sigset_t oset;
 	resource_allocation_response_msg_t *resp = NULL;
-	job_desc_msg_t *j = job_desc_msg_create_from_opts (NULL);
+	job_desc_msg_t *j = job_desc_msg_create_from_opts();
+	slurm_allocation_callbacks_t callbacks;
 
 	if(!j)
 		return NULL;
 	
-	oquitf = xsignal(SIGQUIT, _intr_handler);
-	ointf  = xsignal(SIGINT,  _intr_handler);
-	otermf = xsignal(SIGTERM, _intr_handler);
-
-	xsignal_save_mask(&oset);
-	xsignal_unblock(sigarray);
-
 	/* Do not re-use existing job id when submitting new job
 	 * from within a running job */
 	if ((j->job_id != NO_VAL) && !opt.jobid_set) {
@@ -127,33 +214,60 @@ allocate_nodes(void)
 		if (!opt.jobid_set)	/* Let slurmctld set jobid */
 			j->job_id = NO_VAL;
 	}
-	
-	while ((rc = slurm_allocate_resources(j, &resp) < 0) && _retry()) {
-		if (destroy_job)
-			goto done;
-	} 
+	callbacks.ping = _ping_handler;
+	callbacks.timeout = _timeout_handler;
+	callbacks.job_complete = _job_complete_handler;
+	callbacks.user_msg = _user_msg_handler;
+	callbacks.node_fail = _node_fail_handler;
 
-	if(!resp)
-		goto done;
-	
-	if ((rc == 0) && (resp->node_list == NULL)) {
-		if (resp->error_code)
-			verbose("Warning: %s",
-				slurm_strerror(resp->error_code));
-		_wait_for_resources(&resp);
+	/* create message thread to handle pings and such from slurmctld */
+	msg_thr = slurm_allocation_msg_thr_create(&j->other_port, &callbacks);
+
+	xsignal(SIGHUP, _signal_while_allocating);
+	xsignal(SIGINT, _signal_while_allocating);
+	xsignal(SIGQUIT, _signal_while_allocating);
+	xsignal(SIGPIPE, _signal_while_allocating);
+	xsignal(SIGTERM, _signal_while_allocating);
+	xsignal(SIGUSR1, _signal_while_allocating);
+	xsignal(SIGUSR2, _signal_while_allocating);
+
+	while (!resp) {
+		resp = slurm_allocate_resources_blocking(j, 0,
+							 _set_pending_job_id);
+		if (destroy_job) {
+			/* cancelled by signal */
+			break;
+		} else if(!resp && !_retry()) {
+			break;		
+		}
 	}
-
-    done:
-	xsignal_set_mask(&oset);
-	xsignal(SIGINT,  ointf);
-	xsignal(SIGTERM, otermf);
-	xsignal(SIGQUIT, oquitf);
+	
+	xsignal(SIGHUP, _exit_on_signal);
+	xsignal(SIGINT, ignore_signal);
+	xsignal(SIGQUIT, ignore_signal);
+	xsignal(SIGPIPE, ignore_signal);
+	xsignal(SIGTERM, ignore_signal);
+	xsignal(SIGUSR1, ignore_signal);
+	xsignal(SIGUSR2, ignore_signal);
 
 	job_desc_msg_destroy(j);
 
 	return resp;
 }
 
+void
+ignore_signal(int signo)
+{
+	/* do nothing */
+}
+
+int 
+cleanup_allocation()
+{
+	slurm_allocation_msg_thr_destroy(msg_thr);
+	return SLURM_SUCCESS;
+}
+
 resource_allocation_response_msg_t *
 existing_allocation(void)
 {
@@ -181,222 +295,44 @@ existing_allocation(void)
         return resp;
 }
 
-static void
-_wait_for_resources(resource_allocation_response_msg_t **resp)
+/* Set up port to handle messages from slurmctld */
+slurm_fd
+slurmctld_msg_init(void)
 {
-	resource_allocation_response_msg_t *r = *resp;
-	int sleep_time = MIN_ALLOC_WAIT;
-	int job_id = r->job_id;
+	slurm_addr slurm_address;
+	uint16_t port;
+	static slurm_fd slurmctld_fd   = (slurm_fd) NULL;
 
-	if (!opt.quiet)
-		info ("job %u queued and waiting for resources", r->job_id);
+	if (slurmctld_fd)	/* May set early for queued job allocation */
+		return slurmctld_fd;
 
-	slurm_free_resource_allocation_response_msg(r);
+	slurmctld_fd = -1;
+	slurmctld_comm_addr.hostname = NULL;
+	slurmctld_comm_addr.port = 0;
 
-	/* Keep polling until the job is allocated resources */
-	while (_wait_for_alloc_rpc(sleep_time, resp) <= 0) {
+	if ((slurmctld_fd = slurm_init_msg_engine_port(0)) < 0)
+		fatal("slurm_init_msg_engine_port error %m");
+	if (slurm_get_stream_addr(slurmctld_fd, &slurm_address) < 0)
+		fatal("slurm_get_stream_addr error %m");
+	fd_set_nonblocking(slurmctld_fd);
+	/* hostname is not set,  so slurm_get_addr fails
+	   slurm_get_addr(&slurm_address, &port, hostname, sizeof(hostname)); */
+	port = ntohs(slurm_address.sin_port);
+	slurmctld_comm_addr.hostname = xstrdup(opt.ctrl_comm_ifhn);
+	slurmctld_comm_addr.port     = port;
+	debug2("slurmctld messages to host=%s,port=%u", 
+	       slurmctld_comm_addr.hostname, 
+	       slurmctld_comm_addr.port);
 
-		if (slurm_allocation_lookup_lite(job_id, resp) >= 0)
-			break;
-
-		if (slurm_get_errno() == ESLURM_JOB_PENDING) 
-			debug3 ("Still waiting for allocation");
-		else 
-			fatal ("Unable to confirm allocation for job %u: %m", 
-			       job_id);
-
-		if (destroy_job) {
-			verbose("cancelling job %u", job_id);
-			slurm_complete_job(job_id, 0);
-			debugger_launch_failure(allocate_job);
-			exit(0);
-		}
-
-		if (sleep_time < MAX_ALLOC_WAIT)
-			sleep_time++;
-	}
-	if (!opt.quiet)
-		info ("job %u has been allocated resources", (*resp)->job_id);
+	return slurmctld_fd;
 }
 
-/* Wait up to sleep_time for RPC from slurmctld indicating resource allocation
- * has occured.
- * IN sleep_time: delay in seconds
- * OUT resp: resource allocation response message
- * RET 1 if resp is filled in, 0 otherwise */
-static int
-_wait_for_alloc_rpc(int sleep_time, resource_allocation_response_msg_t **resp)
-{
-	struct pollfd fds[1];
-	slurm_fd slurmctld_fd;
-
-	if ((slurmctld_fd = slurmctld_msg_init()) < 0) {
-		sleep (sleep_time);
-		return (0);
-	}
-
-	fds[0].fd = slurmctld_fd;
-	fds[0].events = POLLIN;
-
-	while (poll (fds, 1, (sleep_time * 1000)) < 0) {
-		switch (errno) {
-			case EAGAIN:
-			case EINTR:
-				return (-1);
-			case ENOMEM:
-			case EINVAL:
-			case EFAULT:
-				fatal("poll: %m");
-			default:
-				error("poll: %m. Continuing...");
-		}
-	}
-
-	if (fds[0].revents & POLLIN)
-		return (_accept_msg_connection(slurmctld_fd, resp));
-
-	return (0);
-}
-
-/* Accept RPC from slurmctld and process it.
- * IN slurmctld_fd: file descriptor for slurmctld communications
- * OUT resp: resource allocation response message
- * RET 1 if resp is filled in, 0 otherwise */
-static int 
-_accept_msg_connection(slurm_fd slurmctld_fd, 
-		resource_allocation_response_msg_t **resp)
-{
-	slurm_fd     fd;
-	slurm_msg_t *msg = NULL;
-	slurm_addr   cli_addr;
-	char         host[256];
-	uint16_t     port;
-	int          rc = 0;
-
-	fd = slurm_accept_msg_conn(slurmctld_fd, &cli_addr);
-	if (fd < 0) {
-		error("Unable to accept connection: %m");
-		return rc;
-	}
-
-	slurm_get_addr(&cli_addr, &port, host, sizeof(host));
-	debug2("got message connection from %s:%hu", host, port);
-
-	msg = xmalloc(sizeof(slurm_msg_t));
-	slurm_msg_t_init(msg);
-	
-  again:
-	if(slurm_receive_msg(fd, msg, 0) != 0) {
-		if (errno == EINTR) {
-			goto again;
-		}			
-		error("_accept_msg_connection[%s]: %m", host);
-		rc = SLURM_ERROR;
-		goto cleanup;
-		
-	}
-		
-	rc = _handle_msg(msg, resp); /* handle_msg frees msg->data */
-cleanup:
-	slurm_free_msg(msg);
-		
-	slurm_close_accepted_conn(fd);
-	return rc;
-}
-
-/* process RPC from slurmctld
- * IN msg: message recieved
- * OUT resp: resource allocation response message
- * RET 1 if resp is filled in, 0 otherwise */
-static int
-_handle_msg(slurm_msg_t *msg, resource_allocation_response_msg_t **resp)
-{
-	uid_t req_uid   = g_slurm_auth_get_uid(msg->auth_cred);
-	uid_t uid       = getuid();
-	uid_t slurm_uid = (uid_t) slurm_get_slurm_user_id();
-	int rc = 0;
-	srun_timeout_msg_t *to;
-	srun_user_msg_t *um;
-
-	if ((req_uid != slurm_uid) && (req_uid != 0) && (req_uid != uid)) {
-		error ("Security violation, slurm message from uid %u",
-			(unsigned int) req_uid);
-		return 0;
-	}
-
-	switch (msg->msg_type) {
-		case SRUN_PING:
-			debug3("slurmctld ping received");
-			slurm_send_rc_msg(msg, SLURM_SUCCESS);
-			slurm_free_srun_ping_msg(msg->data);
-			break;
-		case SRUN_JOB_COMPLETE:
-			debug3("job complete received");
-			/* FIXME: do something here */
-			slurm_free_srun_job_complete_msg(msg->data);	
-			break;
-		case RESPONSE_RESOURCE_ALLOCATION:
-			debug2("resource allocation response received");
-			slurm_send_rc_msg(msg, SLURM_SUCCESS);
-			*resp = msg->data;
-			rc = 1;
-			break;
-		case SRUN_TIMEOUT:
-			debug2("timeout received");
-			to = msg->data;
-			timeout_handler(to->timeout);
-			slurm_free_srun_timeout_msg(msg->data);
-			break;
-		case SRUN_USER_MSG:
-			um = msg->data;
-			info("%s", um->msg);
-			slurm_free_srun_user_msg(msg->data);
-			break;
-		default:
-			error("received spurious message type: %d\n",
-				 msg->msg_type);
-	}
-	return rc;
-}
-
-static bool
-_retry()
-{
-	static int  retries = 0;
-	static char *msg = "Slurm job queue full, sleeping and retrying.";
-
-	if (errno == ESLURM_ERROR_ON_DESC_TO_RECORD_COPY) {
-		if (retries == 0)
-			error (msg);
-		else if (retries < MAX_RETRIES)
-			debug (msg);
-		else
-			return false;
-		sleep (++retries);
-	} else {
-		error("Unable to allocate resources: %m");
-		return false;
-	}
-
-	return true;
-}
-
-/*
- * SIGINT handler while waiting for resources to become available.
- */
-static void
-_intr_handler(int signo)
-{
-	destroy_job = 1;
-}
-
-
 /*
  * Create job description structure based off srun options
  * (see opt.h)
  */
 job_desc_msg_t *
-job_desc_msg_create_from_opts (char *script)
+job_desc_msg_create_from_opts ()
 {
 	job_desc_msg_t *j = xmalloc(sizeof(*j));
 	char buf[8192];
@@ -461,6 +397,8 @@ job_desc_msg_create_from_opts (char *script)
 		j->mail_user = xstrdup(opt.mail_user);
 	if (opt.begin)
 		j->begin_time = opt.begin;
+	if (opt.licenses)
+		j->licenses = xstrdup(opt.licenses);
 	if (opt.network)
 		j->network = xstrdup(opt.network);
 	if (opt.account)
@@ -515,9 +453,9 @@ job_desc_msg_create_from_opts (char *script)
 	if (opt.job_min_threads != NO_VAL)
 		j->job_min_threads  = opt.job_min_threads;
 	if (opt.job_min_memory != NO_VAL)
-		j->job_min_memory   = opt.job_min_memory;
-	if (opt.job_max_memory != NO_VAL)
-		j->job_max_memory   = opt.job_max_memory;
+		j->job_min_memory = opt.job_min_memory;
+	else if (opt.mem_per_cpu != NO_VAL)
+		j->job_min_memory = opt.mem_per_cpu | MEM_PER_CPU;
 	if (opt.job_min_tmp_disk != NO_VAL)
 		j->job_min_tmp_disk = opt.job_min_tmp_disk;
 	if (opt.overcommit) {
@@ -541,45 +479,6 @@ job_desc_msg_create_from_opts (char *script)
 	 * message as all other messages */
 	j->alloc_resp_port = slurmctld_comm_addr.port;
 	j->other_port = slurmctld_comm_addr.port;
-	if (slurmctld_comm_addr.hostname) {
-		j->alloc_resp_hostname = xstrdup(slurmctld_comm_addr.hostname);
-		j->other_hostname = xstrdup(slurmctld_comm_addr.hostname);
-	} else {
-		j->alloc_resp_hostname = NULL;
-		j->other_hostname = NULL;
-	}
-
-	if (script) {
-		/*
-		 * If script is set then we are building a request for
-		 *  a batch job
-		 */
-		xassert (opt.batch);
-
-		j->environment = NULL;
-		if (opt.get_user_env_time >= 0) {
-			struct passwd *pw = NULL;
-			pw = getpwuid(opt.uid);
-			if (pw != NULL) {
-				j->environment =
-					env_array_user_default(pw->pw_name,
-							opt.get_user_env_time,
-							opt.get_user_env_mode);
-				/* FIXME - should we abort if j->environment
-				   is NULL? */
-			}
-		}
-		env_array_merge(&j->environment, (const char **)environ);
-		j->env_size = envcount (j->environment);
-		j->script = script;
-		j->argv = remote_argv;
-		j->argc = remote_argc;
-		j->err  = opt.efname;
-		j->in   = opt.ifname;
-		j->out  = opt.ofname;
-		j->work_dir = opt.cwd;
-		j->no_requeue = opt.no_requeue;
-	}
 
 	return (j);
 }
@@ -590,136 +489,126 @@ job_desc_msg_destroy(job_desc_msg_t *j)
 	if (j) {
 		xfree(j->account);
 		xfree(j->comment);
-		xfree(j->alloc_resp_hostname);
-		xfree(j->other_hostname);
 		xfree(j);
 	}
 }
 
-static job_step_create_request_msg_t *
-_step_req_create(srun_job_t *j)
-{
-	job_step_create_request_msg_t *r = xmalloc(sizeof(*r));
-	r->job_id     = j->jobid;
-	r->user_id    = opt.uid;
-
-	r->node_count = j->nhosts;
-	/* info("send %d or %d? sending %d", opt.max_nodes, */
-/* 		     j->nhosts, r->node_count); */
-	if(r->node_count > j->nhosts) {
-		error("Asking for more nodes that allocated");
-		return NULL;
-	}
-	r->cpu_count  = opt.overcommit ? r->node_count
-		                       : (opt.nprocs*opt.cpus_per_task);
-	r->num_tasks  = opt.nprocs;
-	r->node_list  = xstrdup(opt.nodelist);
-	r->network    = xstrdup(opt.network);
-	r->name       = xstrdup(opt.job_name);
-	r->relative   = (uint16_t)opt.relative;
-	r->overcommit = opt.overcommit ? 1 : 0;
-	debug("requesting job %d, user %d, nodes %d including (%s)", 
-	      r->job_id, r->user_id, r->node_count, r->node_list);
-	debug("cpus %d, tasks %d, name %s, relative %d", 
-	      r->cpu_count, r->num_tasks, r->name, r->relative);
-	
-	switch (opt.distribution) {
-	case SLURM_DIST_BLOCK:
-		r->task_dist = SLURM_DIST_BLOCK;
-		break;
-	case SLURM_DIST_ARBITRARY:
-		r->task_dist = SLURM_DIST_ARBITRARY;
-		break;
-	case SLURM_DIST_CYCLIC:
-		r->task_dist = SLURM_DIST_CYCLIC;
-		break;
-	case SLURM_DIST_CYCLIC_CYCLIC:
-		r->task_dist = SLURM_DIST_CYCLIC_CYCLIC;
-		break;
-	case SLURM_DIST_CYCLIC_BLOCK:
-		r->task_dist = SLURM_DIST_CYCLIC_BLOCK;
-		break;
-	case SLURM_DIST_BLOCK_CYCLIC:
-		r->task_dist = SLURM_DIST_BLOCK_CYCLIC;
-		break;
-	case SLURM_DIST_BLOCK_BLOCK:
-		r->task_dist = SLURM_DIST_BLOCK_BLOCK;
-		break;
-	case SLURM_DIST_PLANE:
-		r->task_dist = SLURM_DIST_PLANE;
-		r->plane_size = opt.plane_size;
-		break;
-	default:
-		r->task_dist = (r->num_tasks <= r->node_count) 
-			? SLURM_DIST_CYCLIC : SLURM_DIST_BLOCK;
-		break;
-
-	}
-	opt.distribution = r->task_dist;
-	
-	if (slurmctld_comm_addr.port) {
-		r->host = xstrdup(slurmctld_comm_addr.hostname);
-		r->port = slurmctld_comm_addr.port;
-	}
-
-	return(r);
-}
-
 int
 create_job_step(srun_job_t *job)
 {
-	job_step_create_request_msg_t  *req  = NULL;
-	job_step_create_response_msg_t *resp = NULL;
-	int i;
-	
-	if (!(req = _step_req_create(job))) {
-		error ("Unable to allocate step request message");
-		return -1;
-	}
+	int i, rc;
+	SigFunc *oquitf = NULL, *ointf = NULL, *otermf = NULL;
 
-	for (i=0; ;i++) {
-		if ((slurm_job_step_create(req, &resp) == SLURM_SUCCESS)
-		&&  (resp != NULL))
+	slurm_step_ctx_params_t_init(&job->ctx_params);
+	job->ctx_params.job_id = job->jobid;
+	job->ctx_params.uid = opt.uid;
+
+	/* set the jobid for totalview */
+	totalview_jobid = NULL;
+	xstrfmtcat(totalview_jobid, "%u", job->ctx_params.job_id);
+
+	job->ctx_params.node_count = job->nhosts;
+	if (!opt.nprocs_set && (opt.ntasks_per_node != NO_VAL))
+		 opt.nprocs = job->nhosts * opt.ntasks_per_node;
+	job->ctx_params.task_count = opt.nprocs;
+	
+	job->ctx_params.cpu_count = opt.overcommit ? job->ctx_params.node_count
+		: (opt.nprocs*opt.cpus_per_task);
+	
+	job->ctx_params.relative = (uint16_t)opt.relative;
+	job->ctx_params.ckpt_interval = (uint16_t)opt.ckpt_interval;
+	job->ctx_params.ckpt_path = opt.ckpt_path;
+	job->ctx_params.exclusive = (uint16_t)opt.exclusive;
+	job->ctx_params.immediate = (uint16_t)opt.immediate;
+	job->ctx_params.verbose_level = (uint16_t)_verbose;
+	switch (opt.distribution) {
+	case SLURM_DIST_BLOCK:
+	case SLURM_DIST_ARBITRARY:
+	case SLURM_DIST_CYCLIC:
+	case SLURM_DIST_CYCLIC_CYCLIC:
+	case SLURM_DIST_CYCLIC_BLOCK:
+	case SLURM_DIST_BLOCK_CYCLIC:
+	case SLURM_DIST_BLOCK_BLOCK:
+		job->ctx_params.task_dist = opt.distribution;
+		break;
+	case SLURM_DIST_PLANE:
+		job->ctx_params.task_dist = SLURM_DIST_PLANE;
+		job->ctx_params.plane_size = opt.plane_size;
+		break;
+	default:
+		job->ctx_params.task_dist = (job->ctx_params.task_count <= 
+			job->ctx_params.node_count) 
+			? SLURM_DIST_CYCLIC : SLURM_DIST_BLOCK;
+		break;
+
+	}
+	job->ctx_params.overcommit = opt.overcommit ? 1 : 0;
+
+	job->ctx_params.node_list = opt.nodelist;
+	
+	job->ctx_params.network = opt.network;
+	job->ctx_params.name = opt.job_name;
+	
+	debug("requesting job %u, user %u, nodes %u including (%s)", 
+	      job->ctx_params.job_id, job->ctx_params.uid,
+	      job->ctx_params.node_count, job->ctx_params.node_list);
+	debug("cpus %u, tasks %u, name %s, relative %u", 
+	      job->ctx_params.cpu_count, job->ctx_params.task_count,
+	      job->ctx_params.name, job->ctx_params.relative);
+
+	for (i=0; (!destroy_job); i++) {
+		if(opt.no_alloc) {
+			job->step_ctx = slurm_step_ctx_create_no_alloc(
+				&job->ctx_params, job->stepid);
+		} else
+			job->step_ctx = slurm_step_ctx_create(
+				&job->ctx_params);
+		if (job->step_ctx != NULL) {
+			if (i > 0)
+				info("Job step created");
+			
 			break;
-		if (slurm_get_errno() != ESLURM_DISABLED) {
+		}
+		rc = slurm_get_errno();
+
+		if (opt.immediate ||
+		    ((rc != ESLURM_NODES_BUSY) && (rc != ESLURM_DISABLED))) {
 			error ("Unable to create job step: %m");
 			return -1;
 		}
-		if (i == 0)
-			info("Job step creation temporarily disabled, retrying");
+		
+		if (i == 0) {
+			info("Job step creation temporarily disabled, retrying");	
+			ointf  = xsignal(SIGINT,  _intr_handler);
+			otermf  = xsignal(SIGTERM, _intr_handler);
+			oquitf  = xsignal(SIGQUIT, _intr_handler);
+		} else
+			info("Job step creation still disabled, retrying");
 		sleep(MIN((i*10), 60));
 	}
-	
-	job->stepid  = resp->job_step_id;
-	job->step_layout = resp->step_layout;
-	job->cred    = resp->cred;
-	job->switch_job = resp->switch_job;
-		
+	if (i > 0) {
+		xsignal(SIGINT,  ointf);
+		xsignal(SIGQUIT, oquitf);
+		xsignal(SIGTERM, otermf);
+		if (destroy_job) {
+			info("Cancelled pending job step");
+			return -1;
+		}
+	}
 
+	slurm_step_ctx_get(job->step_ctx, SLURM_STEP_CTX_STEPID, &job->stepid);
 	/*  Number of hosts in job may not have been initialized yet if 
 	 *    --jobid was used or only SLURM_JOBID was set in user env.
 	 *    Reset the value here just in case.
 	 */
-	job->nhosts = job->step_layout->node_cnt;
-
-	if(!job->step_layout) {
-		error("step_layout not returned");
-		return -1;
-	}
+	slurm_step_ctx_get(job->step_ctx, SLURM_STEP_CTX_NUM_HOSTS,
+			   &job->nhosts);
 	
 	/*
 	 * Recreate filenames which may depend upon step id
 	 */
 	job_update_io_fnames(job);
 
-	slurm_free_job_step_create_request_msg(req);
-	
 	return 0;
 }
 
-void 
-set_allocate_job(srun_job_t *job) 
-{
-	allocate_job = job;
-	return;
-}
