@@ -6,7 +6,7 @@
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
  * 
- *  UCRL-CODE-226842.
+ *  LLNL-CODE-402394.
  *  
  *  This file is part of SLURM, a resource management program.
  *  For details, see <http://www.llnl.gov/linux/slurm/>.
@@ -1049,7 +1049,7 @@ display_it:
 				continue;
 			break;
 		default:
-			g_print("Unkown type %d\n", spec_info->type);
+			g_print("Unknown type %d\n", spec_info->type);
 			continue;
 		}
 		list_push(send_block_list, block_ptr);
