@@ -8,9 +8,12 @@ file_orig=`${slurm_bin}scontrol show config | awk '{ if ( $1 ~ /SLURM_CONFIG_FIL
 grep -iv AuthType <$file_orig >tmp.$$
 echo "AuthType=auth/dummy" >>tmp.$$
 
+echo "#!/bin/sh"     >tmp2.$$
+echo "id"           >>tmp2.$$
+
 # Run srun using this config file
 export SLURM_CONF=tmp.$$ 
-${slurm_bin}srun /usr/bin/id
+${slurm_bin}sbatch tmp2.$$
 
 # Clean up
-rm tmp.$$
+rm -f tmp.$$ tmp2.$$
